@@ -31,6 +31,7 @@ except:
 import gdb
 import os
 import os.path
+import re
 import sys
 import struct
 import tempfile
@@ -78,16 +79,16 @@ def registerCommand(name, func):
 #
 #######################################################################
 
-# Just convienience for 'python print ...'
+# For CLI dumper use, see README.txt
 class PPCommand(gdb.Command):
     def __init__(self):
         super(PPCommand, self).__init__('pp', gdb.COMMAND_OBSCURE)
     def invoke(self, args, from_tty):
-        print(eval(args))
+        print(theCliDumper.fetchVariable(args))
 
 PPCommand()
 
-# Just convienience for 'python print gdb.parse_and_eval(...)'
+# Just convenience for 'python print gdb.parse_and_eval(...)'
 class PPPCommand(gdb.Command):
     def __init__(self):
         super(PPPCommand, self).__init__('ppp', gdb.COMMAND_OBSCURE)
@@ -132,25 +133,34 @@ class PlainDumper:
 
     def __call__(self, d, value):
         try:
-            printer = self.printer.gen_printer(value)
+            printer = self.printer.gen_printer(value.nativeValue)
         except:
-            printer = self.printer.invoke(value)
-        lister = getattr(printer, 'children', None)
-        children = [] if lister is None else list(lister())
-        d.putType(self.printer.name)
+            printer = self.printer.invoke(value.nativeValue)
+        d.putType(value.nativeValue.type.name)
         val = printer.to_string()
         if isinstance(val, str):
-            d.putValue(val)
+            # encode and avoid extra quotes ('"') at beginning and end
+            d.putValue(d.hexencode(val), 'utf8:1:0')
         elif sys.version_info[0] <= 2 and isinstance(val, unicode):
             d.putValue(val)
-        else: # Assuming LazyString
-            d.putCharArrayHelper(val.address, val.length, val.type)
+        elif val is not None: # Assuming LazyString
+            d.putCharArrayValue(val.address, val.length,
+                                val.type.target().sizeof)
 
-        d.putNumChild(len(children))
-        if d.isExpanded():
-            with Children(d):
-                for child in children:
-                    d.putSubItem(child[0], child[1])
+        lister = getattr(printer, 'children', None)
+        if lister is None:
+            d.putNumChild(0)
+        else:
+            if d.isExpanded():
+                children = lister()
+                with Children(d):
+                    i = 0
+                    for (name, child) in children:
+                        d.putSubItem(name, d.fromNativeValue(child))
+                        i += 1
+                        if i > 1000:
+                            break
+            d.putNumChild(1)
 
 def importPlainDumpers(args):
     if args == 'off':
@@ -166,7 +176,7 @@ registerCommand('importPlainDumpers', importPlainDumpers)
 
 
 
-class OutputSafer:
+class OutputSaver:
     def __init__(self, d):
         self.d = d
 
@@ -176,7 +186,7 @@ class OutputSafer:
 
     def __exit__(self, exType, exValue, exTraceBack):
         if self.d.passExceptions and not exType is None:
-            showException('OUTPUTSAFER', exType, exValue, exTraceBack)
+            showException('OUTPUTSAVER', exType, exValue, exTraceBack)
             self.d.output = self.savedOutput
         else:
             self.savedOutput += self.d.output
@@ -244,17 +254,19 @@ class Dumper(DumperBase):
             #warn('TARGET TYPE: %s' % targetType)
             if targetType.code == gdb.TYPE_CODE_ARRAY:
                 val = self.Value(self)
-                val.laddress = toInteger(nativeValue.address)
-                val.nativeValue = nativeValue
             else:
-                # Cast may fail (e.g for arrays, see test for Bug5799)
-                val = self.fromNativeValue(nativeValue.cast(targetType))
-            val.type = self.fromNativeType(nativeType)
-            val.nativeValue = nativeValue
+                try:
+                    # Cast may fail for arrays, for typedefs to __uint128_t with
+                    # gdb.error: That operation is not available on integers
+                    # of more than 8 bytes.
+                    # See test for Bug5799, QTCREATORBUG-18450.
+                    val = self.fromNativeValue(nativeValue.cast(targetType))
+                except:
+                    val = self.Value(self)
             #warn('CREATED TYPEDEF: %s' % val)
-            return val
+        else:
+            val = self.Value(self)
 
-        val = self.Value(self)
         val.nativeValue = nativeValue
         if not nativeValue.address is None:
             val.laddress = toInteger(nativeValue.address)
@@ -305,6 +317,12 @@ class Dumper(DumperBase):
             targetType = self.fromNativeType(nativeType.target().unqualified())
             return self.createReferenceType(targetType)
 
+        if hasattr(gdb, "TYPE_CODE_RVALUE_REF"):
+            if code == gdb.TYPE_CODE_RVALUE_REF:
+                #warn('RVALUEREF')
+                targetType = self.fromNativeType(nativeType.target())
+                return self.createRValueReferenceType(targetType)
+
         if code == gdb.TYPE_CODE_ARRAY:
             #warn('ARRAY')
             nativeTargetType = nativeType.target().unqualified()
@@ -353,10 +371,8 @@ class Dumper(DumperBase):
                 gdb.TYPE_CODE_STRING : TypeCodeFortranString,
             }[code]
             if tdata.code == TypeCodeEnum:
-                tdata.enumDisplay = lambda intval, addr : \
-                    self.nativeTypeEnumDisplay(nativeType, intval, 0)
-                tdata.enumHexDisplay = lambda intval, addr : \
-                    self.nativeTypeEnumDisplay(nativeType, intval, 1)
+                tdata.enumDisplay = lambda intval, addr, form : \
+                    self.nativeTypeEnumDisplay(nativeType, intval, form)
             if tdata.code == TypeCodeStruct:
                 tdata.lalignment = lambda : \
                     self.nativeStructAlignment(nativeType)
@@ -387,17 +403,13 @@ class Dumper(DumperBase):
         targs2 = self.listTemplateParametersManually(str(nativeType))
         return targs if len(targs) >= len(targs2) else targs2
 
-    def nativeTypeEnumDisplay(self, nativeType, intval, useHex):
-        if useHex:
-            format = lambda text, intval: '%s (0x%04x)' % (text, intval)
-        else:
-            format = lambda text, intval: '%s (%d)' % (text, intval)
+    def nativeTypeEnumDisplay(self, nativeType, intval, form):
         try:
             enumerators = []
             for field in nativeType.fields():
                 # If we found an exact match, return it immediately
                 if field.enumval == intval:
-                    return format(field.name, intval)
+                    return field.name + ' (' + (form % intval) + ')'
                 enumerators.append((field.name, field.enumval))
 
             # No match was found, try to return as flags
@@ -412,13 +424,11 @@ class Dumper(DumperBase):
                     found = True
             if not found or v != 0:
                 # Leftover value
-                flags.append('unknown:%d' % v)
-            return format(" | ".join(flags), intval)
+                flags.append('unknown: %d' % v)
+            return '(' + " | ".join(flags) + ') (' + (form % intval) + ')'
         except:
             pass
-        if useHex:
-            return '0x%04x' % intval;
-        return '%d' % intval
+        return form % intval
 
     def nativeTypeId(self, nativeType):
         if nativeType and (nativeType.code == gdb.TYPE_CODE_TYPEDEF):
@@ -714,14 +724,18 @@ class Dumper(DumperBase):
         self.reportResult(self.output)
 
     def parseAndEvaluate(self, exp):
+        val = self.nativeParseAndEvaluate(exp)
+        return None if val is None else self.fromNativeValue(val)
+
+    def nativeParseAndEvaluate(self, exp):
         #warn('EVALUATE "%s"' % exp)
         try:
             val = gdb.parse_and_eval(exp)
+            return val
         except RuntimeError as error:
             if self.passExceptions:
                 warn("Cannot evaluate '%s': %s" % (exp, error))
             return None
-        return self.fromNativeValue(val)
 
     def callHelper(self, rettype, value, function, args):
         # args is a tuple.
@@ -885,16 +899,33 @@ class Dumper(DumperBase):
     def createSpecialBreakpoints(self, args):
         self.specialBreakpoints = []
         def newSpecial(spec):
-            class SpecialBreakpoint(gdb.Breakpoint):
+            # GDB < 8.1 does not have the 'qualified' parameter here,
+            # GDB >= 8.1 applies some generous pattern matching, hitting
+            # e.g. also Foo::abort() when asking for '::abort'
+            class Pre81SpecialBreakpoint(gdb.Breakpoint):
                 def __init__(self, spec):
-                    super(SpecialBreakpoint, self).\
-                        __init__(spec, gdb.BP_BREAKPOINT, internal=True)
+                    super(Pre81SpecialBreakpoint, self).__init__(spec,
+                        gdb.BP_BREAKPOINT, internal=True)
                     self.spec = spec
 
                 def stop(self):
                     print("Breakpoint on '%s' hit." % self.spec)
                     return True
-            return SpecialBreakpoint(spec)
+
+            class SpecialBreakpoint(gdb.Breakpoint):
+                def __init__(self, spec):
+                    super(SpecialBreakpoint, self).__init__(spec,
+                        gdb.BP_BREAKPOINT, internal=True, qualified=True)
+                    self.spec = spec
+
+                def stop(self):
+                    print("Breakpoint on '%s' hit." % self.spec)
+                    return True
+
+            try:
+                return SpecialBreakpoint(spec)
+            except:
+                return Pre81SpecialBreakpoint(spec)
 
         # FIXME: ns is accessed too early. gdb.Breakpoint() has no
         # 'rbreak' replacement, and breakpoints created with
@@ -995,17 +1026,26 @@ class Dumper(DumperBase):
     def handleNewObjectFile(self, objfile):
         name = objfile.filename
         if self.isWindowsTarget():
-            isQtCoreObjFile = name.find('Qt5Cored.dll') >= 0 or name.find('Qt5Core.dll') >= 0
-            if not isQtCoreObjFile:
-                isQtCoreObjFile = name.find('QtCored.dll') >= 0 or name.find('QtCore.dll') >= 0
+            qtCoreMatch = re.match('.*Qt5?Core[^/.]*d?\.dll', name)
         else:
-            isQtCoreObjFile = name.find('/libQt5Core') >= 0
-            if not isQtCoreObjFile:
-                isQtCoreObjFile = name.find('/libQtCore') >= 0
+            qtCoreMatch = re.match('.*/libQt5?Core[^/.]*\.so', name)
 
-        if isQtCoreObjFile:
+        if qtCoreMatch is not None:
+            self.addDebugLibs(objfile)
             self.handleQtCoreLoaded(objfile)
 
+    def addDebugLibs(self, objfile):
+        # The directory where separate debug symbols are searched for
+        # is "/usr/lib/debug".
+        try:
+            cooked = gdb.execute('show debug-file-directory', to_string=True)
+            clean = cooked.split('"')[1]
+            newdir = '/'.join(objfile.filename.split('/')[:-1])
+            gdb.execute('set debug-file-directory %s:%s' % (clean, newdir))
+        except:
+            pass
+
+
     def handleQtCoreLoaded(self, objfile):
         fd, tmppath = tempfile.mkstemp()
         os.close(fd)
@@ -1013,7 +1053,12 @@ class Dumper(DumperBase):
         try:
             symbols = gdb.execute(cmd, to_string = True)
         except:
-            pass
+            # command syntax depends on gdb version - below is gdb 8+
+            cmd = 'maint print msymbols -objfile "%s" -- %s' % (objfile.filename, tmppath)
+            try:
+                symbols = gdb.execute(cmd, to_string = True)
+            except:
+                pass
         ns = ''
         with open(tmppath) as f:
             for line in f:
@@ -1258,7 +1303,7 @@ class Dumper(DumperBase):
             frame = gdb.newest_frame()
             ns = self.qtNamespace()
             needle = self.qtNamespace() + 'QV4::ExecutionEngine'
-            pat = '%sqt_v4StackTrace(((%sQV4::ExecutionEngine *)0x%x)->currentContext)'
+            pat = '%sqt_v4StackTraceForEngine((void*)0x%x)'
             done = False
             while i < limit and frame and not done:
                 block = None
@@ -1275,7 +1320,7 @@ class Dumper(DumperBase):
                                dereftype = typeobj.target().unqualified()
                                if dereftype.name == needle:
                                     addr = toInteger(value)
-                                    expr = pat % (ns, ns, addr)
+                                    expr = pat % (ns, addr)
                                     res = str(gdb.parse_and_eval(expr))
                                     pos = res.find('"stack=[')
                                     if pos != -1:
@@ -1291,7 +1336,7 @@ class Dumper(DumperBase):
         frame = gdb.newest_frame()
         self.currentCallContext = None
         while i < limit and frame:
-            with OutputSafer(self):
+            with OutputSaver(self):
                 name = frame.name()
                 functionName = '??' if name is None else name
                 fileName = ''
@@ -1341,7 +1386,7 @@ class Dumper(DumperBase):
 
             frame = frame.older()
             i += 1
-        self.reportResult('stack={frames=[' + self.output + '].report}')
+        self.reportResult('stack={frames=[' + self.output + ']}')
 
     def createResolvePendingBreakpointsHookBreakpoint(self, args):
         class Resolver(gdb.Breakpoint):
@@ -1387,7 +1432,7 @@ class CliDumper(Dumper):
         self.chidrenSuffix = '] '
         self.indent = 0
         self.isCli = True
-
+        self.setupDumpers({})
 
     def put(self, line):
         if self.output.endswith('\n'):
@@ -1400,26 +1445,25 @@ class CliDumper(Dumper):
     def putOriginalAddress(self, address):
         pass
 
-    def fetchVariables(self, args):
+    def fetchVariable(self, name):
+        args = {}
         args['fancy'] = 1
         args['passexception'] = 1
         args['autoderef'] = 1
         args['qobjectnames'] = 1
-        name = args['varlist']
+        args['varlist'] = name
         self.prepare(args)
         self.output = name + ' = '
-        frame = gdb.selected_frame()
-        value = frame.read_var(name)
+        value = self.parseAndEvaluate(name)
         with TopLevelItem(self, name):
             self.putItem(value)
         return self.output
 
-# Global instance.
-#if gdb.parameter('height') is None:
+
+# Global instances.
 theDumper = Dumper()
-#else:
-#    import codecs
-#    theDumper = CliDumper()
+theCliDumper = CliDumper()
+
 
 ######################################################################
 #
