@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -45,6 +41,7 @@
 #include "instancecontainer.h"
 #include "createinstancescommand.h"
 #include "createscenecommand.h"
+#include "change3dviewcommand.h"
 #include "changevaluescommand.h"
 #include "changebindingscommand.h"
 #include "changeauxiliarycommand.h"
@@ -71,28 +68,31 @@
 #include "endpuppetcommand.h"
 #include "debugoutputcommand.h"
 #include "puppetalivecommand.h"
+#include "changeselectioncommand.h"
+#include "drop3dlibraryitemcommand.h"
 
 namespace QmlDesigner {
 
 NodeInstanceClientProxy::NodeInstanceClientProxy(QObject *parent)
     : QObject(parent),
-      m_inputIoDevice(0),
-      m_outputIoDevice(0),
-      m_nodeInstanceServer(0),
+      m_inputIoDevice(nullptr),
+      m_outputIoDevice(nullptr),
+      m_nodeInstanceServer(nullptr),
       m_writeCommandCounter(0),
       m_synchronizeId(-1)
 {
-    connect(&m_puppetAliveTimer, SIGNAL(timeout()), this, SLOT(sendPuppetAliveCommand()));
-    m_puppetAliveTimer.setInterval(300);
+    connect(&m_puppetAliveTimer, &QTimer::timeout, this, &NodeInstanceClientProxy::sendPuppetAliveCommand);
+    m_puppetAliveTimer.setInterval(1000);
     m_puppetAliveTimer.start();
 }
 
 void NodeInstanceClientProxy::initializeSocket()
 {
     QLocalSocket *localSocket = new QLocalSocket(this);
-    connect(localSocket, SIGNAL(readyRead()), this, SLOT(readDataStream()));
-    connect(localSocket, SIGNAL(error(QLocalSocket::LocalSocketError)), QCoreApplication::instance(), SLOT(quit()));
-    connect(localSocket, SIGNAL(disconnected()), QCoreApplication::instance(), SLOT(quit()));
+    connect(localSocket, &QIODevice::readyRead, this, &NodeInstanceClientProxy::readDataStream);
+    connect(localSocket, QOverload<QLocalSocket::LocalSocketError>::of(&QLocalSocket::error),
+            QCoreApplication::instance(), &QCoreApplication::quit);
+    connect(localSocket, &QLocalSocket::disconnected, QCoreApplication::instance(), &QCoreApplication::quit);
     localSocket->connectToServer(QCoreApplication::arguments().at(1), QIODevice::ReadWrite | QIODevice::Unbuffered);
     localSocket->waitForConnected(-1);
 
@@ -132,6 +132,7 @@ bool compareCommands(const QVariant &command, const QVariant &controlCommand)
 {
     static const int informationChangedCommandType = QMetaType::type("InformationChangedCommand");
     static const int valuesChangedCommandType = QMetaType::type("ValuesChangedCommand");
+    static const int valuesModifiedCommandType = QMetaType::type("ValuesModifiedCommand");
     static const int pixmapChangedCommandType = QMetaType::type("PixmapChangedCommand");
     static const int childrenChangedCommandType = QMetaType::type("ChildrenChangedCommand");
     static const int statePreviewImageChangedCommandType = QMetaType::type("StatePreviewImageChangedCommand");
@@ -139,13 +140,17 @@ bool compareCommands(const QVariant &command, const QVariant &controlCommand)
     static const int synchronizeCommandType = QMetaType::type("SynchronizeCommand");
     static const int tokenCommandType = QMetaType::type("TokenCommand");
     static const int debugOutputCommandType = QMetaType::type("DebugOutputCommand");
+    static const int changeSelectionCommandType = QMetaType::type("ChangeSelectionCommand");
+    static const int drop3DLibraryItemCommandType = QMetaType::type("Drop3DLibraryItemCommand");
 
     if (command.userType() == controlCommand.userType()) {
         if (command.userType() == informationChangedCommandType)
             return command.value<InformationChangedCommand>() == controlCommand.value<InformationChangedCommand>();
         else if (command.userType() == valuesChangedCommandType)
             return command.value<ValuesChangedCommand>() == controlCommand.value<ValuesChangedCommand>();
-         else if (command.userType() == pixmapChangedCommandType)
+        else if (command.userType() == valuesModifiedCommandType)
+            return command.value<ValuesModifiedCommand>() == controlCommand.value<ValuesModifiedCommand>();
+        else if (command.userType() == pixmapChangedCommandType)
             return command.value<PixmapChangedCommand>() == controlCommand.value<PixmapChangedCommand>();
         else if (command.userType() == childrenChangedCommandType)
             return command.value<ChildrenChangedCommand>() == controlCommand.value<ChildrenChangedCommand>();
@@ -159,6 +164,10 @@ bool compareCommands(const QVariant &command, const QVariant &controlCommand)
             return command.value<TokenCommand>() == controlCommand.value<TokenCommand>();
         else if (command.userType() == debugOutputCommandType)
             return command.value<DebugOutputCommand>() == controlCommand.value<DebugOutputCommand>();
+        else if (command.userType() == changeSelectionCommandType)
+            return command.value<ChangeSelectionCommand>() == controlCommand.value<ChangeSelectionCommand>();
+        else if (command.userType() == drop3DLibraryItemCommandType)
+            return command.value<Drop3DLibraryItemCommand>() == controlCommand.value<Drop3DLibraryItemCommand>();
     }
 
     return false;
@@ -201,6 +210,11 @@ void NodeInstanceClientProxy::valuesChanged(const ValuesChangedCommand &command)
     writeCommand(QVariant::fromValue(command));
 }
 
+void NodeInstanceClientProxy::valuesModified(const ValuesModifiedCommand &command)
+{
+    writeCommand(QVariant::fromValue(command));
+}
+
 void NodeInstanceClientProxy::pixmapChanged(const PixmapChangedCommand &command)
 {
     writeCommand(QVariant::fromValue(command));
@@ -236,6 +250,16 @@ void NodeInstanceClientProxy::puppetAlive(const PuppetAliveCommand &command)
     writeCommand(QVariant::fromValue(command));
 }
 
+void NodeInstanceClientProxy::selectionChanged(const ChangeSelectionCommand &command)
+{
+     writeCommand(QVariant::fromValue(command));
+}
+
+void NodeInstanceClientProxy::library3DItemDropped(const Drop3DLibraryItemCommand &command)
+{
+    writeCommand(QVariant::fromValue(command));
+}
+
 void NodeInstanceClientProxy::flush()
 {
 }
@@ -255,9 +279,6 @@ qint64 NodeInstanceClientProxy::bytesToWrite() const
 
 QVariant NodeInstanceClientProxy::readCommandFromIOStream(QIODevice *ioDevice, quint32 *readCommandCounter, quint32 *blockSize)
 {
-
-
-
     QDataStream in(ioDevice);
     in.setVersion(QDataStream::Qt_4_8);
 
@@ -280,7 +301,7 @@ QVariant NodeInstanceClientProxy::readCommandFromIOStream(QIODevice *ioDevice, q
     *blockSize = 0;
 
     if (in.status() != QDataStream::Ok) {
-        qWarning() << "Stream is no ok!!!";
+        qWarning() << "Stream is not OK";
         exit(1);
     }
 
@@ -306,9 +327,8 @@ void NodeInstanceClientProxy::readDataStream()
             break;
     }
 
-    foreach (const QVariant &command, commandList) {
+    for (const QVariant &command : qAsConst(commandList))
         dispatchCommand(command);
-    }
 }
 
 void NodeInstanceClientProxy::sendPuppetAliveCommand()
@@ -341,6 +361,11 @@ void NodeInstanceClientProxy::createScene(const CreateSceneCommand &command)
     nodeInstanceServer()->createScene(command);
 }
 
+void NodeInstanceClientProxy::change3DView(const Change3DViewCommand &command)
+{
+    nodeInstanceServer()->change3DView(command);
+}
+
 void NodeInstanceClientProxy::clearScene(const ClearSceneCommand &command)
 {
     nodeInstanceServer()->clearScene(command);
@@ -420,9 +445,15 @@ void NodeInstanceClientProxy::redirectToken(const EndPuppetCommand & /*command*/
     QCoreApplication::exit();
 }
 
+void NodeInstanceClientProxy::changeSelection(const ChangeSelectionCommand &command)
+{
+    nodeInstanceServer()->changeSelection(command);
+}
+
 void NodeInstanceClientProxy::dispatchCommand(const QVariant &command)
 {
     static const int createInstancesCommandType = QMetaType::type("CreateInstancesCommand");
+    static const int change3DViewCommandType = QMetaType::type("Change3DViewCommand");
     static const int changeFileUrlCommandType = QMetaType::type("ChangeFileUrlCommand");
     static const int createSceneCommandType = QMetaType::type("CreateSceneCommand");
     static const int clearSceneCommandType = QMetaType::type("ClearSceneCommand");
@@ -440,11 +471,14 @@ void NodeInstanceClientProxy::dispatchCommand(const QVariant &command)
     static const int removeSharedMemoryCommandType = QMetaType::type("RemoveSharedMemoryCommand");
     static const int tokenCommandType = QMetaType::type("TokenCommand");
     static const int endPuppetCommandType = QMetaType::type("EndPuppetCommand");
+    static const int changeSelectionCommandType = QMetaType::type("ChangeSelectionCommand");
 
     const int commandType = command.userType();
 
     if (commandType == createInstancesCommandType)
         createInstances(command.value<CreateInstancesCommand>());
+    else if (commandType == change3DViewCommandType)
+        change3DView(command.value<Change3DViewCommand>());
     else if (commandType == changeFileUrlCommandType)
         changeFileUrl(command.value<ChangeFileUrlCommand>());
     else if (commandType == createSceneCommandType)
@@ -480,6 +514,9 @@ void NodeInstanceClientProxy::dispatchCommand(const QVariant &command)
     else if (commandType == synchronizeCommandType) {
         SynchronizeCommand synchronizeCommand = command.value<SynchronizeCommand>();
         m_synchronizeId = synchronizeCommand.synchronizeId();
+    } else if (commandType == changeSelectionCommandType) {
+        ChangeSelectionCommand changeSelectionCommand = command.value<ChangeSelectionCommand>();
+        changeSelection(changeSelectionCommand);
     } else {
         Q_ASSERT(false);
     }
