@@ -66,6 +66,8 @@
 #include <changenodesourcecommand.h>
 #include <tokencommand.h>
 #include <removesharedmemorycommand.h>
+#include <changeselectioncommand.h>
+#include <drop3dlibraryitemcommand.h>
 
 #include <QDebug>
 #include <QQmlEngine>
@@ -88,7 +90,10 @@
 #include <algorithm>
 
 namespace {
-bool testImportStatements(const QStringList &importStatementList, const QUrl &url, QString *errorMessage = 0) {
+
+bool testImportStatements(const QStringList &importStatementList,
+                          const QUrl &url, QString *errorMessage = nullptr)
+{
     if (importStatementList.isEmpty())
         return false;
     // ToDo: move engine outside of this function, this makes it expensive
@@ -157,7 +162,7 @@ void sortFilterImports(const QStringList &imports, QStringList *workingImports,
 
 namespace QmlDesigner {
 
-static NodeInstanceServer *nodeInstanceServerInstance = 0;
+static NodeInstanceServer *nodeInstanceServerInstance = nullptr;
 
 static void notifyPropertyChangeCallBackFunction(QObject *object, const PropertyName &propertyName)
 {
@@ -170,11 +175,7 @@ static void (*notifyPropertyChangeCallBackPointer)(QObject *, const PropertyName
 NodeInstanceServer::NodeInstanceServer(NodeInstanceClientInterface *nodeInstanceClient) :
     NodeInstanceServerInterface(),
     m_childrenChangeEventFilter(new Internal::ChildrenChangeEventFilter(this)),
-    m_nodeInstanceClient(nodeInstanceClient),
-    m_timer(0),
-    m_renderTimerInterval(16),
-    m_slowRenderTimer(false),
-    m_slowRenderTimerInterval(200)
+    m_nodeInstanceClient(nodeInstanceClient)
 {
     qmlRegisterType<DummyContextObject>("QmlDesigner", 1, 0, "DummyContextObject");
 
@@ -184,15 +185,11 @@ NodeInstanceServer::NodeInstanceServer(NodeInstanceClientInterface *nodeInstance
     Internal::QmlPrivateGate::registerFixResourcePathsForObjectCallBack();
 }
 
-NodeInstanceServer::~NodeInstanceServer()
-{
-}
-
 QList<ServerNodeInstance> NodeInstanceServer::createInstances(const QVector<InstanceContainer> &containerVector)
 {
     Q_ASSERT(declarativeView() || quickView());
     QList<ServerNodeInstance> instanceList;
-    foreach (const InstanceContainer &instanceContainer, containerVector) {
+    for (const InstanceContainer &instanceContainer : containerVector) {
         ServerNodeInstance instance;
         if (instanceContainer.nodeSourceType() == InstanceContainer::ComponentSource) {
             instance = ServerNodeInstance::create(this, instanceContainer, ServerNodeInstance::WrapAsComponent);
@@ -247,7 +244,7 @@ ServerNodeInstance NodeInstanceServer::instanceForObject(QObject *object) const
 
 bool NodeInstanceServer::hasInstanceForObject(QObject *object) const
 {
-    if (object == 0)
+    if (object == nullptr)
         return false;
 
     return m_objectInstanceHash.contains(object) && m_objectInstanceHash.value(object).isValid();
@@ -335,6 +332,14 @@ void NodeInstanceServer::clearScene(const ClearSceneCommand &/*command*/)
     m_fileUrl.clear();
 }
 
+void NodeInstanceServer::change3DView(const Change3DViewCommand &/*command*/)
+{
+}
+
+void NodeInstanceServer::changeSelection(const ChangeSelectionCommand & /*command*/)
+{
+}
+
 void NodeInstanceServer::removeInstances(const RemoveInstancesCommand &command)
 {
     ServerNodeInstance oldState = activeStateInstance();
@@ -463,7 +468,7 @@ void NodeInstanceServer::setupImports(const QVector<AddImportContainer> &contain
 
     delete m_importComponent.data();
     delete m_importComponentObject.data();
-    const QStringList importStatementList(importStatementSet.toList());
+    const QStringList importStatementList = QtHelpers::toList(importStatementSet);
     const QStringList fullImportStatementList(QStringList(qtQuickImport) + importStatementList);
 
     // check possible import statements combinations
@@ -590,13 +595,8 @@ QList<ServerNodeInstance> NodeInstanceServer::setupInstances(const CreateSceneCo
         setInstanceAuxiliaryData(container);
     }
 
-
-    QListIterator<ServerNodeInstance> instanceListIterator(instanceList);
-    instanceListIterator.toBack();
-    while (instanceListIterator.hasPrevious()) {
-        ServerNodeInstance instance = instanceListIterator.previous();
-        instance.doComponentComplete();
-    }
+    for (int i = instanceList.size(); --i >= 0; )
+        instanceList[i].doComponentComplete();
 
     return instanceList;
 }
@@ -671,7 +671,7 @@ QQmlContext *NodeInstanceServer::context() const
     if (engine())
         return rootContext();
 
-    return 0;
+    return nullptr;
 }
 
 QQmlContext *NodeInstanceServer::rootContext() const
@@ -968,7 +968,6 @@ void NodeInstanceServer::setInstancePropertyVariant(const PropertyValueContainer
 
 void NodeInstanceServer::setInstanceAuxiliaryData(const PropertyValueContainer &auxiliaryContainer)
 {
-    //instanceId() == 0: the item is root
     if (auxiliaryContainer.instanceId() == 0 && (auxiliaryContainer.name() == "width" ||
                                         auxiliaryContainer.name() == "height")) {
 
@@ -979,7 +978,7 @@ void NodeInstanceServer::setInstanceAuxiliaryData(const PropertyValueContainer &
         }
     }
     if (auxiliaryContainer.name().endsWith("@NodeInstance")) {
-        PropertyName propertyName = auxiliaryContainer.name().left(auxiliaryContainer.name().count() - 12);
+        PropertyName propertyName = auxiliaryContainer.name().left(auxiliaryContainer.name().count() - 13);
         if (!auxiliaryContainer.value().isNull()) {
             setInstancePropertyVariant(PropertyValueContainer(auxiliaryContainer.instanceId(),
                                                               propertyName,
@@ -988,6 +987,14 @@ void NodeInstanceServer::setInstanceAuxiliaryData(const PropertyValueContainer &
         } else {
             rootNodeInstance().resetProperty(propertyName);
         }
+    } else if (auxiliaryContainer.name() == "invisible") {
+        if (hasInstanceForId(auxiliaryContainer.instanceId())) {
+            ServerNodeInstance instance = instanceForId(auxiliaryContainer.instanceId());
+            if (!auxiliaryContainer.value().isNull())
+                instance.setHideInEditor(auxiliaryContainer.value().toBool());
+            else
+                instance.setHideInEditor(false);
+        }
     }
 }
 
@@ -1158,6 +1165,22 @@ ComponentCompletedCommand NodeInstanceServer::createComponentCompletedCommand(co
     return ComponentCompletedCommand(idVector);
 }
 
+ChangeSelectionCommand NodeInstanceServer::createChangeSelectionCommand(const QList<ServerNodeInstance> &instanceList)
+{
+    QVector<qint32> idVector;
+    for (const ServerNodeInstance &instance : instanceList) {
+        if (instance.instanceId() >= 0)
+            idVector.append(instance.instanceId());
+    }
+
+    return ChangeSelectionCommand(idVector);
+}
+
+Drop3DLibraryItemCommand NodeInstanceServer::createDrop3DLibraryItemCommand(const QByteArray &itemData)
+{
+    return Drop3DLibraryItemCommand(itemData);
+}
+
 ValuesChangedCommand NodeInstanceServer::createValuesChangedCommand(const QVector<InstancePropertyPair> &propertyList) const
 {
     QVector<PropertyValueContainer> valueVector;
@@ -1177,6 +1200,31 @@ ValuesChangedCommand NodeInstanceServer::createValuesChangedCommand(const QVecto
     return ValuesChangedCommand(valueVector);
 }
 
+ValuesModifiedCommand NodeInstanceServer::createValuesModifiedCommand(
+    const QVector<InstancePropertyValueTriple> &propertyList) const
+{
+    QVector<PropertyValueContainer> valueVector;
+
+    for (const InstancePropertyValueTriple &property : propertyList) {
+        const PropertyName propertyName = property.propertyName;
+        const ServerNodeInstance instance = property.instance;
+        const QVariant propertyValue = property.propertyValue;
+
+        if (instance.isValid()) {
+            if (QMetaType::isRegistered(propertyValue.userType())
+                && supportedVariantType(propertyValue.type())) {
+                valueVector.append(PropertyValueContainer(instance.instanceId(),
+                                                          propertyName,
+                                                          propertyValue,
+                                                          PropertyName()));
+            }
+        }
+    }
+
+    return ValuesModifiedCommand(valueVector);
+}
+
+
 QByteArray NodeInstanceServer::importCode() const
 {
     return m_importCode;
@@ -1286,7 +1334,7 @@ void NodeInstanceServer::loadDummyContextObjectFile(const QFileInfo& qmlFileInfo
 void NodeInstanceServer::loadDummyDataFiles(const QString& directory)
 {
     QDir dir(directory, "*.qml");
-    QList<QFileInfo> filePathList = dir.entryInfoList();
+    QFileInfoList filePathList = dir.entryInfoList();
     foreach (const QFileInfo &qmlFileInfo, filePathList) {
         loadDummyDataFile(qmlFileInfo);
     }
@@ -1295,7 +1343,7 @@ void NodeInstanceServer::loadDummyDataFiles(const QString& directory)
 void NodeInstanceServer::loadDummyDataContext(const QString& directory)
 {
     QDir dir(directory+"/context", "*.qml");
-    QList<QFileInfo> filePathList = dir.entryInfoList();
+    QFileInfoList filePathList = dir.entryInfoList();
     QString baseName = QFileInfo(fileUrl().toLocalFile()).completeBaseName();
     foreach (const QFileInfo &qmlFileInfo, filePathList) {
         if (qmlFileInfo.completeBaseName() == baseName)
