@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,6 +28,7 @@
 #include <enumeration.h>
 #include <qmlprivategate.h>
 
+#include <QDebug>
 #include <QEvent>
 #include <QQmlContext>
 #include <QQmlError>
@@ -48,6 +44,7 @@
 #include <QQmlParserStatus>
 #include <QTextDocument>
 #include <QLibraryInfo>
+#include <QJSValue>
 
 static bool isSimpleExpression(const QString &expression)
 {
@@ -130,7 +127,8 @@ void ObjectNodeInstance::initializePropertyWatcher(const ObjectNodeInstance::Poi
     m_signalSpy.setObjectNodeInstance(objectNodeInstance);
 }
 
-void ObjectNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance)
+void ObjectNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance,
+                                    InstanceContainer::NodeFlags /*flags*/)
 {
     initializePropertyWatcher(objectNodeInstance);
     QmlPrivateGate::registerNodeInstanceMetaObject(objectNodeInstance->object(), objectNodeInstance->nodeInstanceServer()->engine());
@@ -139,7 +137,7 @@ void ObjectNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNod
 void ObjectNodeInstance::setId(const QString &id)
 {
     if (!m_id.isEmpty() && context()) {
-        context()->engine()->rootContext()->setContextProperty(m_id, 0);
+        context()->engine()->rootContext()->setContextProperty(m_id, nullptr);
     }
 
     if (!id.isEmpty() && context()) {
@@ -258,7 +256,7 @@ bool ObjectNodeInstance::isAnchoredByChildren() const
 
 QPair<PropertyName, ServerNodeInstance> ObjectNodeInstance::anchor(const PropertyName &/*name*/) const
 {
-    return qMakePair(PropertyName(), ServerNodeInstance());
+    return {PropertyName(), ServerNodeInstance()};
 }
 
 
@@ -267,11 +265,17 @@ static bool isList(const QQmlProperty &property)
     return property.propertyTypeCategory() == QQmlProperty::List;
 }
 
+static bool isQJSValue(const QQmlProperty &property)
+{
+    return property.isValid() && !strcmp(property.propertyTypeName(), "QJSValue");
+}
+
 static bool isObject(const QQmlProperty &property)
 {
-    return (property.propertyTypeCategory() == QQmlProperty::Object) ||
-            //QVariant can also store QObjects. Lets trust our model.
-           (QLatin1String(property.propertyTypeName()) == QLatin1String("QVariant"));
+    /* QVariant and QJSValue can also store QObjects. Lets trust our model. */
+    return property.isValid() && (property.propertyTypeCategory() == QQmlProperty::Object
+            || !strcmp(property.propertyTypeName(), "QVariant")
+            || isQJSValue(property));
 }
 
 static QVariant objectToVariant(QObject *object)
@@ -306,7 +310,7 @@ static void removeObjectFromList(const QQmlProperty &property, QObject *objectTo
 
 void ObjectNodeInstance::removeFromOldProperty(QObject *object, QObject *oldParent, const PropertyName &oldParentProperty)
 {
-    QQmlProperty property(oldParent, oldParentProperty, context());
+    QQmlProperty property(oldParent, QString::fromUtf8(oldParentProperty), context());
 
     if (!property.isValid())
         return;
@@ -320,12 +324,12 @@ void ObjectNodeInstance::removeFromOldProperty(QObject *object, QObject *oldPare
     }
 
     if (object && object->parent())
-        object->setParent(0);
+        object->setParent(nullptr);
 }
 
 void ObjectNodeInstance::addToNewProperty(QObject *object, QObject *newParent, const PropertyName &newParentProperty)
 {
-    QQmlProperty property(newParent, newParentProperty, context());
+    QQmlProperty property(newParent, QString::fromUtf8(newParentProperty), context());
 
     if (object)
         object->setParent(newParent);
@@ -340,7 +344,14 @@ void ObjectNodeInstance::addToNewProperty(QObject *object, QObject *newParent, c
 
         list.append(object);
     } else if (isObject(property)) {
-        property.write(objectToVariant(object));
+        if (isQJSValue(property)) /* In this case we have to explcitly generate and convert a QJSValue */
+            property.write(QVariant::fromValue(engine()->newQObject(object)));
+        else
+            property.write(objectToVariant(object));
+
+        if (QQuickItem *item = qobject_cast<QQuickItem *>(object))
+            if (QQuickItem *newParentItem = qobject_cast<QQuickItem *>(newParent))
+                item->setParentItem(newParentItem);
     }
 
     Q_ASSERT(objectToVariant(object).isValid());
@@ -381,6 +392,15 @@ PropertyNameList ObjectNodeInstance::ignoredProperties() const
     return PropertyNameList();
 }
 
+void ObjectNodeInstance::setHideInEditor(bool)
+{
+}
+
+void ObjectNodeInstance::setModifiedFlag(bool b)
+{
+    m_isModified = b;
+}
+
 QVariant ObjectNodeInstance::convertEnumToValue(const QVariant &value, const PropertyName &name)
 {
     Q_ASSERT(value.canConvert<Enumeration>());
@@ -395,7 +415,7 @@ QVariant ObjectNodeInstance::convertEnumToValue(const QVariant &value, const Pro
         QQmlExpression expression(context(), object(), enumeration.toString());
         adjustedValue =  expression.evaluate();
         if (expression.hasError())
-            qDebug() << "Enumeration can not be evaluated:" << object() << name << enumeration;
+            qDebug() << "Enumeration cannot be evaluated:" << object() << name << enumeration;
     }
     return adjustedValue;
 }
@@ -405,7 +425,10 @@ void ObjectNodeInstance::setPropertyVariant(const PropertyName &name, const QVar
     if (ignoredProperties().contains(name))
         return;
 
-    QQmlProperty property(object(), name, context());
+    if (m_isModified)
+        return;
+
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
 
     if (!property.isValid())
         return;
@@ -421,7 +444,7 @@ void ObjectNodeInstance::setPropertyVariant(const PropertyName &name, const QVar
     if (oldValue.type() == QVariant::Url) {
         QUrl url = oldValue.toUrl();
         QString path = url.toLocalFile();
-        if (QFileInfo(path).exists() && nodeInstanceServer() && !path.isEmpty())
+        if (QFileInfo::exists(path) && nodeInstanceServer() && !path.isEmpty())
             nodeInstanceServer()->removeFilePropertyFromFileSystemWatcher(object(), name, path);
     }
 
@@ -438,7 +461,7 @@ void ObjectNodeInstance::setPropertyVariant(const PropertyName &name, const QVar
     if (newValue.type() == QVariant::Url) {
         QUrl url = newValue.toUrl();
         QString path = url.toLocalFile();
-        if (QFileInfo(path).exists() && nodeInstanceServer() && !path.isEmpty())
+        if (QFileInfo::exists(path) && nodeInstanceServer() && !path.isEmpty())
             nodeInstanceServer()->addFilePropertyToFileSystemWatcher(object(), name, path);
     }
 }
@@ -487,7 +510,7 @@ void ObjectNodeInstance::resetProperty(const PropertyName &name)
 
 void ObjectNodeInstance::refreshProperty(const PropertyName &name)
 {
-    QQmlProperty property(object(), name, context());
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
 
     if (!property.isValid())
         return;
@@ -528,7 +551,7 @@ QVariant ObjectNodeInstance::property(const PropertyName &name) const
     if (QmlPrivateGate::isPropertyBlackListed(name))
         return QVariant();
 
-    QQmlProperty property(object(), name, context());
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
     if (property.property().isEnumType()) {
         QVariant value = property.read();
         return property.property().enumerator().valueToKey(value.toInt());
@@ -560,10 +583,10 @@ QString ObjectNodeInstance::instanceType(const PropertyName &name) const
     if (QmlPrivateGate::isPropertyBlackListed(name))
         return QLatin1String("undefined");
 
-    QQmlProperty property(object(), name, context());
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
     if (!property.isValid())
         return QLatin1String("undefined");
-    return property.propertyTypeName();
+    return QString::fromUtf8(property.propertyTypeName());
 }
 
 QList<ServerNodeInstance> ObjectNodeInstance::childItems() const
@@ -606,7 +629,52 @@ ObjectNodeInstance::Pointer ObjectNodeInstance::create(QObject *object)
 
 QObject *ObjectNodeInstance::createPrimitive(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context)
 {
-    return QmlPrivateGate::createPrimitive(typeName, majorNumber, minorNumber, context);
+    QString polishTypeName = typeName;
+    if (typeName == "QtQuick.Controls/Popup"
+            || typeName == "QtQuick.Controls/Drawer"
+            || typeName == "QtQuick.Controls/Dialog"
+            || typeName == "QtQuick.Controls/Menu"
+            || typeName == "QtQuick.Controls/Pane"
+            || typeName == "QtQuick.Controls/ToolTip")
+        polishTypeName = "QtQuick/Item";
+
+    const QHash<QString, QString> mockHash = {{"QtQuick.Controls/SwipeView","qrc:/qtquickplugin/mockfiles/SwipeView.qml"},
+                                             {"QtQuick.Dialogs/Dialog","qrc:/qtquickplugin/mockfiles/Dialog.qml"}};
+
+    QObject *object = nullptr;
+
+    if (mockHash.contains(typeName))
+        object = QmlPrivateGate::createComponent(mockHash.value(typeName), context);
+    else if (majorNumber != -1 && minorNumber != -1)
+        object = QmlPrivateGate::createPrimitive(polishTypeName, majorNumber, minorNumber, context);
+
+    /* Let's try to create the primitive from source, since with incomplete meta info this might be a pure
+     * QML type. This is the case for example if a C++ type is mocked up with a QML file.
+     */
+
+    if (!object)
+        object = createPrimitiveFromSource(polishTypeName, majorNumber, minorNumber, context);
+
+    return object;
+}
+
+QObject *ObjectNodeInstance::createPrimitiveFromSource(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context)
+{
+    if (typeName.isEmpty())
+        return nullptr;
+
+    QStringList parts = typeName.split("/");
+    const QString unqualifiedTypeName = parts.last();
+    parts.removeLast();
+
+    if (parts.isEmpty())
+        return nullptr;
+
+    QString importString = parts.join(".") + " " + QString::number(majorNumber) + "." + QString::number(minorNumber);
+    if (importString == "QtQuick 1.0") /* Workaround for implicit QQml import */
+        importString = "QtQuick 2.0";
+    QString source = "import " + importString + "\n" + unqualifiedTypeName + " {\n" + "}\n";
+    return createCustomParserObject(source, "", context);
 }
 
 QObject *ObjectNodeInstance::createComponentWrap(const QString &nodeSource, const QByteArray &importCode, QQmlContext *context)
@@ -643,16 +711,16 @@ static inline QString fixComponentPathForIncompatibleQt(const QString &component
     if (componentPath.contains(importString)) {
         int index = componentPath.indexOf(importString) + 8;
         const QString relativeImportPath = componentPath.right(componentPath.length() - index);
-        QString fixedComponentPath = QLibraryInfo::location(QLibraryInfo::ImportsPath) + relativeImportPath;
+        QString fixedComponentPath = QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath) + relativeImportPath;
         fixedComponentPath.replace(QLatin1Char('\\'), QLatin1Char('/'));
-        if (QFileInfo(fixedComponentPath).exists())
+        if (QFileInfo::exists(fixedComponentPath))
             return fixedComponentPath;
         QString fixedPath = QFileInfo(fixedComponentPath).path();
         if (fixedPath.endsWith(QLatin1String(".1.0"))) {
         //plugin directories might contain the version number
             fixedPath.chop(4);
             fixedPath += QLatin1Char('/') + QFileInfo(componentPath).fileName();
-            if (QFileInfo(fixedPath).exists())
+            if (QFileInfo::exists(fixedPath))
                 return fixedPath;
         }
     }
@@ -716,12 +784,12 @@ QObject *ObjectNodeInstance::object() const
 {
         if (!m_object.isNull() && !QmlPrivateGate::objectWasDeleted(m_object.data()))
             return m_object.data();
-        return 0;
+        return nullptr;
 }
 
 QQuickItem *ObjectNodeInstance::contentItem() const
 {
-    return 0;
+    return nullptr;
 }
 
 bool ObjectNodeInstance::hasContent() const
@@ -763,7 +831,7 @@ QQmlContext *ObjectNodeInstance::context() const
         return nodeInstanceServer()->context();
 
     qWarning() << "Error: No NodeInstanceServer";
-    return 0;
+    return nullptr;
 }
 
 QQmlEngine *ObjectNodeInstance::engine() const
@@ -811,7 +879,7 @@ QImage ObjectNodeInstance::renderPreviewImage(const QSize & /*previewImageSize*/
 QObject *ObjectNodeInstance::parent() const
 {
     if (!object())
-        return 0;
+        return nullptr;
 
     return object()->parent();
 }
@@ -819,14 +887,8 @@ QObject *ObjectNodeInstance::parent() const
 QObject *ObjectNodeInstance::parentObject(QObject *object)
 {
     QQuickItem *quickItem = qobject_cast<QQuickItem*>(object);
-    if (quickItem && quickItem->parentItem()) {
-
-        //QQuickRootItem is used by Window and we want to return the Window as parent
-        if (strcmp(quickItem->metaObject()->className(), "QQuickRootItem"))
-            return object->parent();
-
+    if (quickItem && quickItem->parentItem())
         return quickItem->parentItem();
-    }
 
     return object->parent();
 }
