@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,36 +9,32 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef OBJECTNODEINSTANCE_H
-#define OBJECTNODEINSTANCE_H
+#pragma once
 
 #include "nodeinstanceserver.h"
 #include "nodeinstancesignalspy.h"
 
+#include "instancecontainer.h"
+
 #include <QPainter>
 #include <QSharedPointer>
 #include <QWeakPointer>
 
-#include "enumeration.h"
+#include <enumeration.h>
 
 QT_BEGIN_NAMESPACE
 class QQmlContext;
@@ -64,8 +60,8 @@ class WidgetNodeInstance;
 class ObjectNodeInstance
 {
 public:
-    typedef QSharedPointer<ObjectNodeInstance> Pointer;
-    typedef QWeakPointer<ObjectNodeInstance> WeakPointer;
+    using Pointer = QSharedPointer<ObjectNodeInstance>;
+    using WeakPointer = QWeakPointer<ObjectNodeInstance>;
 
     virtual ~ObjectNodeInstance();
     void destroy();
@@ -73,6 +69,7 @@ public:
 
     static Pointer create(QObject *objectToBeWrapped);
     static QObject *createPrimitive(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context);
+    static QObject *createPrimitiveFromSource(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context);
     static QObject *createCustomParserObject(const QString &nodeSource, const QByteArray &importCode, QQmlContext *context);
     static QObject *createComponent(const QString &componentPath, QQmlContext *context);
     static QObject *createComponent(const QUrl &componentUrl, QQmlContext *context);
@@ -83,7 +80,7 @@ public:
 
     NodeInstanceServer *nodeInstanceServer() const;
     void setNodeInstanceServer(NodeInstanceServer *server);
-    virtual void initialize(const Pointer &objectNodeInstance);
+    virtual void initialize(const Pointer &objectNodeInstance, InstanceContainer::NodeFlags flags);
     virtual QImage renderImage() const;
     virtual QImage renderPreviewImage(const QSize &previewImageSize) const;
 
@@ -166,7 +163,7 @@ public:
     void setInLayoutable(bool isInLayoutable);
     virtual void refreshLayoutable();
 
-    bool hasBindingForProperty(const PropertyName &propertyName, bool *hasChanged = 0) const;
+    bool hasBindingForProperty(const PropertyName &propertyName, bool *hasChanged = nullptr) const;
 
     QQmlContext *context() const;
     QQmlEngine *engine() const;
@@ -196,6 +193,10 @@ public:
 
     virtual PropertyNameList ignoredProperties() const;
 
+    void virtual setHideInEditor(bool b);
+
+    void setModifiedFlag(bool b);
+
 protected:
     explicit ObjectNodeInstance(QObject *object);
     void doResetProperty(const PropertyName &propertyName);
@@ -221,10 +222,9 @@ private:
     qint32 m_instanceId;
     bool m_deleteHeldInstance;
     bool m_isInLayoutable;
+    bool m_isModified = false;
     static QHash<EnumerationName, QVariant> m_enumationValueHash;
 };
 
 } // namespace Internal
 } // namespace QmlDesigner
-
-#endif // OBJECTNODEINSTANCE_H
