@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,25 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "qt5nodeinstanceserver.h"
 
+#include <QSurfaceFormat>
+
+#include <QQmlFileSelector>
 
 #include <QQuickItem>
 #include <QQuickView>
@@ -39,6 +37,9 @@
 #include <createscenecommand.h>
 #include <reparentinstancescommand.h>
 
+#include <QDebug>
+#include <QOpenGLContext>
+
 namespace QmlDesigner {
 
 Qt5NodeInstanceServer::Qt5NodeInstanceServer(NodeInstanceClientInterface *nodeInstanceClient)
@@ -62,12 +63,35 @@ void Qt5NodeInstanceServer::initializeView()
     Q_ASSERT(!quickView());
 
     m_quickView = new QQuickView;
+
+    QSurfaceFormat surfaceFormat = m_quickView->requestedFormat();
+    surfaceFormat.setVersion(4, 1);
+    surfaceFormat.setProfile(QSurfaceFormat::CoreProfile);
+    m_quickView->setFormat(surfaceFormat);
+
     DesignerSupport::createOpenGLContext(m_quickView.data());
+
+    if (qEnvironmentVariableIsSet("QMLDESIGNER_QUICK3D_MODE")
+        && QCoreApplication::arguments().at(2) == "editormode") {
+        /* In '3d editormode' we do not use the DesignerWindowManager
+         * and since we do not show the QQuickView we have to manually create the OpenGL context */
+        auto context = new QOpenGLContext(m_quickView);
+        context->setFormat(surfaceFormat);
+        context->create();
+        if (!context->makeCurrent(m_quickView))
+            qWarning("QOpenGLContext: makeCurrent() failed...");
+    }
+
+    if (qEnvironmentVariableIsSet("QML_FILE_SELECTORS")) {
+        QQmlFileSelector *fileSelector = new QQmlFileSelector(engine(), engine());
+        QStringList customSelectors = QString::fromUtf8(qgetenv("QML_FILE_SELECTORS")).split(",");
+        fileSelector->setExtraSelectors(customSelectors);
+    }
 }
 
 QQmlView *Qt5NodeInstanceServer::declarativeView() const
 {
-    return 0;
+    return nullptr;
 }
 
 QQmlEngine *Qt5NodeInstanceServer::engine() const
@@ -75,7 +99,7 @@ QQmlEngine *Qt5NodeInstanceServer::engine() const
     if (quickView())
         return quickView()->engine();
 
-    return 0;
+    return nullptr;
 }
 
 void Qt5NodeInstanceServer::resizeCanvasSizeToRootItemSize()
@@ -90,6 +114,8 @@ void Qt5NodeInstanceServer::resetAllItems()
 
 void Qt5NodeInstanceServer::setupScene(const CreateSceneCommand &command)
 {
+
+    setupMockupTypes(command.mockupTypes());
     setupFileUrl(command.fileUrl());
     setupImports(command.imports());
     setupDummyData(command.fileUrl());
