@@ -32,8 +32,10 @@
 #include <extensionsystem/pluginspec.h>
 #include <qtsingleapplication.h>
 
+#include <utils/environment.h>
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
+#include <utils/optional.h>
 #include <utils/temporarydirectory.h>
 
 #include <QDebug>
@@ -41,7 +43,6 @@
 #include <QFontDatabase>
 #include <QFileInfo>
 #include <QLibraryInfo>
-#include <QLoggingCategory>
 #include <QSettings>
 #include <QStyle>
 #include <QTextStream>
@@ -58,6 +59,10 @@
 #include <QStandardPaths>
 #include <QTemporaryDir>
 
+#include <string>
+#include <vector>
+#include <iterator>
+
 #ifdef ENABLE_QT_BREAKPAD
 #include <qtsystemexceptionhandler.h>
 #endif
@@ -74,6 +79,8 @@ const char fixedOptionsC[] =
 "    -version                      Display program version\n"
 "    -client                       Attempt to connect to already running first instance\n"
 "    -settingspath <path>          Override the default path where user settings are stored\n"
+"    -installsettingspath <path>   Override the default path from where user-independent settings are read\n"
+"    -temporarycleansettings       Use clean settings for debug or testing reasons\n"
 "    -pid <pid>                    Attempt to connect to instance given by pid\n"
 "    -block                        Block until editor is closed\n"
 "    -pluginpath <path>            Add a custom search path for plugins\n";
@@ -85,12 +92,16 @@ const char HELP_OPTION4[] = "--help";
 const char VERSION_OPTION[] = "-version";
 const char CLIENT_OPTION[] = "-client";
 const char SETTINGS_OPTION[] = "-settingspath";
+const char INSTALL_SETTINGS_OPTION[] = "-installsettingspath";
 const char TEST_OPTION[] = "-test";
+const char TEMPORARY_CLEAN_SETTINGS1[] = "-temporarycleansettings";
+const char TEMPORARY_CLEAN_SETTINGS2[] = "-tcs";
 const char PID_OPTION[] = "-pid";
 const char BLOCK_OPTION[] = "-block";
 const char PLUGINPATH_OPTION[] = "-pluginpath";
+const char USER_LIBRARY_PATH_OPTION[] = "-user-library-path"; // hidden option for qtcreator.sh
 
-typedef QList<PluginSpec *> PluginSpecSet;
+using PluginSpecSet = QVector<PluginSpec *>;
 
 // Helpers for displaying messages. Note that there is no console on Windows.
 
@@ -108,16 +119,16 @@ static inline QString toHtml(const QString &t)
 
 static void displayHelpText(const QString &t)
 {
-    if (Utils::HostOsInfo::isWindowsHost())
-        QMessageBox::information(0, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), toHtml(t));
+    if (Utils::HostOsInfo::isWindowsHost() && qApp)
+        QMessageBox::information(nullptr, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), toHtml(t));
     else
         qWarning("%s", qPrintable(t));
 }
 
 static void displayError(const QString &t)
 {
-    if (Utils::HostOsInfo::isWindowsHost())
-        QMessageBox::critical(0, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), t);
+    if (Utils::HostOsInfo::isWindowsHost() && qApp)
+        QMessageBox::critical(nullptr, QLatin1String(Core::Constants::IDE_DISPLAY_NAME), t);
     else
         qCritical("%s", qPrintable(t));
 }
@@ -142,6 +153,24 @@ static void printHelp(const QString &a0)
     displayHelpText(help);
 }
 
+QString applicationDirPath(char *arg = nullptr)
+{
+    static QString dir;
+
+    if (arg)
+        dir = QFileInfo(QString::fromLocal8Bit(arg)).dir().absolutePath();
+
+    if (QCoreApplication::instance())
+        return QApplication::applicationDirPath();
+
+    return dir;
+}
+
+static QString resourcePath()
+{
+    return QDir::cleanPath(applicationDirPath() + '/' + RELATIVE_DATA_PATH);
+}
+
 static inline QString msgCoreLoadFailure(const QString &why)
 {
     return QCoreApplication::translate("Application", "Failed to load core: %1").arg(why);
@@ -149,26 +178,16 @@ static inline QString msgCoreLoadFailure(const QString &why)
 
 static inline int askMsgSendFailed()
 {
-    return QMessageBox::question(0, QApplication::translate("Application","Could not send message"),
-                                 QCoreApplication::translate("Application", "Unable to send command line arguments to the already running instance. "
-                                                             "It appears to be not responding. Do you want to start a new instance of Creator?"),
-                                 QMessageBox::Yes | QMessageBox::No | QMessageBox::Retry,
-                                 QMessageBox::Retry);
+    return QMessageBox::question(nullptr, QApplication::translate("Application","Could not send message"),
+                QCoreApplication::translate("Application", "Unable to send command line arguments "
+                                            "to the already running instance. It does not appear to "
+                                            "be responding. Do you want to start a new instance of "
+                                            "%1?").arg(Core::Constants::IDE_DISPLAY_NAME),
+                QMessageBox::Yes | QMessageBox::No | QMessageBox::Retry,
+                QMessageBox::Retry);
 }
 
-static void setHighDpiEnvironmentVariable()
-{
-    static const char ENV_VAR_QT_DEVICE_PIXEL_RATIO[] = "QT_DEVICE_PIXEL_RATIO";
-    if (Utils::HostOsInfo().isWindowsHost()
-            && !qEnvironmentVariableIsSet(ENV_VAR_QT_DEVICE_PIXEL_RATIO) // legacy in 5.6, but still functional
-            && !qEnvironmentVariableIsSet("QT_AUTO_SCREEN_SCALE_FACTOR")
-            && !qEnvironmentVariableIsSet("QT_SCALE_FACTOR")
-            && !qEnvironmentVariableIsSet("QT_SCREEN_SCALE_FACTORS")) {
-        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
-    }
-}
-
-// taken from utils/fileutils.cpp. We can not use utils here since that depends app_version.h.
+// taken from utils/fileutils.cpp. We cannot use utils here since that depends app_version.h.
 static bool copyRecursively(const QString &srcFilePath,
                             const QString &tgtFilePath)
 {
@@ -176,7 +195,7 @@ static bool copyRecursively(const QString &srcFilePath,
     if (srcFileInfo.isDir()) {
         QDir targetDir(tgtFilePath);
         targetDir.cdUp();
-        if (!targetDir.mkdir(Utils::FileName::fromString(tgtFilePath).fileName()))
+        if (!targetDir.mkdir(Utils::FilePath::fromString(tgtFilePath).fileName()))
             return false;
         QDir sourceDir(srcFilePath);
         QStringList fileNames = sourceDir.entryList(QDir::Files | QDir::Dirs | QDir::NoDotAndDotDot | QDir::Hidden | QDir::System);
@@ -214,26 +233,36 @@ static inline QStringList getPluginPaths()
                                     Core::Constants::IDE_DISPLAY_NAME :
                                     Core::Constants::IDE_ID);
     pluginPath += QLatin1String("/plugins/");
-    pluginPath += QLatin1String(Core::Constants::IDE_VERSION_LONG);
-    rc.push_back(pluginPath);
+    // Qt Creator X.Y.Z can load plugins from X.Y.(Z-1) etc, so add current and previous
+    // patch versions
+    const QString minorVersion = QString::number(IDE_VERSION_MAJOR) + '.'
+                                 + QString::number(IDE_VERSION_MINOR) + '.';
+    for (int patchVersion = IDE_VERSION_RELEASE; patchVersion >= 0; --patchVersion)
+        rc.push_back(pluginPath + minorVersion + QString::number(patchVersion));
     return rc;
 }
 
-static void setupInstallSettings()
+static void setupInstallSettings(QString &installSettingspath)
 {
+    if (!installSettingspath.isEmpty() && !QFileInfo(installSettingspath).isDir()) {
+        displayError(QString("-installsettingspath \"%0\" needs to be the path where a %1/%2.ini exist.").arg(installSettingspath,
+            QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR), QLatin1String(Core::Constants::IDE_CASED_ID)));
+        installSettingspath.clear();
+    }
     // Check if the default install settings contain a setting for the actual install settings.
     // This can be an absolute path, or a path relative to applicationDirPath().
     // The result is interpreted like -settingspath, but for SystemScope
     static const char kInstallSettingsKey[] = "Settings/InstallSettings";
     QSettings::setPath(QSettings::IniFormat, QSettings::SystemScope,
-                       QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH);
+        installSettingspath.isEmpty() ? resourcePath() : installSettingspath);
+
     QSettings installSettings(QSettings::IniFormat, QSettings::UserScope,
                               QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
                               QLatin1String(Core::Constants::IDE_CASED_ID));
     if (installSettings.contains(kInstallSettingsKey)) {
         QString installSettingsPath = installSettings.value(kInstallSettingsKey).toString();
         if (QDir::isRelativePath(installSettingsPath))
-            installSettingsPath = QCoreApplication::applicationDirPath() + '/' + installSettingsPath;
+            installSettingsPath = applicationDirPath() + '/' + installSettingsPath;
         QSettings::setPath(QSettings::IniFormat, QSettings::SystemScope, installSettingsPath);
     }
 }
@@ -288,16 +317,105 @@ static inline QSettings *userSettings()
     return createUserSettings();
 }
 
+static void setHighDpiEnvironmentVariable()
+{
+
+    if (Utils::HostOsInfo().isMacHost())
+        return;
+
+    std::unique_ptr<QSettings> settings(createUserSettings());
+
+    const bool defaultValue = Utils::HostOsInfo().isWindowsHost();
+    const bool enableHighDpiScaling = settings->value("Core/EnableHighDpiScaling", defaultValue).toBool();
+
+    static const char ENV_VAR_QT_DEVICE_PIXEL_RATIO[] = "QT_DEVICE_PIXEL_RATIO";
+    if (enableHighDpiScaling
+            && !qEnvironmentVariableIsSet(ENV_VAR_QT_DEVICE_PIXEL_RATIO) // legacy in 5.6, but still functional
+            && !qEnvironmentVariableIsSet("QT_AUTO_SCREEN_SCALE_FACTOR")
+            && !qEnvironmentVariableIsSet("QT_SCALE_FACTOR")
+            && !qEnvironmentVariableIsSet("QT_SCREEN_SCALE_FACTORS")) {
+        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+    }
+}
+
 void loadFonts()
 {
-    const QDir dir(QCoreApplication::applicationDirPath() + '/' + RELATIVE_DATA_PATH + "/fonts/");
+    const QDir dir(resourcePath() + "/fonts/");
 
     foreach (const QFileInfo &fileInfo, dir.entryInfoList(QStringList("*.ttf"), QDir::Files))
         QFontDatabase::addApplicationFont(fileInfo.absoluteFilePath());
 }
 
+struct Options
+{
+    QString settingsPath;
+    QString installSettingsPath;
+    QStringList customPluginPaths;
+    std::vector<char *> appArguments;
+    Utils::optional<QString> userLibraryPath;
+    bool hasTestOption = false;
+    bool wantsCleanSettings = false;
+};
+
+Options parseCommandLine(int argc, char *argv[])
+{
+    Options options;
+    auto it = argv;
+    const auto end = argv + argc;
+    while (it != end) {
+        const auto arg = QString::fromLocal8Bit(*it);
+        const bool hasNext = it + 1 != end;
+        const auto nextArg = hasNext ? QString::fromLocal8Bit(*(it + 1)) : QString();
+
+        if (arg == SETTINGS_OPTION && hasNext) {
+            ++it;
+            options.settingsPath = QDir::fromNativeSeparators(nextArg);
+        } else if (arg == INSTALL_SETTINGS_OPTION && hasNext) {
+            ++it;
+            options.installSettingsPath = QDir::fromNativeSeparators(nextArg);
+        } else if (arg == PLUGINPATH_OPTION && hasNext) {
+            ++it;
+            options.customPluginPaths += QDir::fromNativeSeparators(nextArg);
+        } else if (arg == USER_LIBRARY_PATH_OPTION && hasNext) {
+            ++it;
+            options.userLibraryPath = nextArg;
+        } else if (arg == TEMPORARY_CLEAN_SETTINGS1 || arg == TEMPORARY_CLEAN_SETTINGS2) {
+            options.wantsCleanSettings = true;
+        } else { // arguments that are still passed on to the application
+            if (arg == TEST_OPTION)
+                options.hasTestOption = true;
+            options.appArguments.push_back(*it);
+        }
+        ++it;
+    }
+    return options;
+}
+
 int main(int argc, char **argv)
 {
+    Utils::Environment::systemEnvironment(); // cache system environment before we do any changes
+
+    // Manually determine various command line options
+    // We can't use the regular way of the plugin manager,
+    // because settings can change the way plugin manager behaves
+    Options options = parseCommandLine(argc, argv);
+    applicationDirPath(argv[0]);
+
+    if (options.userLibraryPath) {
+        if ((*options.userLibraryPath).isEmpty()) {
+            Utils::Environment::modifySystemEnvironment(
+                {{"LD_LIBRARY_PATH", "", Utils::EnvironmentItem::Unset}});
+        } else {
+            Utils::Environment::modifySystemEnvironment(
+                {{"LD_LIBRARY_PATH", *options.userLibraryPath, Utils::EnvironmentItem::SetEnabled}});
+        }
+    }
+
+#ifdef Q_OS_WIN
+    if (!qEnvironmentVariableIsSet("QT_OPENGL"))
+        QCoreApplication::setAttribute(Qt::AA_UseOpenGLES);
+#endif
+
     if (qEnvironmentVariableIsSet("QTCREATOR_DISABLE_NATIVE_MENUBAR")
             || qgetenv("XDG_CURRENT_DESKTOP").startsWith("Unity")) {
         QApplication::setAttribute(Qt::AA_DontUseNativeMenuBar);
@@ -305,10 +423,6 @@ int main(int argc, char **argv)
 
     Utils::TemporaryDirectory::setMasterTemporaryDirectory(QDir::tempPath() + "/" + Core::Constants::IDE_CASED_ID + "-XXXXXX");
 
-    setHighDpiEnvironmentVariable();
-
-    QLoggingCategory::setFilterRules(QLatin1String("qtc.*.debug=false\nqtc.*.info=false"));
-
 #ifdef Q_OS_MAC
     // increase the number of file that can be opened in Qt Creator.
     struct rlimit rl;
@@ -318,9 +432,38 @@ int main(int argc, char **argv)
     setrlimit(RLIMIT_NOFILE, &rl);
 #endif
 
-    SharedTools::QtSingleApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
-    SharedTools::QtSingleApplication app((QLatin1String(Core::Constants::IDE_DISPLAY_NAME)), argc, argv);
+    QScopedPointer<Utils::TemporaryDirectory> temporaryCleanSettingsDir;
+    if (options.settingsPath.isEmpty() && (options.hasTestOption || options.wantsCleanSettings)) {
+        temporaryCleanSettingsDir.reset(new Utils::TemporaryDirectory("qtc-test-settings"));
+        if (!temporaryCleanSettingsDir->isValid())
+            return 1;
+        options.settingsPath = temporaryCleanSettingsDir->path();
+    }
+    if (!options.settingsPath.isEmpty())
+        QSettings::setPath(QSettings::IniFormat, QSettings::UserScope, options.settingsPath);
 
+    // Must be done before any QSettings class is created
+    QSettings::setDefaultFormat(QSettings::IniFormat);
+    setupInstallSettings(options.installSettingsPath);
+    // plugin manager takes control of this settings object
+
+    setHighDpiEnvironmentVariable();
+
+    SharedTools::QtSingleApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
+
+    int numberofArguments = static_cast<int>(options.appArguments.size());
+
+    SharedTools::QtSingleApplication app((QLatin1String(Core::Constants::IDE_DISPLAY_NAME)),
+                                         numberofArguments,
+                                         options.appArguments.data());
+    const QStringList pluginArguments = app.arguments();
+
+    /*Initialize global settings and resetup install settings with QApplication::applicationDirPath */
+    setupInstallSettings(options.installSettingsPath);
+    QSettings *settings = userSettings();
+    QSettings *globalSettings = new QSettings(QSettings::IniFormat, QSettings::SystemScope,
+                                              QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
+                                              QLatin1String(Core::Constants::IDE_CASED_ID));
     loadFonts();
 
     if (Utils::HostOsInfo().isWindowsHost()
@@ -344,51 +487,6 @@ int main(int argc, char **argv)
 
     app.setAttribute(Qt::AA_UseHighDpiPixmaps);
 
-    // Manually determine -settingspath command line option
-    // We can't use the regular way of the plugin manager, because that needs to parse plugin meta data
-    // but the settings path can influence which plugins are enabled
-    QString settingsPath;
-    QStringList customPluginPaths;
-    QStringList arguments = app.arguments(); // adapted arguments list is passed to plugin manager later
-    QMutableStringListIterator it(arguments);
-    bool testOptionProvided = false;
-    while (it.hasNext()) {
-        const QString &arg = it.next();
-        if (arg == QLatin1String(SETTINGS_OPTION)) {
-            it.remove();
-            if (it.hasNext()) {
-                settingsPath = QDir::fromNativeSeparators(it.next());
-                it.remove();
-            }
-        } else if (arg == QLatin1String(PLUGINPATH_OPTION)) {
-            it.remove();
-            if (it.hasNext()) {
-                customPluginPaths << QDir::fromNativeSeparators(it.next());
-                it.remove();
-            }
-        } else if (arg == QLatin1String(TEST_OPTION)) {
-            testOptionProvided = true;
-        }
-    }
-    QScopedPointer<Utils::TemporaryDirectory> temporaryCleanSettingsDir;
-    if (settingsPath.isEmpty() && testOptionProvided) {
-        temporaryCleanSettingsDir.reset(new Utils::TemporaryDirectory("qtc-test-settings"));
-        if (!temporaryCleanSettingsDir->isValid())
-            return 1;
-        settingsPath = temporaryCleanSettingsDir->path();
-    }
-    if (!settingsPath.isEmpty())
-        QSettings::setPath(QSettings::IniFormat, QSettings::UserScope, settingsPath);
-
-    // Must be done before any QSettings class is created
-    QSettings::setDefaultFormat(QSettings::IniFormat);
-    setupInstallSettings();
-    // plugin manager takes control of this settings object
-    QSettings *settings = userSettings();
-
-    QSettings *globalSettings = new QSettings(QSettings::IniFormat, QSettings::SystemScope,
-                                              QLatin1String(Core::Constants::IDE_SETTINGSVARIANT_STR),
-                                              QLatin1String(Core::Constants::IDE_CASED_ID));
     PluginManager pluginManager;
     PluginManager::setPluginIID(QLatin1String("org.qt-project.Qt.QtCreatorPlugin"));
     PluginManager::setGlobalSettings(globalSettings);
@@ -401,11 +499,10 @@ int main(int argc, char **argv)
     QString overrideLanguage = settings->value(QLatin1String("General/OverrideLanguage")).toString();
     if (!overrideLanguage.isEmpty())
         uiLanguages.prepend(overrideLanguage);
-    const QString &creatorTrPath = QCoreApplication::applicationDirPath()
-            + '/' + RELATIVE_DATA_PATH + "/translations";
+    const QString &creatorTrPath = resourcePath() + "/translations";
     foreach (QString locale, uiLanguages) {
         locale = QLocale(locale).name();
-        if (translator.load(QString::fromLatin1(Core::Constants::IDE_ID) + "_" + locale, creatorTrPath)) {
+        if (translator.load("qtcreator_" + locale, creatorTrPath)) {
             const QString &qtTrPath = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
             const QString &qtTrFile = QLatin1String("qt_") + locale;
             // Binary installer puts Qt tr files into creatorTrPath
@@ -425,14 +522,16 @@ int main(int argc, char **argv)
         }
     }
 
+    app.setDesktopFileName("org.qt-project.qtcreator.desktop");
+
     // Make sure we honor the system's proxy settings
     QNetworkProxyFactory::setUseSystemConfiguration(true);
 
     // Load
-    const QStringList pluginPaths = getPluginPaths() + customPluginPaths;
+    const QStringList pluginPaths = getPluginPaths() + options.customPluginPaths;
     PluginManager::setPluginPaths(pluginPaths);
     QMap<QString, QString> foundAppOptions;
-    if (arguments.size() > 1) {
+    if (pluginArguments.size() > 1) {
         QMap<QString, bool> appOptions;
         appOptions.insert(QLatin1String(HELP_OPTION1), false);
         appOptions.insert(QLatin1String(HELP_OPTION2), false);
@@ -443,7 +542,7 @@ int main(int argc, char **argv)
         appOptions.insert(QLatin1String(PID_OPTION), true);
         appOptions.insert(QLatin1String(BLOCK_OPTION), false);
         QString errorMessage;
-        if (!PluginManager::parseOptions(arguments, appOptions, &foundAppOptions, &errorMessage)) {
+        if (!PluginManager::parseOptions(pluginArguments, appOptions, &foundAppOptions, &errorMessage)) {
             displayError(errorMessage);
             printHelp(QFileInfo(app.applicationFilePath()).baseName());
             return -1;
@@ -451,7 +550,7 @@ int main(int argc, char **argv)
     }
 
     const PluginSpecSet plugins = PluginManager::plugins();
-    PluginSpec *coreplugin = 0;
+    PluginSpec *coreplugin = nullptr;
     foreach (PluginSpec *spec, plugins) {
         if (spec->name() == QLatin1String(corePluginNameC)) {
             coreplugin = spec;
