@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -47,14 +42,11 @@ using namespace CPlusPlus;
 
 FindUsages::FindUsages(const QByteArray &originalSource, Document::Ptr doc, const Snapshot &snapshot)
     : ASTVisitor(doc->translationUnit()),
-      _id(0),
-      _declSymbol(0),
       _doc(doc),
       _snapshot(snapshot),
       _context(doc, snapshot),
       _originalSource(originalSource),
-      _source(_doc->utf8Source()),
-      _currentScope(0)
+      _source(_doc->utf8Source())
 {
     _snapshot.insert(_doc);
     typeofExpression.init(_doc, _snapshot, _context.bindings());
@@ -64,14 +56,11 @@ FindUsages::FindUsages(const QByteArray &originalSource, Document::Ptr doc, cons
 
 FindUsages::FindUsages(const LookupContext &context)
     : ASTVisitor(context.thisDocument()->translationUnit()),
-      _id(0),
-      _declSymbol(0),
       _doc(context.thisDocument()),
       _snapshot(context.snapshot()),
       _context(context),
       _originalSource(_doc->utf8Source()),
-      _source(_doc->utf8Source()),
-      _currentScope(0)
+      _source(_doc->utf8Source())
 {
     typeofExpression.init(_doc, _snapshot, _context.bindings());
 
@@ -109,7 +98,7 @@ void FindUsages::operator()(Symbol *symbol)
 
 void FindUsages::reportResult(unsigned tokenIndex, const Name *name, Scope *scope)
 {
-    if (! (tokenIndex && name != 0))
+    if (! (tokenIndex && name != nullptr))
         return;
 
     if (name->identifier() != _id)
@@ -168,10 +157,10 @@ void FindUsages::reportResult(unsigned tokenIndex)
 
     _processed.insert(tokenIndex);
 
-    unsigned line, col;
+    int line, col;
     getTokenStartPosition(tokenIndex, &line, &col);
     QString lineText;
-    if (line < _sourceLineEnds.size())
+    if (line < int(_sourceLineEnds.size()))
         lineText = fetchLine(line);
     else
         lineText = matchingLine(tk);
@@ -181,7 +170,7 @@ void FindUsages::reportResult(unsigned tokenIndex)
 
     const int len = tk.utf16chars();
 
-    const Usage u(_doc->fileName(), lineText, line, col, len);
+    const Usage u(Utils::FilePath::fromString(_doc->fileName()), lineText, line, col, len);
     _usages.append(u);
     _references.append(tokenIndex);
 }
@@ -309,7 +298,7 @@ const Name *FindUsages::name(NameAST *ast)
         return ast->name;
     }
 
-    return 0;
+    return nullptr;
 }
 
 void FindUsages::specifier(SpecifierAST *ast)
@@ -532,7 +521,7 @@ void FindUsages::memInitializer(MemInitializerAST *ast)
     if (_currentScope->isFunction()) {
         Class *classScope = _currentScope->enclosingClass();
         if (! classScope) {
-            if (LookupScope *binding = _context.lookupType(_currentScope)) {
+            if (ClassOrNamespace *binding = _context.lookupType(_currentScope)) {
                 foreach (Symbol *s, binding->symbols()) {
                     if (Class *k = s->asClass()) {
                         classScope = k;
@@ -1381,6 +1370,12 @@ bool FindUsages::visit(ThrowExpressionAST *ast)
     return false;
 }
 
+bool FindUsages::visit(NoExceptOperatorExpressionAST* ast)
+{
+    this->expression(ast->expression);
+    return false;
+}
+
 bool FindUsages::visit(TypeIdAST *ast)
 {
     for (SpecifierListAST *it = ast->type_specifier_list; it; it = it->next) {
@@ -1872,7 +1867,7 @@ bool FindUsages::visit(QualifiedNameAST *ast)
         if (NameAST *class_or_namespace_name = nested_name_specifier->class_or_namespace_name) {
             SimpleNameAST *simple_name = class_or_namespace_name->asSimpleName();
 
-            TemplateIdAST *template_id = 0;
+            TemplateIdAST *template_id = nullptr;
             if (! simple_name) {
                 template_id = class_or_namespace_name->asTemplateId();
 
@@ -1902,7 +1897,7 @@ bool FindUsages::visit(QualifiedNameAST *ast)
         else if (DestructorNameAST *dtor = unqualified_name->asDestructorName())
             identifier_token = dtor->unqualified_name->firstToken();
 
-        TemplateIdAST *template_id = 0;
+        TemplateIdAST *template_id = nullptr;
         if (! identifier_token) {
             template_id = unqualified_name->asTemplateId();
 
