@@ -83,7 +83,7 @@ public:
         // use an "alreadyResolved" container. FIXME: We might overcome this by resolving the
         // template parameters.
         unsigned maxDepth = 15;
-        for (NamedType *namedTy = 0; maxDepth && (namedTy = getNamedType(*type)); --maxDepth) {
+        for (NamedType *namedTy = nullptr; maxDepth && (namedTy = getNamedType(*type)); --maxDepth) {
             QList<LookupItem> namedTypeItems = getNamedTypeItems(namedTy->name(), *scope, _binding);
 
             if (Q_UNLIKELY(debug))
@@ -125,9 +125,9 @@ private:
         QList<LookupItem> results;
         if (!scope)
             return results;
-        Scope *enclosingBlockScope = 0;
+        Scope *enclosingBlockScope = nullptr;
         for (Block *block = scope->asBlock(); block;
-             block = enclosingBlockScope ? enclosingBlockScope->asBlock() : 0) {
+             block = enclosingBlockScope ? enclosingBlockScope->asBlock() : nullptr) {
             const unsigned memberCount = block->memberCount();
             for (unsigned i = 0; i < memberCount; ++i) {
                 Symbol *symbol = block->memberAt(i);
@@ -233,7 +233,7 @@ static int evaluateFunctionArgument(const FullySpecifiedType &actualTy,
 ResolveExpression::ResolveExpression(const LookupContext &context,
                                      const QSet<const Declaration *> &autoDeclarationsBeingResolved)
     : ASTVisitor(context.expressionDocument()->translationUnit()),
-      _scope(0),
+      _scope(nullptr),
       _context(context),
       bind(context.expressionDocument()->translationUnit()),
       _autoDeclarationsBeingResolved(autoDeclarationsBeingResolved),
@@ -314,9 +314,9 @@ bool ResolveExpression::visit(IdExpressionAST *ast)
 
 bool ResolveExpression::visit(BinaryExpressionAST *ast)
 {
-    if (tokenKind(ast->binary_op_token) == T_COMMA && ast->right_expression && ast->right_expression->asQtMethod() != 0) {
+    if (tokenKind(ast->binary_op_token) == T_COMMA && ast->right_expression && ast->right_expression->asQtMethod() != nullptr) {
 
-        if (ast->left_expression && ast->left_expression->asQtMethod() != 0)
+        if (ast->left_expression && ast->left_expression->asQtMethod() != nullptr)
             thisObject();
         else
             accept(ast->left_expression);
@@ -404,7 +404,7 @@ bool ResolveExpression::visit(TypeidExpressionAST *)
 {
     const Name *stdName = control()->identifier("std");
     const Name *tiName = control()->identifier("type_info");
-    const Name *q = control()->qualifiedNameId(control()->qualifiedNameId(/* :: */ 0, stdName), tiName);
+    const Name *q = control()->qualifiedNameId(control()->qualifiedNameId(/* :: */ nullptr, stdName), tiName);
 
     FullySpecifiedType ty(control()->namedType(q));
     addResult(ty, _scope);
@@ -443,7 +443,7 @@ bool ResolveExpression::visit(NumericLiteralAST *ast)
 {
     const Token &tk = tokenAt(ast->literal_token);
 
-    Type *type = 0;
+    Type *type = nullptr;
     bool isUnsigned = false;
 
     if (tk.is(T_CHAR_LITERAL)) {
@@ -581,21 +581,17 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
     accept(ast->expression);
     unsigned unaryOp = tokenKind(ast->unary_op_token);
     if (unaryOp == T_AMPER) {
-        QMutableListIterator<LookupItem > it(_results);
-        while (it.hasNext()) {
-            LookupItem p = it.next();
+        for (LookupItem &p : _results) {
             FullySpecifiedType ty = p.type();
             ty.setType(control()->pointerType(ty));
             p.setType(ty);
-            it.setValue(p);
         }
     } else if (unaryOp == T_STAR) {
-        QMutableListIterator<LookupItem > it(_results);
-        while (it.hasNext()) {
-            LookupItem p = it.next();
+        for (int i = 0; i < _results.size(); ++i) {
+            LookupItem &p = _results[i];
             FullySpecifiedType ty = p.type();
             NamedType *namedTy = ty->asNamedType();
-            if (namedTy != 0) {
+            if (namedTy != nullptr) {
                 const QList<LookupItem> types = _context.lookup(namedTy->name(), p.scope());
                 if (!types.empty())
                     ty = types.front().type();
@@ -603,9 +599,8 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
             bool added = false;
             if (PointerType *ptrTy = ty->asPointerType()) {
                 p.setType(ptrTy->elementType());
-                it.setValue(p);
                 added = true;
-            } else if (namedTy != 0) {
+            } else if (namedTy != nullptr) {
                 const Name *starOp = control()->operatorNameId(OperatorNameId::StarOp);
                 if (ClassOrNamespace *b = _context.lookupType(namedTy->name(), p.scope(), p.binding())) {
                     foreach (const LookupItem &r, b->find(starOp)) {
@@ -616,7 +611,6 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
                                     FullySpecifiedType retTy = proto->returnType().simplified();
                                     p.setType(retTy);
                                     p.setScope(proto->enclosingScope());
-                                    it.setValue(p);
                                     added = true;
                                     break;
                                 }
@@ -626,7 +620,7 @@ bool ResolveExpression::visit(UnaryExpressionAST *ast)
                 }
             }
             if (!added)
-                it.remove();
+                _results.removeAt(i--);
         }
     }
     return false;
@@ -725,7 +719,7 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
         if (!item.type()->isUndefinedType())
             continue;
 
-        if (item.declaration() == 0)
+        if (item.declaration() == nullptr)
             continue;
 
         if (item.type().isAuto()) {
@@ -738,7 +732,7 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
                 continue;
 
             const StringLiteral *initializationString = decl->getInitializer();
-            if (initializationString == 0)
+            if (initializationString == nullptr)
                 continue;
 
             const QByteArray &initializer =
@@ -771,7 +765,7 @@ bool ResolveExpression::visit(SimpleNameAST *ast)
             Clone cloner(_context.bindings()->control().data());
 
             for (int n = 0; n < typeItems.size(); ++ n) {
-                FullySpecifiedType newType = cloner.type(typeItems[n].type(), 0);
+                FullySpecifiedType newType = cloner.type(typeItems[n].type(), nullptr);
                 if (n == 0) {
                     item.setType(newType);
                     item.setScope(typeItems[n].scope());
@@ -963,8 +957,8 @@ bool ResolveExpression::visit(ArrayAccessAST *ast)
 
 QList<LookupItem> ResolveExpression::getMembers(ClassOrNamespace *binding, const Name *memberName) const
 {
-    Q_UNUSED(binding);
-    Q_UNUSED(memberName);
+    Q_UNUSED(binding)
+    Q_UNUSED(memberName)
 
     // ### port me
     QList<LookupItem> members;
@@ -1023,7 +1017,7 @@ bool ResolveExpression::visit(MemberAccessAST *ast)
     const QList<LookupItem> baseResults = resolve(ast->base_expression, _scope);
 
     // Evaluate the expression-id that follows the access operator.
-    const Name *memberName = 0;
+    const Name *memberName = nullptr;
     if (ast->member_name)
         memberName = ast->member_name->name;
 
@@ -1040,7 +1034,7 @@ ClassOrNamespace *ResolveExpression::findClass(const FullySpecifiedType &origina
                                                ClassOrNamespace *enclosingBinding) const
 {
     FullySpecifiedType ty = originalTy.simplified();
-    ClassOrNamespace *binding = 0;
+    ClassOrNamespace *binding = nullptr;
 
     if (Class *klass = ty->asClassType()) {
         if (scope->isBlock())
@@ -1119,7 +1113,7 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
                             FullySpecifiedType overloadTy
                                     = instantiate(binding->templateId(), overload);
                             Function *instantiatedFunction = overloadTy->asFunctionType();
-                            Q_ASSERT(instantiatedFunction != 0);
+                            Q_ASSERT(instantiatedFunction != nullptr);
 
                             FullySpecifiedType retTy
                                     = instantiatedFunction->returnType().simplified();
@@ -1168,7 +1162,7 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
                 return binding;
             }
 
-            ClassOrNamespace *enclosingBinding = 0;
+            ClassOrNamespace *enclosingBinding = nullptr;
             if (ClassOrNamespace *binding = r.binding()) {
                 if (binding->instantiationOrigin())
                     enclosingBinding = binding;
@@ -1179,7 +1173,7 @@ ClassOrNamespace *ResolveExpression::baseExpression(const QList<LookupItem> &bas
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 ClassOrNamespace *ResolveExpression::findClassForTemplateParameterInExpressionScope(
@@ -1197,7 +1191,7 @@ ClassOrNamespace *ResolveExpression::findClassForTemplateParameterInExpressionSc
         }
     }
 
-    return 0;
+    return nullptr;
 }
 
 FullySpecifiedType ResolveExpression::instantiate(const Name *className, Symbol *candidate) const
@@ -1219,7 +1213,7 @@ bool ResolveExpression::visit(ObjCMessageExpressionAST *ast)
 
     foreach (const LookupItem &result, receiverResults) {
         FullySpecifiedType ty = result.type().simplified();
-        ClassOrNamespace *binding = 0;
+        ClassOrNamespace *binding = nullptr;
 
         if (ObjCClass *clazz = ty->asObjCClassType()) {
             // static access, e.g.:
