@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -43,26 +38,26 @@
 using namespace CPlusPlus;
 
 TypeOfExpression::TypeOfExpression():
-    m_ast(0),
-    m_scope(0),
+    m_ast(nullptr),
+    m_scope(nullptr),
     m_expandTemplates(false)
 {
 }
 
 void TypeOfExpression::init(Document::Ptr thisDocument, const Snapshot &snapshot,
-                            CreateBindings::Ptr bindings,
+                            QSharedPointer<CreateBindings> bindings,
                             const QSet<const Declaration *> &autoDeclarationsBeingResolved)
 {
     m_thisDocument = thisDocument;
     m_snapshot = snapshot;
-    m_ast = 0;
-    m_scope = 0;
+    m_ast = nullptr;
+    m_scope = nullptr;
     m_lookupContext = LookupContext();
 
     Q_ASSERT(m_bindings.isNull());
     m_bindings = bindings;
     if (m_bindings.isNull())
-        m_bindings = CreateBindings::Ptr(new CreateBindings(thisDocument, snapshot));
+        m_bindings = QSharedPointer<CreateBindings>(new CreateBindings(thisDocument, snapshot));
 
     m_environment.clear();
     m_autoDeclarationsBeingResolved = autoDeclarationsBeingResolved;
@@ -105,7 +100,7 @@ QList<LookupItem> TypeOfExpression::operator()(ExpressionAST *expression,
 
     m_scope = scope;
 
-    m_bindings->addExpressionDocument(document);
+    m_documents.append(document);
     m_lookupContext = LookupContext(document, m_thisDocument, m_snapshot, m_bindings);
     Q_ASSERT(!m_bindings.isNull());
     m_lookupContext.setExpandTemplates(m_expandTemplates);
@@ -122,7 +117,7 @@ QList<LookupItem> TypeOfExpression::reference(ExpressionAST *expression,
 
     m_scope = scope;
 
-    m_bindings->addExpressionDocument(document);
+    m_documents.append(document);
     m_lookupContext = LookupContext(document, m_thisDocument, m_snapshot, m_bindings);
     Q_ASSERT(!m_bindings.isNull());
     m_lookupContext.setExpandTemplates(m_expandTemplates);
@@ -183,7 +178,7 @@ QByteArray TypeOfExpression::preprocessedExpression(const QByteArray &utf8code)
         m_environment = QSharedPointer<Environment>(env);
     }
 
-    Preprocessor preproc(0, m_environment.data());
+    Preprocessor preproc(nullptr, m_environment.data());
     return preproc.run(QLatin1String("<expression>"), utf8code);
 }
 
@@ -192,7 +187,7 @@ namespace CPlusPlus {
 ExpressionAST *extractExpressionAST(Document::Ptr doc)
 {
     if (! doc->translationUnit()->ast())
-        return 0;
+        return nullptr;
 
     return doc->translationUnit()->ast()->asExpression();
 }
