@@ -27,11 +27,8 @@
 
 #include "iplugin.h"
 
-#include <QString>
-
 namespace ExtensionSystem {
 
-class PluginManager;
 class PluginSpec;
 
 namespace Internal {
@@ -40,8 +37,6 @@ class IPluginPrivate
 {
 public:
     PluginSpec *pluginSpec;
-
-    QList<QObject *> addedObjectsInReverseOrder;
 };
 
 } // namespace Internal
