@@ -168,7 +168,6 @@ public:
 
     IconCommandParameter(int keyword, ShapeValueF::Unit unit, ShapeValueF::Origin origin = ShapeValueF::OriginSmart)
         : m_keyword(keyword),
-          m_type(ShapeValue),
           m_unit(unit),
           m_origin(origin)
     {
@@ -424,7 +423,8 @@ void StereotypeDefinitionParser::parseIcon()
             const static QHash<QString, StereotypeIcon::TextAlignment> alignNames = QHash<QString, StereotypeIcon::TextAlignment>()
                     << qMakePair(QString("below"), StereotypeIcon::TextalignBelow)
                     << qMakePair(QString("center"), StereotypeIcon::TextalignCenter)
-                    << qMakePair(QString("none"), StereotypeIcon::TextalignNone);
+                    << qMakePair(QString("none"), StereotypeIcon::TextalignNone)
+                    << qMakePair(QString("top"), StereotypeIcon::TextalignTop);
             parseEnum<StereotypeIcon::TextAlignment>(
                         parseIdentifierProperty(), alignNames, token.sourcePos(),
                         [&](StereotypeIcon::TextAlignment align) { stereotypeIcon.setTextAlignment(align); });
@@ -436,6 +436,10 @@ void StereotypeDefinitionParser::parseIcon()
         case KEYWORD_SHAPE:
             stereotypeIcon.setIconShape(parseIconShape());
             break;
+        case KEYWORD_NAME:
+            stereotypeIcon.setName(parseStringProperty());
+            stereotypeIcon.setHasName(true);
+            break;
         default:
             throwUnknownPropertyError(token);
         }
