@@ -429,18 +429,12 @@ DElement *DiagramController::findElement(const Uid &key, const MDiagram *diagram
 
 bool DiagramController::hasDelegate(const MElement *modelElement, const MDiagram *diagram) const
 {
-    // PERFORM smarter implementation after map is introduced
     return findDelegate(modelElement, diagram) != nullptr;
 }
 
 DElement *DiagramController::findDelegate(const MElement *modelElement, const MDiagram *diagram) const
 {
-    // PERFORM use map to increase performance
-    foreach (DElement *diagramElement, diagram->diagramElements()) {
-        if (diagramElement->modelUid().isValid() && diagramElement->modelUid() == modelElement->uid())
-            return diagramElement;
-    }
-    return nullptr;
+    return diagram->findDelegate(modelElement->uid());
 }
 
 void DiagramController::startUpdateElement(DElement *element, MDiagram *diagram, UpdateAction updateAction)
@@ -487,7 +481,7 @@ DContainer DiagramController::copyElements(const DSelection &diagramSelection, c
     return copiedElements;
 }
 
-void DiagramController::pasteElements(const DContainer &diagramContainer, MDiagram *diagram)
+void DiagramController::pasteElements(const DReferences &diagramContainer, MDiagram *diagram)
 {
     QMT_ASSERT(diagram, return);
 
@@ -582,8 +576,8 @@ void DiagramController::onEndResetModel()
 
 void DiagramController::onBeginUpdateObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
 
     // nothing to do
 }
@@ -614,8 +608,8 @@ void DiagramController::onEndUpdateObject(int row, const MObject *parent)
 
 void DiagramController::onBeginInsertObject(int row, const MObject *owner)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(owner);
+    Q_UNUSED(row)
+    Q_UNUSED(owner)
 }
 
 void DiagramController::onEndInsertObject(int row, const MObject *owner)
@@ -640,14 +634,14 @@ void DiagramController::onBeginRemoveObject(int row, const MObject *parent)
 
 void DiagramController::onEndRemoveObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
 }
 
 void DiagramController::onBeginMoveObject(int formerRow, const MObject *formerOwner)
 {
-    Q_UNUSED(formerRow);
-    Q_UNUSED(formerOwner);
+    Q_UNUSED(formerRow)
+    Q_UNUSED(formerOwner)
 }
 
 void DiagramController::onEndMoveObject(int row, const MObject *owner)
@@ -669,8 +663,8 @@ void DiagramController::onEndMoveObject(int row, const MObject *owner)
 
 void DiagramController::onBeginUpdateRelation(int row, const MObject *owner)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(owner);
+    Q_UNUSED(row)
+    Q_UNUSED(owner)
 
     // nothing to do
 }
@@ -697,14 +691,14 @@ void DiagramController::onBeginRemoveRelation(int row, const MObject *owner)
 
 void DiagramController::onEndRemoveRelation(int row, const MObject *owner)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(owner);
+    Q_UNUSED(row)
+    Q_UNUSED(owner)
 }
 
 void DiagramController::onBeginMoveRelation(int formerRow, const MObject *formerOwner)
 {
-    Q_UNUSED(formerRow);
-    Q_UNUSED(formerOwner);
+    Q_UNUSED(formerRow)
+    Q_UNUSED(formerOwner)
 
     // nothing to do
 }
@@ -764,13 +758,13 @@ void DiagramController::removeObjects(MObject *modelObject)
         DElement *diagramElement = findDelegate(modelObject, diagram);
         if (diagramElement)
             removeElement(diagramElement, diagram);
-        foreach (const Handle<MRelation> &relation, modelObject->relations()) {
+        for (const Handle<MRelation> &relation : modelObject->relations()) {
             DElement *diagramElement = findDelegate(relation.target(), diagram);
             if (diagramElement)
                 removeElement(diagramElement, diagram);
         }
     }
-    foreach (const Handle<MObject> &object, modelObject->children()) {
+    for (const Handle<MObject> &object : modelObject->children()) {
         if (object.hasTarget())
             removeObjects(object.target());
     }
