@@ -80,13 +80,13 @@ public:
 
     void visitMElement(const MElement *element)
     {
-        Q_UNUSED(element);
+        Q_UNUSED(element)
         QMT_CHECK(false);
     }
 
     void visitMObject(const MObject *object)
     {
-        Q_UNUSED(object);
+        Q_UNUSED(object)
         QMT_ASSERT(m_item, return);
         m_item->setEditable(false);
     }
@@ -156,7 +156,7 @@ public:
 
     void visitMRelation(const MRelation *relation)
     {
-        Q_UNUSED(relation);
+        Q_UNUSED(relation)
         QMT_ASSERT(m_item, return);
         m_item->setEditable(false);
         m_item->setData(TreeModel::Relation, TreeModel::RoleItemType);
@@ -216,7 +216,7 @@ public:
 
     void visitMElement(const MElement *element)
     {
-        Q_UNUSED(element);
+        Q_UNUSED(element)
         QMT_CHECK(false);
     }
 
@@ -502,8 +502,8 @@ void TreeModel::onEndResetModel()
 
 void TreeModel::onBeginUpdateObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = UpdateElement;
 }
@@ -536,8 +536,8 @@ void TreeModel::onEndUpdateObject(int row, const MObject *parent)
 
 void TreeModel::onBeginInsertObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = InsertElement;
 }
@@ -569,8 +569,8 @@ void TreeModel::onBeginRemoveObject(int row, const MObject *parent)
 
 void TreeModel::onEndRemoveObject(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == RemoveElement);
     m_busyState = NotBusy;
 }
@@ -602,8 +602,8 @@ void TreeModel::onEndMoveObject(int row, const MObject *owner)
 
 void TreeModel::onBeginUpdateRelation(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = UpdateRelation;
 }
@@ -637,8 +637,8 @@ void TreeModel::onEndUpdateRelation(int row, const MObject *parent)
 
 void TreeModel::onBeginInsertRelation(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == NotBusy);
     m_busyState = InsertRelation;
 }
@@ -668,8 +668,8 @@ void TreeModel::onBeginRemoveRelation(int row, const MObject *parent)
 
 void TreeModel::onEndRemoveRelation(int row, const MObject *parent)
 {
-    Q_UNUSED(row);
-    Q_UNUSED(parent);
+    Q_UNUSED(row)
+    Q_UNUSED(parent)
     QMT_CHECK(m_busyState == RemoveRelation);
     m_busyState = NotBusy;
 }
@@ -699,7 +699,7 @@ void TreeModel::onEndMoveRelation(int row, const MObject *owner)
 
 void TreeModel::onRelationEndChanged(MRelation *relation, MObject *endObject)
 {
-    Q_UNUSED(endObject);
+    Q_UNUSED(endObject)
     QMT_CHECK(m_busyState == NotBusy);
 
     MObject *parent = relation->owner();
@@ -725,8 +725,8 @@ void TreeModel::onRelationEndChanged(MRelation *relation, MObject *endObject)
 
 void TreeModel::onModelDataChanged(const QModelIndex &topleft, const QModelIndex &bottomright)
 {
-    Q_UNUSED(topleft);
-    Q_UNUSED(bottomright);
+    Q_UNUSED(topleft)
+    Q_UNUSED(bottomright)
     // TODO fix editing object name in model tree
     // item->text() no longer returns a simple object name
     // classes contains namespace label
@@ -755,14 +755,14 @@ void TreeModel::createChildren(const MObject *parentObject, ModelItem *parentIte
     m_objectToItemMap.insert(parentObject, parentItem);
     QMT_CHECK(!m_itemToObjectMap.contains(parentItem));
     m_itemToObjectMap.insert(parentItem, parentObject);
-    foreach (const Handle<MObject> &object, parentObject->children()) {
+    for (const Handle<MObject> &object : parentObject->children()) {
         if (object.hasTarget()) {
             ModelItem *item = createItem(object.target());
             parentItem->appendRow(item);
             createChildren(object.target(), item);
         }
     }
-    foreach (const Handle<MRelation> &handle, parentObject->relations()) {
+    for (const Handle<MRelation> &handle : parentObject->relations()) {
         if (handle.hasTarget()) {
             MRelation *relation = handle.target();
             ModelItem *item = createItem(relation);
@@ -780,12 +780,18 @@ void TreeModel::removeObjectFromItemMap(const MObject *object)
     QMT_CHECK(m_itemToObjectMap.contains(item));
     m_itemToObjectMap.remove(item);
     m_objectToItemMap.remove(object);
-    foreach (const Handle<MObject> &child, object->children()) {
+    for (const Handle<MObject> &child : object->children()) {
         if (child.hasTarget())
             removeObjectFromItemMap(child.target());
     }
 }
 
+QString TreeModel::filterLabel(const QString &label) const
+{
+    QString s = label;
+    return s.replace("\n"," ");
+}
+
 QString TreeModel::createObjectLabel(const MObject *object)
 {
     QMT_ASSERT(object, return QString());
@@ -793,16 +799,16 @@ QString TreeModel::createObjectLabel(const MObject *object)
     if (object->name().isEmpty()) {
         if (auto item = dynamic_cast<const MItem *>(object)) {
             if (!item->variety().isEmpty())
-                return QString("[%1]").arg(item->variety());
+                return filterLabel(QString("[%1]").arg(item->variety()));
         }
         return tr("[unnamed]");
     }
 
     if (auto klass = dynamic_cast<const MClass *>(object)) {
         if (!klass->umlNamespace().isEmpty())
-            return QString("%1 [%2]").arg(klass->name()).arg(klass->umlNamespace());
+            return filterLabel(QString("%1 [%2]").arg(klass->name()).arg(klass->umlNamespace()));
     }
-    return object->name();
+    return filterLabel(object->name());
 }
 
 QString TreeModel::createRelationLabel(const MRelation *relation)
@@ -817,7 +823,7 @@ QString TreeModel::createRelationLabel(const MRelation *relation)
     name += " - ";
     if (MObject *endB = m_modelController->findObject(relation->endBUid()))
         name += createObjectLabel(endB);
-    return name;
+    return filterLabel(name);
 }
 
 QIcon TreeModel::createIcon(StereotypeIcon::Element stereotypeIconElement, StyleEngine::ElementType styleElementType,
@@ -825,7 +831,7 @@ QIcon TreeModel::createIcon(StereotypeIcon::Element stereotypeIconElement, Style
 {
     const Style *style = m_styleController->adaptStyle(styleElementType);
     return m_stereotypeController->createIcon(stereotypeIconElement, stereotypes, defaultIconPath, style,
-                                              QSize(48, 48), QMarginsF(3.0, 2.0, 3.0, 4.0));
+                                              QSize(48, 48), QMarginsF(3.0, 2.0, 3.0, 4.0), 3.0);
 }
 
 } // namespace qmt
