@@ -67,10 +67,12 @@
 #include <QQueue>
 #include <QPair>
 
+#include <QtMath>
+
 namespace qmt {
 
 namespace {
-static VoidElementTasks dummyElementTasks;
+VoidElementTasks dummyElementTasks;
 }
 
 class DiagramSceneController::AcceptRelationVisitor : public MVoidConstVisitor
@@ -88,7 +90,7 @@ public:
 
     void visitMObject(const MObject *object) override
     {
-        Q_UNUSED(object);
+        Q_UNUSED(object)
         if (auto connection = dynamic_cast<const MConnection *>(m_relation)) {
             CustomRelation customRelation = m_stereotypeController->findCustomRelation(connection->customRelationId());
             if (!customRelation.isNull()) {
@@ -397,28 +399,23 @@ void DiagramSceneController::dropNewElement(const QString &newElementId, const Q
     } else {
         MPackage *parentPackage = findSuitableParentPackage(topMostElementAtPos, diagram);
         MObject *newObject = nullptr;
-        QString newName;
         if (newElementId == QLatin1String(ELEMENT_TYPE_PACKAGE)) {
             auto package = new MPackage();
-            newName = tr("New Package");
             if (!stereotype.isEmpty())
                 package->setStereotypes({stereotype});
             newObject = package;
         } else if (newElementId == QLatin1String(ELEMENT_TYPE_COMPONENT)) {
             auto component = new MComponent();
-            newName = tr("New Component");
             if (!stereotype.isEmpty())
                 component->setStereotypes({stereotype});
             newObject = component;
         } else if (newElementId == QLatin1String(ELEMENT_TYPE_CLASS)) {
             auto klass = new MClass();
-            newName = tr("New Class");
             if (!stereotype.isEmpty())
                 klass->setStereotypes({stereotype});
             newObject = klass;
         } else if (newElementId == QLatin1String(ELEMENT_TYPE_ITEM)) {
             auto item = new MItem();
-            newName = tr("New Item");
             if (!stereotype.isEmpty()) {
                 item->setVariety(stereotype);
                 item->setVarietyEditable(false);
@@ -426,9 +423,7 @@ void DiagramSceneController::dropNewElement(const QString &newElementId, const Q
             newObject = item;
         }
         if (newObject) {
-            if (!name.isEmpty())
-                newName = tr("New %1").arg(name);
-            newObject->setName(newName);
+            newObject->setName(name);
             dropNewModelElement(newObject, parentPackage, pos, diagram);
         }
     }
@@ -445,6 +440,49 @@ void DiagramSceneController::dropNewModelElement(MObject *modelObject, MPackage
         emit newElementCreated(element, diagram);
 }
 
+void DiagramSceneController::addRelatedElements(const DSelection &selection, MDiagram *diagram)
+{
+    m_diagramController->undoController()->beginMergeSequence(tr("Add Related Element"));
+    foreach (const DSelection::Index &index, selection.indices()) {
+        DElement *delement = m_diagramController->findElement(index.elementKey(), diagram);
+        QMT_ASSERT(delement, return);
+        DObject *dobject = dynamic_cast<DObject *>(delement);
+        if (dobject && dobject->modelUid().isValid()) {
+            MObject *mobject = m_modelController->findElement<MObject>(delement->modelUid());
+            if (mobject) {
+                qreal dAngle = 360.0 / 11.5;
+                qreal dRadius = 100.0;
+                const QList<MRelation *> relations = m_modelController->findRelationsOfObject(mobject);
+                int count = 0;
+                for (MRelation *relation : relations) {
+                    if (relation->endAUid() != mobject->uid() || relation->endBUid() != mobject->uid())
+                        ++count;
+                }
+                if (count <= 12) {
+                    dAngle = 360.0 / 12.0;
+                    dRadius = 0.0;
+                }
+                qreal radius = 200.0;
+                qreal angle = 0.0;
+                for (MRelation *relation : relations) {
+                    QPointF pos(dobject->pos());
+                    pos += QPointF(radius * sin(angle / 180 * M_PI), -radius * cos(angle / 180 * M_PI));
+                    bool added = false;
+                    if (relation->endAUid() != mobject->uid())
+                        added = addModelElement(relation->endAUid(), pos, diagram) != nullptr;
+                    else if (relation->endBUid() != mobject->uid())
+                        added = addModelElement(relation->endBUid(), pos, diagram) != nullptr;
+                    if (added) {
+                        radius += dRadius / (360.0 / dAngle);
+                        angle += dAngle;
+                    }
+                }
+            }
+        }
+    }
+    m_diagramController->undoController()->endMergeSequence();
+}
+
 MPackage *DiagramSceneController::findSuitableParentPackage(DElement *topmostDiagramElement, MDiagram *diagram)
 {
     MPackage *parentPackage = nullptr;
@@ -465,7 +503,7 @@ MPackage *DiagramSceneController::findSuitableParentPackage(DElement *topmostDia
 MDiagram *DiagramSceneController::findDiagramBySearchId(MPackage *package, const QString &diagramName)
 {
     QString diagramSearchId = NameController::calcElementNameSearchId(diagramName);
-    foreach (const Handle<MObject> &handle, package->children()) {
+    for (const Handle<MObject> &handle : package->children()) {
         if (handle.hasTarget()) {
             if (auto diagram = dynamic_cast<MDiagram *>(handle.target())) {
                 if (NameController::calcElementNameSearchId(diagram->name()) == diagramSearchId)
@@ -478,7 +516,7 @@ MDiagram *DiagramSceneController::findDiagramBySearchId(MPackage *package, const
 
 namespace {
 
-static QPointF alignObjectLeft(DObject *object, DObject *otherObject)
+QPointF alignObjectLeft(DObject *object, DObject *otherObject)
 {
     qreal left = object->pos().x() + object->rect().left();
     QPointF pos = otherObject->pos();
@@ -488,7 +526,7 @@ static QPointF alignObjectLeft(DObject *object, DObject *otherObject)
     return pos;
 }
 
-static QPointF alignObjectRight(DObject *object, DObject *otherObject)
+QPointF alignObjectRight(DObject *object, DObject *otherObject)
 {
     qreal right = object->pos().x() + object->rect().right();
     QPointF pos = otherObject->pos();
@@ -498,7 +536,7 @@ static QPointF alignObjectRight(DObject *object, DObject *otherObject)
     return pos;
 }
 
-static QPointF alignObjectHCenter(DObject *object, DObject *otherObject)
+QPointF alignObjectHCenter(DObject *object, DObject *otherObject)
 {
     qreal center = object->pos().x();
     QPointF pos = otherObject->pos();
@@ -508,7 +546,7 @@ static QPointF alignObjectHCenter(DObject *object, DObject *otherObject)
     return pos;
 }
 
-static QPointF alignObjectTop(DObject *object, DObject *otherObject)
+QPointF alignObjectTop(DObject *object, DObject *otherObject)
 {
     qreal top = object->pos().y() + object->rect().top();
     QPointF pos = otherObject->pos();
@@ -518,7 +556,7 @@ static QPointF alignObjectTop(DObject *object, DObject *otherObject)
     return pos;
 }
 
-static QPointF alignObjectBottom(DObject *object, DObject *otherObject)
+QPointF alignObjectBottom(DObject *object, DObject *otherObject)
 {
     qreal bottom = object->pos().y() + object->rect().bottom();
     QPointF pos = otherObject->pos();
@@ -528,7 +566,7 @@ static QPointF alignObjectBottom(DObject *object, DObject *otherObject)
     return pos;
 }
 
-static QPointF alignObjectVCenter(DObject *object, DObject *otherObject)
+QPointF alignObjectVCenter(DObject *object, DObject *otherObject)
 {
     qreal center = object->pos().y();
     QPointF pos = otherObject->pos();
@@ -538,7 +576,7 @@ static QPointF alignObjectVCenter(DObject *object, DObject *otherObject)
     return pos;
 }
 
-static QRectF alignObjectWidth(DObject *object, const QSizeF &size)
+QRectF alignObjectWidth(DObject *object, const QSizeF &size)
 {
     QRectF rect = object->rect();
     rect.setX(-size.width() / 2.0);
@@ -546,7 +584,7 @@ static QRectF alignObjectWidth(DObject *object, const QSizeF &size)
     return rect;
 }
 
-static QRectF alignObjectHeight(DObject *object, const QSizeF &size)
+QRectF alignObjectHeight(DObject *object, const QSizeF &size)
 {
     QRectF rect = object->rect();
     rect.setY(-size.height() / 2.0);
@@ -554,9 +592,9 @@ static QRectF alignObjectHeight(DObject *object, const QSizeF &size)
     return rect;
 }
 
-static QRectF alignObjectSize(DObject *object, const QSizeF &size)
+QRectF alignObjectSize(DObject *object, const QSizeF &size)
 {
-    Q_UNUSED(object);
+    Q_UNUSED(object)
 
     QRectF rect;
     rect.setX(-size.width() / 2.0);
@@ -616,6 +654,118 @@ void DiagramSceneController::alignSize(DObject *object, const DSelection &select
     alignSize(object, selection, minimumSize, alignObjectSize, diagram);
 }
 
+void DiagramSceneController::alignHCenterDistance(const DSelection &selection, MDiagram *diagram)
+{
+    QList<DObject *> sortedObjects = collectObjects(selection, diagram);
+    if (sortedObjects.length() > 2) {
+        std::sort(sortedObjects.begin(), sortedObjects.end(), [](const DObject *lhs, const DObject *rhs) {
+            return lhs->pos().x() < rhs->pos().x();
+        });
+        int n = sortedObjects.length() - 1;
+        DObject *leftObject = sortedObjects.at(0);
+        DObject *rightObject = sortedObjects.at(n);
+        double distance = rightObject->pos().x() - leftObject->pos().x();
+        double step = distance / n;
+        double startX = leftObject->pos().x();
+        for (int i = 1; i < n; ++i) {
+            DObject *selectedObject = sortedObjects.at(i);
+            QPointF newPos = selectedObject->pos();
+            newPos.setX(startX + i * step);
+            if (newPos != selectedObject->pos()) {
+                m_diagramController->startUpdateElement(selectedObject, diagram, DiagramController::UpdateGeometry);
+                selectedObject->setPos(newPos);
+                m_diagramController->finishUpdateElement(selectedObject, diagram, false);
+            }
+        }
+    }
+}
+
+void DiagramSceneController::alignVCenterDistance(const DSelection &selection, MDiagram *diagram)
+{
+    QList<DObject *> sortedObjects = collectObjects(selection, diagram);
+    if (sortedObjects.length() > 2) {
+        std::sort(sortedObjects.begin(), sortedObjects.end(), [](const DObject *lhs, const DObject *rhs) {
+            return lhs->pos().y() < rhs->pos().y();
+        });
+        int n = sortedObjects.length() - 1;
+        DObject *topObject = sortedObjects.at(0);
+        DObject *bottomObject = sortedObjects.at(n);
+        double distance = bottomObject->pos().y() - topObject->pos().y();
+        double step = distance / n;
+        double startY = topObject->pos().y();
+        for (int i = 1; i < n; ++i) {
+            DObject *selectedObject = sortedObjects.at(i);
+            QPointF newPos = selectedObject->pos();
+            newPos.setY(startY + i * step);
+            if (newPos != selectedObject->pos()) {
+                m_diagramController->startUpdateElement(selectedObject, diagram, DiagramController::UpdateGeometry);
+                selectedObject->setPos(newPos);
+                m_diagramController->finishUpdateElement(selectedObject, diagram, false);
+            }
+        }
+    }
+}
+
+void DiagramSceneController::alignHBorderDistance(const DSelection &selection, MDiagram *diagram)
+{
+    QList<DObject *> sortedObjects = collectObjects(selection, diagram);
+    if (sortedObjects.length() > 2) {
+        std::sort(sortedObjects.begin(), sortedObjects.end(), [](const DObject *lhs, const DObject *rhs) {
+            return lhs->pos().x() < rhs->pos().x();
+        });
+        int n = sortedObjects.length() - 1;
+        DObject *leftObject = sortedObjects.at(0);
+        DObject *rightObject = sortedObjects.at(n);
+        double space = rightObject->pos().x() + rightObject->rect().left() - (leftObject->pos().x() + leftObject->rect().right());
+        for (int i = 1; i < n; ++i)
+            space -= sortedObjects.at(i)->rect().width();
+        double step = space / n;
+        double x = leftObject->pos().x() + leftObject->rect().right();
+        for (int i = 1 ; i < n; ++i) {
+            DObject *selectedObject = sortedObjects.at(i);
+            QPointF newPos = selectedObject->pos();
+            x += step;
+            newPos.setX(x - selectedObject->rect().left());
+            x += selectedObject->rect().width();
+            if (newPos != selectedObject->pos()) {
+                m_diagramController->startUpdateElement(selectedObject, diagram, DiagramController::UpdateGeometry);
+                selectedObject->setPos(newPos);
+                m_diagramController->finishUpdateElement(selectedObject, diagram, false);
+            }
+        }
+    }
+}
+
+void DiagramSceneController::alignVBorderDistance(const DSelection &selection, MDiagram *diagram)
+{
+    QList<DObject *> sortedObjects = collectObjects(selection, diagram);
+    if (sortedObjects.length() > 2) {
+        std::sort(sortedObjects.begin(), sortedObjects.end(), [](const DObject *lhs, const DObject *rhs) {
+            return lhs->pos().y() < rhs->pos().y();
+        });
+        int n = sortedObjects.length() - 1;
+        DObject *topObject = sortedObjects.at(0);
+        DObject *bottomObject = sortedObjects.at(n);
+        double space = bottomObject->pos().y() + bottomObject->rect().top() - (topObject->pos().y() + topObject->rect().bottom());
+        for (int i = 1; i < n; ++i)
+            space -= sortedObjects.at(i)->rect().height();
+        double step = space / n;
+        double y = topObject->pos().y() + topObject->rect().bottom();
+        for (int i = 1 ; i < n; ++i) {
+            DObject *selectedObject = sortedObjects.at(i);
+            QPointF newPos = selectedObject->pos();
+            y += step;
+            newPos.setY(y - selectedObject->rect().top());
+            y += selectedObject->rect().height();
+            if (newPos != selectedObject->pos()) {
+                m_diagramController->startUpdateElement(selectedObject, diagram, DiagramController::UpdateGeometry);
+                selectedObject->setPos(newPos);
+                m_diagramController->finishUpdateElement(selectedObject, diagram, false);
+            }
+        }
+    }
+}
+
 void DiagramSceneController::alignPosition(DObject *object, const DSelection &selection,
                                            QPointF (*aligner)(DObject *, DObject *), MDiagram *diagram)
 {
@@ -669,6 +819,17 @@ void DiagramSceneController::alignOnRaster(DElement *element, MDiagram *diagram)
     element->accept(&visitor);
 }
 
+QList<DObject *> DiagramSceneController::collectObjects(const DSelection &selection, MDiagram *diagram)
+{
+    QList<DObject *> list;
+    foreach (const DSelection::Index &index, selection.indices()) {
+        DObject *object = m_diagramController->findElement<DObject>(index.elementKey(), diagram);
+        if (object)
+            list.append(object);
+    }
+    return list;
+}
+
 DElement *DiagramSceneController::addModelElement(const Uid &modelElementKey, const QPointF &pos, MDiagram *diagram)
 {
     DElement *element = nullptr;
@@ -706,7 +867,7 @@ DObject *DiagramSceneController::addObject(MObject *modelObject, const QPointF &
             if (dobject) {
                 MObject *mobject = m_modelController->findObject(dobject->modelUid());
                 if (mobject) {
-                    foreach (const Handle<MRelation> &handle, mobject->relations()) {
+                    for (const Handle<MRelation> &handle : mobject->relations()) {
                         if (handle.hasTarget()
                                 && ((handle.target()->endAUid() == modelObject->uid()
                                      && handle.target()->endBUid() == mobject->uid())
@@ -715,7 +876,7 @@ DObject *DiagramSceneController::addObject(MObject *modelObject, const QPointF &
                             addRelation(handle.target(), QList<QPointF>(), diagram);
                         }
                     }
-                    foreach (const Handle<MRelation> &handle, modelObject->relations()) {
+                    for (const Handle<MRelation> &handle : modelObject->relations()) {
                         if (handle.hasTarget()
                                 && ((handle.target()->endAUid() == modelObject->uid()
                                      && handle.target()->endBUid() == mobject->uid())
@@ -730,7 +891,7 @@ DObject *DiagramSceneController::addObject(MObject *modelObject, const QPointF &
     }
 
     // add all self relations
-    foreach (const Handle<MRelation> &handle, modelObject->relations()) {
+    for (const Handle<MRelation> &handle : modelObject->relations()) {
         if (handle.hasTarget ()
                 && handle.target()->endAUid() == modelObject->uid()
                 && handle.target()->endBUid() == modelObject->uid()) {
