@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -77,8 +72,8 @@ void QmlOutputParser::processOutput(const QString &output)
             static QString waitingForConnection = QLatin1String(Constants::STR_WAITING_FOR_CONNECTION);
             static QString unableToListen = QLatin1String(Constants::STR_UNABLE_TO_LISTEN);
             static QString debuggingNotEnabled = QLatin1String(Constants::STR_IGNORING_DEBUGGER);
-            static QString debuggingNotEnabled2 = QLatin1String(Constants::STR_IGNORING_DEBUGGER2);
             static QString connectionEstablished = QLatin1String(Constants::STR_CONNECTION_ESTABLISHED);
+            static QString connectingToSocket = QLatin1String(Constants::STR_CONNECTING_TO_SOCKET);
 
             if (status.startsWith(waitingForConnection)) {
                 status.remove(0, waitingForConnection.size()); // chop of 'Waiting for connection '
@@ -89,17 +84,19 @@ void QmlOutputParser::processOutput(const QString &output)
                     bool canConvert;
                     quint16 port = waitingTcp.cap(1).toUShort(&canConvert);
                     if (canConvert)
-                        emit waitingForConnectionOnPort(port);
+                        emit waitingForConnectionOnPort(Utils::Port(port));
                     continue;
                 }
             } else if (status.startsWith(unableToListen)) {
                 //: Error message shown after 'Could not connect ... debugger:"
                 emit errorMessage(tr("The port seems to be in use."));
-            } else if (status.startsWith(debuggingNotEnabled) || status.startsWith(debuggingNotEnabled2)) {
+            } else if (status.startsWith(debuggingNotEnabled)) {
                 //: Error message shown after 'Could not connect ... debugger:"
                 emit errorMessage(tr("The application is not set up for QML/JS debugging."));
             } else if (status.startsWith(connectionEstablished)) {
                 emit connectionEstablishedMessage();
+            } else if (status.startsWith(connectingToSocket)) {
+                emit connectingToSocketMessage();
             } else {
                 emit unknownMessage(status);
             }
