@@ -29,6 +29,8 @@
 #include "qmljsdocument.h"
 #include "qmljsmodelmanagerinterface.h"
 
+#include <utils/algorithm.h>
+
 using namespace LanguageUtils;
 using namespace QmlJS;
 using namespace QmlJS::AST;
@@ -51,9 +53,9 @@ using namespace QmlJS::AST;
 
 Bind::Bind(Document *doc, QList<DiagnosticMessage> *messages, bool isJsLibrary, const QList<ImportInfo> &jsImports)
     : _doc(doc),
-      _currentObjectValue(0),
-      _idEnvironment(0),
-      _rootObjectValue(0),
+      _currentObjectValue(nullptr),
+      _idEnvironment(nullptr),
+      _rootObjectValue(nullptr),
       _isJsLibrary(isJsLibrary),
       _imports(jsImports),
       _diagnosticMessages(messages)
@@ -132,7 +134,7 @@ ObjectValue *Bind::switchObjectValue(ObjectValue *newObjectValue)
 
 ObjectValue *Bind::bindObject(UiQualifiedId *qualifiedTypeNameId, UiObjectInitializer *initializer)
 {
-    ObjectValue *parentObjectValue = 0;
+    ObjectValue *parentObjectValue = nullptr;
 
     // normal component instance
     ASTObjectValue *objectValue = new ASTObjectValue(qualifiedTypeNameId, initializer, _doc, &_valueOwner);
@@ -167,17 +169,28 @@ void Bind::accept(Node *node)
 
 bool Bind::visit(AST::UiProgram *)
 {
-    _idEnvironment = _valueOwner.newObject(/*prototype =*/ 0);
+    _idEnvironment = _valueOwner.newObject(/*prototype =*/ nullptr);
     return true;
 }
 
 bool Bind::visit(AST::Program *)
 {
-    _currentObjectValue = _valueOwner.newObject(/*prototype =*/ 0);
+    _currentObjectValue = _valueOwner.newObject(/*prototype =*/ nullptr);
     _rootObjectValue = _currentObjectValue;
     return true;
 }
 
+void Bind::endVisit(UiProgram *)
+{
+    if (_doc->language() == Dialect::QmlQbs) {
+        static const QString qbsBaseImport = QStringLiteral("qbs");
+        static auto isQbsBaseImport = [] (const ImportInfo &ii) {
+            return ii.name() == qbsBaseImport; };
+        if (!Utils::anyOf(_imports, isQbsBaseImport))
+            _imports += ImportInfo::moduleImport(qbsBaseImport, ComponentVersion(), QString());
+    }
+}
+
 bool Bind::visit(UiImport *ast)
 {
     ComponentVersion version;
@@ -202,13 +215,9 @@ bool Bind::visit(UiImport *ast)
             const QString importStr = import.name() + importId;
             if (ModelManagerInterface::instance()) {
                 QmlLanguageBundles langBundles = ModelManagerInterface::instance()->extendedBundles();
-                QmlBundle qq1 = langBundles.bundleForLanguage(Dialect::QmlQtQuick1);
                 QmlBundle qq2 = langBundles.bundleForLanguage(Dialect::QmlQtQuick2);
-                bool isQQ1 = qq1.supportedImports().contains(importStr);
                 bool isQQ2 = qq2.supportedImports().contains(importStr);
-                if (isQQ1 && ! isQQ2)
-                    _doc->setLanguage(Dialect::QmlQtQuick1);
-                if (isQQ2 && ! isQQ1)
+                if (isQQ2)
                     _doc->setLanguage(Dialect::QmlQtQuick2);
             }
         }
@@ -227,7 +236,7 @@ bool Bind::visit(UiPublicMember *ast)
     const Block *block = AST::cast<const Block*>(ast->statement);
     if (block) {
         // build block scope
-        ObjectValue *blockScope = _valueOwner.newObject(/*prototype=*/0);
+        ObjectValue *blockScope = _valueOwner.newObject(/*prototype=*/nullptr);
         _attachedJSScopes.insert(ast, blockScope); // associated with the UiPublicMember, not with the block
         ObjectValue *parent = switchObjectValue(blockScope);
         accept(ast->statement);
@@ -250,7 +259,7 @@ bool Bind::visit(UiObjectDefinition *ast)
         _qmlObjects.insert(ast, value);
     } else {
         _groupedPropertyBindings.insert(ast);
-        ObjectValue *oldObjectValue = switchObjectValue(0);
+        ObjectValue *oldObjectValue = switchObjectValue(nullptr);
         accept(ast->initializer);
         switchObjectValue(oldObjectValue);
     }
@@ -280,7 +289,7 @@ bool Bind::visit(UiScriptBinding *ast)
     const Block *block = AST::cast<const Block*>(ast->statement);
     if (block) {
         // build block scope
-        ObjectValue *blockScope = _valueOwner.newObject(/*prototype=*/0);
+        ObjectValue *blockScope = _valueOwner.newObject(/*prototype=*/nullptr);
         _attachedJSScopes.insert(ast, blockScope); // associated with the UiScriptBinding, not with the block
         ObjectValue *parent = switchObjectValue(blockScope);
         accept(ast->statement);
@@ -297,14 +306,14 @@ bool Bind::visit(UiArrayBinding *)
     return true;
 }
 
-bool Bind::visit(VariableDeclaration *ast)
+bool Bind::visit(PatternElement *ast)
 {
-    if (ast->name.isEmpty())
+    if (ast->bindingIdentifier.isEmpty() || !ast->isVariableDeclaration())
         return false;
 
     ASTVariableReference *ref = new ASTVariableReference(ast, _doc, &_valueOwner);
     if (_currentObjectValue)
-        _currentObjectValue->setMember(ast->name.toString(), ref);
+        _currentObjectValue->setMember(ast->bindingIdentifier, ref);
     return true;
 }
 
@@ -319,7 +328,7 @@ bool Bind::visit(FunctionExpression *ast)
         _currentObjectValue->setMember(ast->name.toString(), function);
 
     // build function scope
-    ObjectValue *functionScope = _valueOwner.newObject(/*prototype=*/0);
+    ObjectValue *functionScope = _valueOwner.newObject(/*prototype=*/nullptr);
     _attachedJSScopes.insert(ast, functionScope);
     ObjectValue *parent = switchObjectValue(functionScope);
 
@@ -328,15 +337,15 @@ bool Bind::visit(FunctionExpression *ast)
 
     // 1. Function formal arguments
     for (FormalParameterList *it = ast->formals; it; it = it->next) {
-        if (!it->name.isEmpty())
-            functionScope->setMember(it->name.toString(), _valueOwner.unknownValue());
+        if (!it->element->bindingIdentifier.isEmpty())
+            functionScope->setMember(it->element->bindingIdentifier, _valueOwner.unknownValue());
     }
 
     // 2. Functions defined inside the function body
     // ### TODO, currently covered by the accept(body)
 
     // 3. Arguments object
-    ObjectValue *arguments = _valueOwner.newObject(/*prototype=*/0);
+    ObjectValue *arguments = _valueOwner.newObject(/*prototype=*/nullptr);
     arguments->setMember(QLatin1String("callee"), function);
     arguments->setMember(QLatin1String("length"), _valueOwner.numberValue());
     functionScope->setMember(QLatin1String("arguments"), arguments);
