@@ -99,31 +99,31 @@ class LookupMember: public MemberProcessor
 
 public:
     LookupMember(const QString &name)
-        : m_name(name), m_value(0) {}
+        : m_name(name), m_value(nullptr) {}
 
     const Value *value() const { return m_value; }
 
-    virtual bool processProperty(const QString &name, const Value *value, const PropertyInfo &)
+    bool processProperty(const QString &name, const Value *value, const PropertyInfo &) override
     {
         return process(name, value);
     }
 
-    virtual bool processEnumerator(const QString &name, const Value *value)
+    bool processEnumerator(const QString &name, const Value *value) override
     {
         return process(name, value);
     }
 
-    virtual bool processSignal(const QString &name, const Value *value)
+    bool processSignal(const QString &name, const Value *value) override
     {
         return process(name, value);
     }
 
-    virtual bool processSlot(const QString &name, const Value *value)
+    bool processSlot(const QString &name, const Value *value) override
     {
         return process(name, value);
     }
 
-    virtual bool processGeneratedSlot(const QString &name, const Value *value)
+    bool processGeneratedSlot(const QString &name, const Value *value) override
     {
         return process(name, value);
     }
@@ -204,6 +204,24 @@ QString PropertyInfo::toString() const
     return list.join('|');
 }
 
+static QList<CustomImportsProvider *> g_customImportProviders;
+
+CustomImportsProvider::CustomImportsProvider(QObject *parent)
+    : QObject(parent)
+{
+    g_customImportProviders.append(this);
+}
+
+CustomImportsProvider::~CustomImportsProvider()
+{
+    g_customImportProviders.removeOne(this);
+}
+
+const QList<CustomImportsProvider *> CustomImportsProvider::allProviders()
+{
+    return g_customImportProviders;
+}
+
 } // namespace QmlJS
 
 CppComponentValue::CppComponentValue(FakeMetaObject::ConstPtr metaObject, const QString &className,
@@ -272,7 +290,7 @@ void CppComponentValue::processMembers(MemberProcessor *processor) const
         signatures->reserve(m_metaObject->methodCount());
         for (int index = 0; index < m_metaObject->methodCount(); ++index)
             signatures->append(new MetaFunction(m_metaObject->method(index), valueOwner()));
-        if (!m_metaSignatures.testAndSetOrdered(0, signatures)) {
+        if (!m_metaSignatures.testAndSetOrdered(nullptr, signatures)) {
             delete signatures;
             signatures = m_metaSignatures.load();
         }
@@ -482,7 +500,7 @@ FakeMetaEnum CppComponentValue::getEnum(const QString &typeName, const CppCompon
         }
     }
     if (foundInScope)
-        *foundInScope = 0;
+        *foundInScope = nullptr;
     return FakeMetaEnum();
 }
 
@@ -496,8 +514,8 @@ const QmlEnumValue *CppComponentValue::getEnumValue(const QString &typeName, con
         }
     }
     if (foundInScope)
-        *foundInScope = 0;
-    return 0;
+        *foundInScope = nullptr;
+    return nullptr;
 }
 
 const ObjectValue *CppComponentValue::signalScope(const QString &signalName) const
@@ -516,7 +534,7 @@ const ObjectValue *CppComponentValue::signalScope(const QString &signalName) con
             const QStringList &parameterTypes = method.parameterTypes();
             QTC_ASSERT(parameterNames.size() == parameterTypes.size(), continue);
 
-            ObjectValue *scope = valueOwner()->newObject(/*prototype=*/0);
+            ObjectValue *scope = valueOwner()->newObject(/*prototype=*/nullptr);
             for (int i = 0; i < parameterNames.size(); ++i) {
                 const QString &name = parameterNames.at(i);
                 const QString &type = parameterTypes.at(i);
@@ -526,7 +544,7 @@ const ObjectValue *CppComponentValue::signalScope(const QString &signalName) con
             }
             scopes->insert(generatedSlotName(method.methodName()), scope);
         }
-        if (!m_signalScopes.testAndSetOrdered(0, scopes)) {
+        if (!m_signalScopes.testAndSetOrdered(nullptr, scopes)) {
             delete scopes;
             scopes = m_signalScopes.load();
         }
@@ -690,137 +708,137 @@ bool Value::getSourceLocation(QString *, int *, int *) const
 
 const NullValue *Value::asNullValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const UndefinedValue *Value::asUndefinedValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const UnknownValue *Value::asUnknownValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const NumberValue *Value::asNumberValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const IntValue *Value::asIntValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const RealValue *Value::asRealValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const BooleanValue *Value::asBooleanValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const StringValue *Value::asStringValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const UrlValue *Value::asUrlValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ObjectValue *Value::asObjectValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const FunctionValue *Value::asFunctionValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const Reference *Value::asReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ColorValue *Value::asColorValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const AnchorLineValue *Value::asAnchorLineValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const CppComponentValue *Value::asCppComponentValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTObjectValue *Value::asAstObjectValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const QmlEnumValue *Value::asQmlEnumValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const QmlPrototypeReference *Value::asQmlPrototypeReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTPropertyReference *Value::asAstPropertyReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTVariableReference *Value::asAstVariableReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const Internal::QtObjectPrototypeReference *Value::asQtObjectPrototypeReference() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTSignal *Value::asAstSignal() const
 {
-    return 0;
+    return nullptr;
 }
 
 const ASTFunctionValue *Value::asAstFunctionValue() const
 {
-    return 0;
+    return nullptr;
 }
 
 const Function *Value::asFunction() const
 {
-    return 0;
+    return nullptr;
 }
 
 const MetaFunction *Value::asMetaFunction() const
 {
-    return 0;
+    return nullptr;
 }
 
 const JSImportScope *Value::asJSImportScope() const
 {
-    return 0;
+    return nullptr;
 }
 
 const TypeScope *Value::asTypeScope() const
 {
-    return 0;
+    return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -985,7 +1003,7 @@ bool MemberProcessor::processGeneratedSlot(const QString &, const Value *)
 
 ObjectValue::ObjectValue(ValueOwner *valueOwner, const QString &originId)
     : m_valueOwner(valueOwner), m_originId(originId),
-      _prototype(0)
+      _prototype(nullptr)
 {
     valueOwner->registerValue(this);
 }
@@ -1034,6 +1052,11 @@ void ObjectValue::setMember(const QString &name, const Value *value)
     m_members[name].value = value;
 }
 
+void ObjectValue::setMember(const QStringRef &name, const Value *value)
+{
+    m_members[name.toString()].value = value;
+}
+
 void ObjectValue::setPropertyInfo(const QString &name, const PropertyInfo &propertyInfo)
 {
     m_members[name].propertyInfo = propertyInfo;
@@ -1075,11 +1098,7 @@ bool ObjectValue::checkPrototype(const ObjectValue *, QSet<const ObjectValue *>
 
 void ObjectValue::processMembers(MemberProcessor *processor) const
 {
-    QHashIterator<QString, PropertyData> it(m_members);
-
-    while (it.hasNext()) {
-        it.next();
-
+    for (auto it = m_members.cbegin(), end = m_members.cend(); it != end; ++it) {
         if (! processor->processProperty(it.key(), it.value().value, it.value().propertyInfo))
             break;
     }
@@ -1103,7 +1122,7 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
         }
     }
 
-    const ObjectValue *prototypeObject = 0;
+    const ObjectValue *prototypeObject = nullptr;
 
     if (examinePrototypes && context) {
         PrototypeIterator iter(this, context);
@@ -1116,13 +1135,13 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
     }
 
     if (foundInObject)
-        *foundInObject = 0;
+        *foundInObject = nullptr;
 
-    return 0;
+    return nullptr;
 }
 
 PrototypeIterator::PrototypeIterator(const ObjectValue *start, const Context *context)
-    : m_current(0)
+    : m_current(nullptr)
     , m_next(start)
     , m_context(context)
     , m_error(NoError)
@@ -1132,7 +1151,7 @@ PrototypeIterator::PrototypeIterator(const ObjectValue *start, const Context *co
 }
 
 PrototypeIterator::PrototypeIterator(const ObjectValue *start, const ContextPtr &context)
-    : m_current(0)
+    : m_current(nullptr)
     , m_next(start)
     , m_context(context.data())
     , m_error(NoError)
@@ -1160,7 +1179,7 @@ bool PrototypeIterator::hasNext()
     }
     if (m_prototypes.contains(m_next)) {
         m_error = CycleError;
-        m_next = 0;
+        m_next = nullptr;
         return false;
     }
     return true;
@@ -1171,17 +1190,17 @@ const ObjectValue *PrototypeIterator::next()
     if (hasNext()) {
         m_current = m_next;
         m_prototypes += m_next;
-        m_next = 0;
+        m_next = nullptr;
         return m_current;
     }
-    return 0;
+    return nullptr;
 }
 
 const ObjectValue *PrototypeIterator::peekNext()
 {
     if (hasNext())
         return m_next;
-    return 0;
+    return nullptr;
 }
 
 PrototypeIterator::Error PrototypeIterator::error() const
@@ -1250,7 +1269,7 @@ void FunctionValue::accept(ValueVisitor *visitor) const
 
 Function::Function(ValueOwner *valueOwner)
     : FunctionValue(valueOwner)
-    , m_returnValue(0)
+    , m_returnValue(nullptr)
     , m_optionalNamedArgumentCount(0)
     , m_isVariadic(false)
 {
@@ -1345,7 +1364,7 @@ CppQmlTypesLoader::BuiltinObjects CppQmlTypesLoader::loadQmlTypes(const QFileInf
             file.close();
 
 
-            parseQmlTypeDescriptions(contents, &newObjects, 0, &newDependencies, &error, &warning,
+            parseQmlTypeDescriptions(contents, &newObjects, nullptr, &newDependencies, &error, &warning,
                                      qmlTypeFile.absoluteFilePath());
         } else {
             error = file.errorString();
@@ -1398,7 +1417,7 @@ void CppQmlTypesLoader::parseQmlTypeDescriptions(const QByteArray &contents,
 }
 
 CppQmlTypes::CppQmlTypes(ValueOwner *valueOwner)
-    : m_cppContextProperties(0)
+    : m_cppContextProperties(nullptr)
     , m_valueOwner(valueOwner)
 
 {
@@ -1584,13 +1603,13 @@ const ObjectValue *CppQmlTypes::cppContextProperties() const
 
 
 ConvertToNumber::ConvertToNumber(ValueOwner *valueOwner)
-    : m_valueOwner(valueOwner), m_result(0)
+    : m_valueOwner(valueOwner), m_result(nullptr)
 {
 }
 
 const Value *ConvertToNumber::operator()(const Value *value)
 {
-    const Value *previousValue = switchResult(0);
+    const Value *previousValue = switchResult(nullptr);
 
     if (value)
         value->accept(this);
@@ -1647,13 +1666,13 @@ void ConvertToNumber::visit(const FunctionValue *object)
 }
 
 ConvertToString::ConvertToString(ValueOwner *valueOwner)
-    : m_valueOwner(valueOwner), m_result(0)
+    : m_valueOwner(valueOwner), m_result(nullptr)
 {
 }
 
 const Value *ConvertToString::operator()(const Value *value)
 {
-    const Value *previousValue = switchResult(0);
+    const Value *previousValue = switchResult(nullptr);
 
     if (value)
         value->accept(this);
@@ -1710,13 +1729,13 @@ void ConvertToString::visit(const FunctionValue *object)
 }
 
 ConvertToObject::ConvertToObject(ValueOwner *valueOwner)
-    : m_valueOwner(valueOwner), m_result(0)
+    : m_valueOwner(valueOwner), m_result(nullptr)
 {
 }
 
 const Value *ConvertToObject::operator()(const Value *value)
 {
-    const Value *previousValue = switchResult(0);
+    const Value *previousValue = switchResult(nullptr);
 
     if (value)
         value->accept(this);
@@ -1832,13 +1851,13 @@ ASTObjectValue::ASTObjectValue(UiQualifiedId *typeName,
                                const Document *doc,
                                ValueOwner *valueOwner)
     : ObjectValue(valueOwner, doc->importId()),
-      m_typeName(typeName), m_initializer(initializer), m_doc(doc), m_defaultPropertyRef(0)
+      m_typeName(typeName), m_initializer(initializer), m_doc(doc), m_defaultPropertyRef(nullptr)
 {
     if (m_initializer) {
         for (UiObjectMemberList *it = m_initializer->members; it; it = it->next) {
             UiObjectMember *member = it->member;
             if (UiPublicMember *def = cast<UiPublicMember *>(member)) {
-                if (def->type == UiPublicMember::Property && !def->name.isEmpty() && def->isValid()) {
+                if (def->type == UiPublicMember::Property && !def->name.isEmpty()) {
                     ASTPropertyReference *ref = new ASTPropertyReference(def, m_doc, valueOwner);
                     m_properties.append(ref);
                     if (def->defaultToken.isValid())
@@ -1913,7 +1932,7 @@ const Document *ASTObjectValue::document() const
     return m_doc;
 }
 
-ASTVariableReference::ASTVariableReference(VariableDeclaration *ast, const Document *doc, ValueOwner *valueOwner)
+ASTVariableReference::ASTVariableReference(PatternElement *ast, const Document *doc, ValueOwner *valueOwner)
     : Reference(valueOwner)
     , m_ast(ast)
     , m_doc(doc)
@@ -1929,7 +1948,7 @@ const ASTVariableReference *ASTVariableReference::asAstVariableReference() const
     return this;
 }
 
-const VariableDeclaration *ASTVariableReference::ast() const
+const PatternElement *ASTVariableReference::ast() const
 {
     return m_ast;
 }
@@ -1937,16 +1956,16 @@ const VariableDeclaration *ASTVariableReference::ast() const
 const Value *ASTVariableReference::value(ReferenceContext *referenceContext) const
 {
     // may be assigned to later
-    if (!m_ast->expression)
+    if (!m_ast->expressionCast())
         return valueOwner()->unknownValue();
 
     Document::Ptr doc = m_doc->ptr();
     ScopeChain scopeChain(doc, referenceContext->context());
     ScopeBuilder builder(&scopeChain);
-    builder.push(ScopeAstPath(doc)(m_ast->expression->firstSourceLocation().begin()));
+    builder.push(ScopeAstPath(doc)(m_ast->expressionCast()->firstSourceLocation().begin()));
 
     Evaluate evaluator(&scopeChain, referenceContext);
-    return evaluator(m_ast->expression);
+    return evaluator(m_ast->expressionCast());
 }
 
 bool ASTVariableReference::getSourceLocation(QString *fileName, int *line, int *column) const
@@ -1963,12 +1982,12 @@ class UsesArgumentsArray : protected Visitor
     bool m_usesArgumentsArray;
 
 public:
-    bool operator()(FunctionBody *ast)
+    bool operator()(StatementList *ast)
     {
-        if (!ast || !ast->elements)
+        if (!ast)
             return false;
         m_usesArgumentsArray = false;
-        Node::accept(ast->elements, this);
+        Node::accept(ast, this);
         return m_usesArgumentsArray;
     }
 
@@ -1983,7 +2002,8 @@ protected:
     }
 
     // don't go into nested functions
-    bool visit(FunctionBody *) { return false; }
+    bool visit(Program *) { return false; }
+    bool visit(StatementList *) { return false; }
 };
 } // anonymous namespace
 
@@ -1995,7 +2015,7 @@ ASTFunctionValue::ASTFunctionValue(FunctionExpression *ast, const Document *doc,
     setPrototype(valueOwner->functionPrototype());
 
     for (FormalParameterList *it = ast->formals; it; it = it->next)
-        m_argumentNames.append(it->name.toString());
+        m_argumentNames.append(it->element->bindingIdentifier.toString());
 
     m_isVariadic = UsesArgumentsArray()(ast->body);
 }
@@ -2103,10 +2123,9 @@ bool ASTPropertyReference::getSourceLocation(QString *fileName, int *line, int *
 const Value *ASTPropertyReference::value(ReferenceContext *referenceContext) const
 {
     if (m_ast->statement
-            && (!m_ast->isValid()
-                || m_ast->memberTypeName() == QLatin1String("variant")
-                || m_ast->memberTypeName() == QLatin1String("var")
-                || m_ast->memberTypeName() == QLatin1String("alias"))) {
+            && (m_ast->memberType->name == QLatin1String("variant")
+                || m_ast->memberType->name == QLatin1String("var")
+                || m_ast->memberType->name == QLatin1String("alias"))) {
 
         // Adjust the context for the current location - expensive!
         // ### Improve efficiency by caching the 'use chain' constructed in ScopeBuilder.
@@ -2122,7 +2141,7 @@ const Value *ASTPropertyReference::value(ReferenceContext *referenceContext) con
         return evaluator(m_ast->statement);
     }
 
-    const QString memberType = m_ast->memberTypeName().toString();
+    const QString memberType = m_ast->memberType->name.toString();
 
     const Value *builtin = valueOwner()->defaultValueForBuiltinType(memberType);
     if (!builtin->asUndefinedValue())
@@ -2143,7 +2162,7 @@ ASTSignal::ASTSignal(UiPublicMember *ast, const Document *doc, ValueOwner *value
     const QString &signalName = ast->name.toString();
     m_slotName = generatedSlotName(signalName);
 
-    ObjectValue *v = valueOwner->newObject(/*prototype=*/0);
+    ObjectValue *v = valueOwner->newObject(/*prototype=*/nullptr);
     for (UiParameterList *it = ast->parameters; it; it = it->next) {
         if (!it->name.isEmpty())
             v->setMember(it->name.toString(), valueOwner->defaultValueForBuiltinType(it->type->name.toString()));
@@ -2199,19 +2218,13 @@ bool ASTSignal::getSourceLocation(QString *fileName, int *line, int *column) con
 
 ImportInfo::ImportInfo()
     : m_type(ImportType::Invalid)
-    , m_ast(0)
+    , m_ast(nullptr)
 {
 }
 
 ImportInfo ImportInfo::moduleImport(QString uri, ComponentVersion version,
                                     const QString &as, UiImport *ast)
 {
-    // treat Qt 4.7 as QtQuick 1.0
-    if (uri == QLatin1String("Qt") && version == ComponentVersion(4, 7)) {
-        uri = QLatin1String("QtQuick");
-        version = ComponentVersion(1, 0);
-    }
-
     ImportInfo info;
     info.m_type = ImportType::Library;
     info.m_name = uri;
@@ -2239,11 +2252,13 @@ ImportInfo ImportInfo::pathImport(const QString &docPath, const QString &path,
     } else if (importFileInfo.isDir()) {
         info.m_type = ImportType::Directory;
     } else if (path.startsWith(QLatin1String("qrc:"))) {
+        ModelManagerInterface *model = ModelManagerInterface::instance();
         info.m_path = path;
-        if (ModelManagerInterface::instance()->filesAtQrcPath(info.path()).isEmpty())
-            info.m_type = ImportType::QrcDirectory;
-        else
-            info.m_type = ImportType::QrcFile;
+        info.m_type = !model
+                ? ImportType::UnknownFile
+                : model->filesAtQrcPath(info.path()).isEmpty()
+                  ? ImportType::QrcDirectory
+                  : ImportType::QrcFile;
     } else {
         info.m_type = ImportType::UnknownFile;
     }
@@ -2313,7 +2328,7 @@ UiImport *ImportInfo::ast() const
 }
 
 Import::Import()
-    : object(0), valid(false), used(false)
+    : object(nullptr), valid(false), used(false)
 {}
 
 Import::Import(const Import &other)
@@ -2321,6 +2336,16 @@ Import::Import(const Import &other)
       valid(other.valid), used(false)
 { }
 
+Import &Import::operator=(const Import &other)
+{
+    object = other.object;
+    info = other.info;
+    libraryPath = other.libraryPath;
+    valid = other.valid;
+    used = false;
+    return *this;
+}
+
 TypeScope::TypeScope(const Imports *imports, ValueOwner *valueOwner)
     : ObjectValue(valueOwner)
     , m_imports(imports)
@@ -2330,10 +2355,9 @@ TypeScope::TypeScope(const Imports *imports, ValueOwner *valueOwner)
 const Value *TypeScope::lookupMember(const QString &name, const Context *context,
                                            const ObjectValue **foundInObject, bool) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2345,7 +2369,7 @@ const Value *TypeScope::lookupMember(const QString &name, const Context *context
             if (info.as() == name) {
                 if (foundInObject)
                     *foundInObject = this;
-                i.used = true;
+                i.used = true;  // FIXME: This evilly modifies a 'const' object
                 return import;
             }
             continue;
@@ -2357,16 +2381,15 @@ const Value *TypeScope::lookupMember(const QString &name, const Context *context
         }
     }
     if (foundInObject)
-        *foundInObject = 0;
-    return 0;
+        *foundInObject = nullptr;
+    return nullptr;
 }
 
 void TypeScope::processMembers(MemberProcessor *processor) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2395,10 +2418,9 @@ JSImportScope::JSImportScope(const Imports *imports, ValueOwner *valueOwner)
 const Value *JSImportScope::lookupMember(const QString &name, const Context *,
                                          const ObjectValue **foundInObject, bool) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2414,16 +2436,15 @@ const Value *JSImportScope::lookupMember(const QString &name, const Context *,
         }
     }
     if (foundInObject)
-        *foundInObject = 0;
-    return 0;
+        *foundInObject = nullptr;
+    return nullptr;
 }
 
 void JSImportScope::processMembers(MemberProcessor *processor) const
 {
-    QListIterator<Import> it(m_imports->all());
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    const QList<Import> &imports = m_imports->all();
+    for (int pos = imports.size(); --pos >= 0; ) {
+        const Import &i = imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2476,10 +2497,8 @@ ImportInfo Imports::info(const QString &name, const Context *context) const
     if (dotIdx != -1)
         firstId = firstId.left(dotIdx);
 
-    QListIterator<Import> it(m_imports);
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    for (int pos = m_imports.size(); --pos >= 0; ) {
+        const Import &i = m_imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2502,10 +2521,8 @@ ImportInfo Imports::info(const QString &name, const Context *context) const
 
 QString Imports::nameForImportedObject(const ObjectValue *value, const Context *context) const
 {
-    QListIterator<Import> it(m_imports);
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    for (int pos = m_imports.size(); --pos >= 0; ) {
+        const Import &i = m_imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
@@ -2554,31 +2571,31 @@ class MemberDumper: public MemberProcessor
 public:
     MemberDumper() {}
 
-    virtual bool processProperty(const QString &name, const Value *, const PropertyInfo &pInfo)
+    bool processProperty(const QString &name, const Value *, const PropertyInfo &pInfo) override
     {
         qCDebug(qmljsLog) << "property: " << name << " flags:" << pInfo.toString();
         return true;
     }
 
-    virtual bool processEnumerator(const QString &name, const Value *)
+    bool processEnumerator(const QString &name, const Value *) override
     {
         qCDebug(qmljsLog) << "enumerator: " << name;
         return true;
     }
 
-    virtual bool processSignal(const QString &name, const Value *)
+    bool processSignal(const QString &name, const Value *) override
     {
         qCDebug(qmljsLog) << "signal: " << name;
         return true;
     }
 
-    virtual bool processSlot(const QString &name, const Value *)
+    bool processSlot(const QString &name, const Value *) override
     {
         qCDebug(qmljsLog) << "slot: " << name;
         return true;
     }
 
-    virtual bool processGeneratedSlot(const QString &name, const Value *)
+    bool processGeneratedSlot(const QString &name, const Value *) override
     {
         qCDebug(qmljsLog) << "generated slot: " << name;
         return true;
@@ -2588,10 +2605,8 @@ public:
 void Imports::dump() const
 {
     qCDebug(qmljsLog) << "Imports contents, in search order:";
-    QListIterator<Import> it(m_imports);
-    it.toBack();
-    while (it.hasPrevious()) {
-        const Import &i = it.previous();
+    for (int pos = m_imports.size(); --pos >= 0; ) {
+        const Import &i = m_imports.at(pos);
         const ObjectValue *import = i.object;
         const ImportInfo &info = i.info;
 
