@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -35,10 +30,9 @@
 #include "qmljsbind.h"
 #include "qmljsutils.h"
 #include "qmljsmodelmanagerinterface.h"
-#include "qmljsqrcparser.h"
 #include "qmljsconstants.h"
 
-#include <extensionsystem/pluginmanager.h>
+#include <utils/qrcparser.h>
 
 #include <QDir>
 
@@ -144,29 +138,24 @@ Link::Link(const Snapshot &snapshot, const ViewerContext &vContext, const Librar
     d->builtins = builtins;
     d->vContext = vContext;
 
-    d->diagnosticMessages = 0;
-    d->allDiagnosticMessages = 0;
+    d->diagnosticMessages = nullptr;
+    d->allDiagnosticMessages = nullptr;
 
     ModelManagerInterface *modelManager = ModelManagerInterface::instance();
     if (modelManager) {
-        ModelManagerInterface::CppDataHash cppDataHash = modelManager->cppData();
+        const ModelManagerInterface::CppDataHash cppDataHash = modelManager->cppData();
         {
             // populate engine with types from C++
-            ModelManagerInterface::CppDataHashIterator cppDataHashIterator(cppDataHash);
-            while (cppDataHashIterator.hasNext()) {
-                cppDataHashIterator.next();
-                d->valueOwner->cppQmlTypes().load(cppDataHashIterator.key(),
-                                                  cppDataHashIterator.value().exportedTypes);
-            }
+            for (auto it = cppDataHash.cbegin(), end = cppDataHash.cend(); it != end; ++it)
+                d->valueOwner->cppQmlTypes().load(it.key(), it.value().exportedTypes);
         }
 
         // build an object with the context properties from C++
-        ObjectValue *cppContextProperties = d->valueOwner->newObject(/* prototype = */ 0);
-        foreach (const ModelManagerInterface::CppData &cppData, cppDataHash) {
-            QHashIterator<QString, QString> it(cppData.contextProperties);
-            while (it.hasNext()) {
-                it.next();
-                const Value *value = 0;
+        ObjectValue *cppContextProperties = d->valueOwner->newObject(/* prototype = */ nullptr);
+        for (const ModelManagerInterface::CppData &cppData : cppDataHash) {
+            for (auto it = cppData.contextProperties.cbegin(), end = cppData.contextProperties.cend();
+                    it != end; ++it) {
+                const Value *value = nullptr;
                 const QString cppTypeName = it.value();
                 if (!cppTypeName.isEmpty())
                     value = d->valueOwner->cppQmlTypes().objectByCppName(cppTypeName);
@@ -217,8 +206,7 @@ Context::ImportsPerDocument LinkPrivate::linkImports()
         Imports *imports = new Imports(valueOwner);
 
         // Add custom imports for the opened document
-        auto providers = ExtensionSystem::PluginManager::getObjects<CustomImportsProvider>();
-        foreach (const auto &provider, providers)
+        for (const auto &provider : CustomImportsProvider::allProviders())
             foreach (const auto &import, provider->imports(valueOwner, document.data()))
                 importCache.insert(ImportCacheKey(import.info), import);
 
@@ -273,7 +261,7 @@ void LinkPrivate::populateImportedTypes(Imports *imports, Document::Ptr doc)
                 imports->setImportFailed();
                 if (info.ast()) {
                     error(doc, info.ast()->fileNameToken,
-                          Link::tr("file or directory not found"));
+                          Link::tr("File or directory not found."));
                 }
                 break;
             default:
@@ -301,7 +289,7 @@ Import LinkPrivate::importFileOrDirectory(Document::Ptr doc, const ImportInfo &i
 {
     Import import;
     import.info = importInfo;
-    import.object = 0;
+    import.object = nullptr;
     import.valid = true;
 
     QString path = importInfo.path();
@@ -326,7 +314,7 @@ Import LinkPrivate::importFileOrDirectory(Document::Ptr doc, const ImportInfo &i
     } else if (importInfo.type() == ImportType::QrcFile) {
         QLocale locale;
         QStringList filePaths = ModelManagerInterface::instance()
-                ->filesAtQrcPath(path, &locale, 0, ModelManagerInterface::ActiveQrcResources);
+                ->filesAtQrcPath(path, &locale, nullptr, ModelManagerInterface::ActiveQrcResources);
         if (filePaths.isEmpty())
             filePaths = ModelManagerInterface::instance()->filesAtQrcPath(path);
         if (!filePaths.isEmpty()) {
@@ -339,10 +327,8 @@ Import LinkPrivate::importFileOrDirectory(Document::Ptr doc, const ImportInfo &i
 
         importLibrary(doc, path, &import);
 
-        QMapIterator<QString,QStringList> iter(ModelManagerInterface::instance()
-                                               ->filesInQrcPath(path));
-        while (iter.hasNext()) {
-            iter.next();
+        const QMap<QString, QStringList> paths = ModelManagerInterface::instance()->filesInQrcPath(path);
+        for (auto iter = paths.cbegin(), end = paths.cend(); iter != end; ++iter) {
             if (ModelManagerInterface::guessLanguageOfFile(iter.key()).isQmlLikeLanguage()) {
                 Document::Ptr importedDoc = snapshot.document(iter.value().at(0));
                 if (importedDoc && importedDoc->bind()->rootObjectValue()) {
@@ -370,7 +356,6 @@ static ModuleApiInfo findBestModuleApi(const QList<ModuleApiInfo> &apis, const C
 /*
   import Qt 4.6
   import Qt 4.6 as Xxx
-  (import com.nokia.qt is the same as the ones above)
 */
 Import LinkPrivate::importNonFile(Document::Ptr doc, const ImportInfo &importInfo)
 {
@@ -382,38 +367,8 @@ Import LinkPrivate::importNonFile(Document::Ptr doc, const ImportInfo &importInf
     const QString packageName = importInfo.name();
     const ComponentVersion version = importInfo.version();
 
-    bool importFound = false;
-
-    const QString &packagePath = importInfo.path();
-    // check the filesystem with full version
-    foreach (const QString &importPath, importPaths) {
-        QString libraryPath = QString::fromLatin1("%1/%2.%3").arg(importPath, packagePath, version.toString());
-        if (importLibrary(doc, libraryPath, &import, importPath)) {
-            importFound = true;
-            break;
-        }
-    }
-    if (!importFound) {
-        // check the filesystem with major version
-        foreach (const QString &importPath, importPaths) {
-            QString libraryPath = QString::fromLatin1("%1/%2.%3").arg(importPath, packagePath,
-                                                          QString::number(version.majorVersion()));
-            if (importLibrary(doc, libraryPath, &import, importPath)) {
-                importFound = true;
-                break;
-            }
-        }
-    }
-    if (!importFound) {
-        // check the filesystem with no version
-        foreach (const QString &importPath, importPaths) {
-            QString libraryPath = QString::fromLatin1("%1/%2").arg(importPath, packagePath);
-            if (importLibrary(doc, libraryPath, &import, importPath)) {
-                importFound = true;
-                break;
-            }
-        }
-    }
+    QString libraryPath = modulePath(packageName, version.toString(), importPaths);
+    bool importFound = !libraryPath.isEmpty() && importLibrary(doc, libraryPath, &import);
 
     // if there are cpp-based types for this package, use them too
     if (valueOwner->cppQmlTypes().hasModule(packageName)) {
@@ -431,19 +386,24 @@ Import LinkPrivate::importNonFile(Document::Ptr doc, const ImportInfo &importInf
         import.object->setPrototype(valueOwner->cppQmlTypes().objectByCppName(moduleApi.cppName));
     }
 
+    // TODO: at the moment there is not any types information on Qbs imports.
+    if (doc->language() == Dialect::QmlQbs)
+        importFound = true;
+
     if (!importFound && importInfo.ast()) {
         import.valid = false;
         error(doc, locationFromRange(importInfo.ast()->firstSourceLocation(),
                                      importInfo.ast()->lastSourceLocation()),
               Link::tr(
-                  "QML module not found\n\n"
+                  "QML module not found (%1).\n\n"
                   "Import paths:\n"
-                  "%1\n\n"
+                  "%2\n\n"
                   "For qmake projects, use the QML_IMPORT_PATH variable to add import paths.\n"
-                  "For qbs projects, declare and set a qmlImportPaths property in your product "
+                  "For Qbs projects, declare and set a qmlImportPaths property in your product "
                   "to add import paths.\n"
-                  "For qmlproject projects, use the importPaths property to add import paths.").arg(
-                  importPaths.join(QLatin1Char('\n'))));
+                  "For qmlproject projects, use the importPaths property to add import paths.\n"
+                  "For CMake projects, make sure QML_IMPORT_PATH variable is in CMakeCache.txt.\n").arg(
+                  importInfo.name(), importPaths.join(QLatin1Char('\n'))));
     }
 
     return import;
@@ -458,13 +418,8 @@ bool LinkPrivate::importLibrary(Document::Ptr doc,
     QString libraryPath = libraryPath_;
 
     LibraryInfo libraryInfo = snapshot.libraryInfo(libraryPath);
-    if (!libraryInfo.isValid()) {
-        // try canonical path
-        libraryPath = QFileInfo(libraryPath).canonicalFilePath();
-        libraryInfo = snapshot.libraryInfo(libraryPath);
-        if (!libraryInfo.isValid())
-            return false;
-    }
+    if (!libraryInfo.isValid())
+        return false;
 
     import->libraryPath = libraryPath;
 
@@ -491,8 +446,9 @@ bool LinkPrivate::importLibrary(Document::Ptr doc,
                 }
             }
             if (errorLoc.isValid()) {
-                warning(doc, errorLoc,
-                        Link::tr("QML module contains C++ plugins, currently reading type information..."));
+                appendDiagnostic(doc, DiagnosticMessage(Severity::ReadingTypeInfoWarning,
+                                                        errorLoc,
+                                                        Link::tr("QML module contains C++ plugins, currently reading type information...")));
                 import->valid = false;
             }
         } else if (libraryInfo.pluginTypeInfoStatus() == LibraryInfo::DumpError
@@ -580,16 +536,23 @@ void LinkPrivate::loadQmldirComponents(ObjectValue *import, ComponentVersion ver
 
 void LinkPrivate::loadImplicitDirectoryImports(Imports *imports, Document::Ptr doc)
 {
-    ImportInfo implcitDirectoryImportInfo = ImportInfo::implicitDirectoryImport(doc->path());
-
-    Import directoryImport = importCache.value(ImportCacheKey(implcitDirectoryImportInfo));
-    if (!directoryImport.object) {
-        directoryImport = importFileOrDirectory(doc, implcitDirectoryImportInfo);
+    auto processImport = [this, imports, doc](const ImportInfo &importInfo){
+        Import directoryImport = importCache.value(ImportCacheKey(importInfo));
+        if (!directoryImport.object) {
+            directoryImport = importFileOrDirectory(doc, importInfo);
+            if (directoryImport.object)
+                importCache.insert(ImportCacheKey(importInfo), directoryImport);
+        }
         if (directoryImport.object)
-            importCache.insert(ImportCacheKey(implcitDirectoryImportInfo), directoryImport);
+            imports->append(directoryImport);
+    };
+
+    processImport(ImportInfo::implicitDirectoryImport(doc->path()));
+    foreach (const QString &path,
+             ModelManagerInterface::instance()->qrcPathsForFile(doc->fileName())) {
+        processImport(ImportInfo::qrcDirectoryImport(
+                          Utils::QrcParser::qrcDirectoryPathForQrcFilePath(path)));
     }
-    if (directoryImport.object)
-        imports->append(directoryImport);
 }
 
 void LinkPrivate::loadImplicitDefaultImports(Imports *imports)
