@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,31 +9,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "qmljsreformatter.h"
-
 #include "qmljscodeformatter.h"
+#include "qmljsbind.h"
+
 #include "parser/qmljsast_p.h"
 #include "parser/qmljsastvisitor_p.h"
 #include "parser/qmljsengine_p.h"
+#include "parser/qmljslexer_p.h"
 
 #include <QString>
 #include <QTextBlock>
@@ -57,12 +54,12 @@ protected:
         BlockData data;
     };
 
-    virtual void saveBlockData(QTextBlock *block, const BlockData &data) const
+    void saveBlockData(QTextBlock *block, const BlockData &data) const override
     {
         block->setUserData(new FormatterData(data));
     }
 
-    virtual bool loadBlockData(const QTextBlock &block, BlockData *data) const
+    bool loadBlockData(const QTextBlock &block, BlockData *data) const override
     {
         if (!block.userData())
             return false;
@@ -71,12 +68,12 @@ protected:
         return true;
     }
 
-    virtual void saveLexerState(QTextBlock *block, int state) const
+    void saveLexerState(QTextBlock *block, int state) const override
     {
         block->setUserState(state);
     }
 
-    virtual int loadLexerState(const QTextBlock &block) const
+    int loadLexerState(const QTextBlock &block) const override
     {
         return block.userState();
     }
@@ -95,11 +92,12 @@ class Rewriter : protected Visitor
     QList<Split> _possibleSplits;
     QTextDocument _resultDocument;
     SimpleFormatter _formatter;
-    int _indent;
-    int _nextComment;
-    int _lastNewlineOffset;
-    bool _hadEmptyLine;
-    int _binaryExpDepth;
+    int _indent = 0;
+    int _nextComment = 0;
+    int _lastNewlineOffset = -1;
+    bool _hadEmptyLine = false;
+    int _binaryExpDepth = 0;
+    bool _hasOpenComment = false;
 
 public:
     Rewriter(Document::Ptr doc)
@@ -107,6 +105,9 @@ public:
     {
     }
 
+    void setIndentSize(int size) { _formatter.setIndentSize(size); }
+    void setTabSize(int size) { _formatter.setTabSize(size); }
+
     QString operator()(Node *node)
     {
         Q_ASSERT(node == _doc->ast()); // comment handling fails otherwise
@@ -120,6 +121,27 @@ public:
         _hadEmptyLine = false;
         _binaryExpDepth = 0;
 
+
+        // emit directives
+        if (_doc->bind()->isJsLibrary()) {
+            out(QLatin1String(".pragma library"));
+            newLine();
+        }
+        const QList<SourceLocation> &directives = _doc->jsDirectives();
+        for (const auto &d: directives) {
+            quint32 line = 1;
+            int i = 0;
+            while (line++ < d.startLine && i++ >= 0)
+                i = _doc->source().indexOf(QChar('\n'), i);
+            quint32 offset = static_cast<quint32>(i) + d.startColumn;
+            int endline = _doc->source().indexOf('\n', static_cast<int>(offset) + 1);
+            int end = endline == -1 ? _doc->source().length() : endline;
+            quint32 length =  static_cast<quint32>(end) - offset;
+            out(SourceLocation(offset, length, d.startLine, d.startColumn));
+        }
+        if (!directives.isEmpty())
+            newLine();
+
         accept(node);
 
         // emit the final comments
@@ -152,13 +174,31 @@ protected:
         out(QString::fromLatin1(str), lastLoc);
     }
 
+    void outCommentText(const QString &str)
+    {
+        QStringList lines = str.split(QLatin1Char('\n'));
+        bool multiline = lines.length() > 1;
+        for (int i = 0; i < lines.size(); ++i) {
+            if (multiline) {
+                if (i == 0)
+                    newLine();
+                _line = lines.at(i);  // multiline comments don't keep track of previos lines
+            }
+            else
+                _line += lines.at(i);
+            if (i != lines.size() - 1)
+                newLine();
+        }
+        _hadEmptyLine = false;
+    }
+
     void outComment(const SourceLocation &commentLoc)
     {
         SourceLocation fixedLoc = commentLoc;
         fixCommentLocation(fixedLoc);
-        if (precededByEmptyLine(fixedLoc))
+        if (precededByEmptyLine(fixedLoc) && !_result.endsWith(QLatin1String("\n\n")))
             newLine();
-        out(toString(fixedLoc)); // don't use the sourceloc overload here
+        outCommentText(toString(fixedLoc)); // don't use the sourceloc overload here
         if (followedByNewLine(fixedLoc))
             newLine();
         else
@@ -167,6 +207,9 @@ protected:
 
     void out(const QString &str, const SourceLocation &lastLoc = SourceLocation())
     {
+        if (_hasOpenComment) {
+            newLine();
+        }
         if (lastLoc.isValid()) {
             QList<SourceLocation> comments = _doc->engine()->comments();
             for (; _nextComment < comments.size(); ++_nextComment) {
@@ -287,6 +330,15 @@ protected:
     {
         BestSplit result;
 
+        while (possibleSplits.count() > 12) {
+             QList<Split> newPossibleSplits;
+             for (int i = 0; i < possibleSplits.count(); i++) {
+                 if (!(i % 2))
+                     newPossibleSplits.push_back(possibleSplits.at(i));
+             }
+             possibleSplits = newPossibleSplits;
+        }
+
         result.badnessFromSplits = 0;
         result.lines = QStringList(line);
 
@@ -328,6 +380,7 @@ protected:
     {
         // if preceded by a newline, it's an empty line!
         _hadEmptyLine = _line.trimmed().isEmpty();
+        _hasOpenComment = false;
 
         // if the preceding line wasn't empty, reindent etc.
         if (!_hadEmptyLine) {
@@ -407,15 +460,6 @@ protected:
         return false;
     }
 
-    bool firstOnLine()
-    {
-        foreach (const QChar &c, _line) {
-            if (!c.isSpace())
-                return false;
-        }
-        return true;
-    }
-
     void addPossibleSplit(qreal badness, int offset = 0)
     {
         Split s;
@@ -433,7 +477,7 @@ protected:
             loc.length += 2;
     }
 
-    virtual bool preVisit(Node *ast)
+    bool preVisit(Node *ast) override
     {
         SourceLocation firstLoc;
         if (ExpressionNode *expr = ast->expressionCast())
@@ -455,7 +499,7 @@ protected:
         return true;
     }
 
-    virtual void postVisit(Node *ast)
+    void postVisit(Node *ast) override
     {
         SourceLocation lastLoc;
         if (ExpressionNode *expr = ast->expressionCast())
@@ -490,11 +534,51 @@ protected:
 
                 out(" ");
                 out(toString(nextCommentLoc));
+                _hasOpenComment = true;
             }
         }
     }
 
-    virtual bool visit(UiImport *ast)
+    bool visit(UiPragma *ast) override
+    {
+        out("pragma ", ast->pragmaToken);
+        out(ast->name.toString());
+        newLine();
+        return false;
+    }
+
+    bool visit(UiEnumDeclaration *ast) override
+    {
+        out(ast->enumToken);
+        out(" ");
+        out(ast->name.toString());
+        out(" ");
+        out("{"); // TODO: out(ast->lbraceToken);
+        newLine();
+
+        accept(ast->members);
+
+        out(ast->rbraceToken);
+        return false;
+    }
+
+    bool visit(UiEnumMemberList *list) override
+    {
+        for (UiEnumMemberList *it = list; it; it = it->next) {
+            out(it->memberToken);
+            if (it->valueToken.isValid()) {
+                out(" = ");
+                out(it->valueToken);
+            }
+            if (it->next) {
+                out(",");
+            }
+            newLine();
+        }
+        return false;
+    }
+
+    bool visit(UiImport *ast) override
     {
         out("import ", ast->importToken);
         if (!ast->fileName.isNull())
@@ -512,7 +596,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiObjectDefinition *ast)
+    bool visit(UiObjectDefinition *ast) override
     {
         accept(ast->qualifiedTypeNameId);
         out(" ");
@@ -520,17 +604,18 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiObjectInitializer *ast)
+    bool visit(UiObjectInitializer *ast) override
     {
         out(ast->lbraceToken);
-        if (ast->members)
+        if (ast->members) {
             lnAcceptIndented(ast->members);
-        newLine();
+            newLine();
+        }
         out(ast->rbraceToken);
         return false;
     }
 
-    virtual bool visit(UiParameterList *list)
+    bool visit(UiParameterList *list) override
     {
         for (UiParameterList *it = list; it; it = it->next) {
             out(it->propertyTypeToken);
@@ -542,31 +627,34 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiPublicMember *ast)
+    bool visit(UiPublicMember *ast) override
     {
         if (ast->type == UiPublicMember::Property) {
             if (ast->isDefaultMember)
                 out("default ", ast->defaultToken);
+            else if (ast->isReadonlyMember)
+                out("readonly ", ast->readonlyToken);
             out("property ", ast->propertyToken);
             if (!ast->typeModifier.isNull()) {
                 out(ast->typeModifierToken);
                 out("<");
-                out(ast->typeToken);
+                accept(ast->memberType);
                 out(">");
             } else {
-                out(ast->typeToken);
+                accept(ast->memberType);
             }
             out(" ");
-            out(ast->identifierToken);
             if (ast->statement) {
+                out(ast->identifierToken);
                 out(": ", ast->colonToken);
                 accept(ast->statement);
             } else if (ast->binding) {
-                out(": ", ast->colonToken);
                 accept(ast->binding);
+            } else {
+                out(ast->identifierToken);
             }
         } else { // signal
-            out("signal ");
+            out("signal ", ast->identifierToken);
             out(ast->identifierToken);
             if (ast->parameters) {
                 out("(");
@@ -577,7 +665,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiObjectBinding *ast)
+    bool visit(UiObjectBinding *ast) override
     {
         if (ast->hasOnToken) {
             accept(ast->qualifiedTypeNameId);
@@ -593,7 +681,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiScriptBinding *ast)
+    bool visit(UiScriptBinding *ast) override
     {
         accept(ast->qualifiedId);
         out(": ", ast->colonToken);
@@ -601,7 +689,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiArrayBinding *ast)
+    bool visit(UiArrayBinding *ast) override
     {
         accept(ast->qualifiedId);
         out(ast->colonToken);
@@ -613,90 +701,92 @@ protected:
         return false;
     }
 
-    virtual bool visit(ThisExpression *ast) { out(ast->thisToken); return true; }
-    virtual bool visit(NullExpression *ast) { out(ast->nullToken); return true; }
-    virtual bool visit(TrueLiteral *ast) { out(ast->trueToken); return true; }
-    virtual bool visit(FalseLiteral *ast) { out(ast->falseToken); return true; }
-    virtual bool visit(IdentifierExpression *ast) { out(ast->identifierToken); return true; }
-    virtual bool visit(StringLiteral *ast) { out(ast->literalToken); return true; }
-    virtual bool visit(NumericLiteral *ast) { out(ast->literalToken); return true; }
-    virtual bool visit(RegExpLiteral *ast) { out(ast->literalToken); return true; }
+    bool visit(ThisExpression *ast) override { out(ast->thisToken); return true; }
+    bool visit(NullExpression *ast) override { out(ast->nullToken); return true; }
+    bool visit(TrueLiteral *ast) override { out(ast->trueToken); return true; }
+    bool visit(FalseLiteral *ast) override { out(ast->falseToken); return true; }
+    bool visit(IdentifierExpression *ast) override { out(ast->identifierToken); return true; }
+    bool visit(StringLiteral *ast) override { out(ast->literalToken); return true; }
+    bool visit(NumericLiteral *ast) override { out(ast->literalToken); return true; }
+    bool visit(RegExpLiteral *ast) override { out(ast->literalToken); return true; }
 
-    virtual bool visit(ArrayLiteral *ast)
+    bool visit(ArrayPattern *ast) override
     {
         out(ast->lbracketToken);
         if (ast->elements)
             accept(ast->elements);
-        if (ast->elements && ast->elision)
-            out(", ", ast->commaToken);
-        if (ast->elision)
-            accept(ast->elision);
         out(ast->rbracketToken);
         return false;
     }
 
-    virtual bool visit(ObjectLiteral *ast)
+    bool visit(ObjectPattern *ast) override
     {
         out(ast->lbraceToken);
-        lnAcceptIndented(ast->properties);
-        newLine();
+        if (ast->properties) {
+            lnAcceptIndented(ast->properties);
+            newLine();
+        }
         out(ast->rbraceToken);
         return false;
     }
 
-    virtual bool visit(ElementList *ast)
+    bool visit(PatternElementList *ast) override
     {
-        for (ElementList *it = ast; it; it = it->next) {
+        for (PatternElementList *it = ast; it; it = it->next) {
             if (it->elision)
                 accept(it->elision);
-            if (it->elision && it->expression)
+            if (it->elision && it->element)
                 out(", ");
-            if (it->expression)
-                accept(it->expression);
+            if (it->element)
+                accept(it->element);
             if (it->next)
-                out(", ", ast->commaToken);
+                out(", ");
         }
         return false;
     }
 
-    virtual bool visit(PropertyAssignmentList *ast)
+    bool visit(PatternPropertyList *ast) override
     {
-        for (PropertyAssignmentList *it = ast; it; it = it->next) {
-            PropertyNameAndValue *assignment = AST::cast<PropertyNameAndValue *>(it->assignment);
+        for (PatternPropertyList *it = ast; it; it = it->next) {
+            PatternProperty *assignment = AST::cast<PatternProperty *>(it->property);
             if (assignment) {
+                out("\"");
                 accept(assignment->name);
+                out("\"");
                 out(": ", assignment->colonToken);
-                accept(assignment->value);
+                accept(assignment->initializer);
                 if (it->next) {
-                    out(",", ast->commaToken); // always invalid?
+                    out(","); // always invalid?
                     newLine();
                 }
                 continue;
             }
-            PropertyGetterSetter *getterSetter = AST::cast<PropertyGetterSetter *>(it->assignment);
-            if (getterSetter) {
-                switch (getterSetter->type) {
-                case PropertyGetterSetter::Getter:
+            PatternPropertyList *getterSetter = AST::cast<PatternPropertyList *>(it->next);
+            if (getterSetter->property) {
+                switch (getterSetter->property->type) {
+                case PatternElement::Getter:
                     out("get");
                     break;
-                case PropertyGetterSetter::Setter:
+                case PatternElement::Setter:
                     out("set");
                     break;
+                default:
+                    break;
                 }
 
-                accept(getterSetter->name);
-                out("(", getterSetter->lparenToken);
-                accept(getterSetter->formals);
-                out("(", getterSetter->rparenToken);
-                out(" {", getterSetter->lbraceToken);
-                accept(getterSetter->functionBody);
-                out(" }", getterSetter->rbraceToken);
+                accept(getterSetter->property->name);
+                out("(");
+                //accept(getterSetter->formals);  // TODO
+                out(")");
+                out(" {");
+                //accept(getterSetter->functionBody);  // TODO
+                out(" }");
             }
         }
         return false;
     }
 
-    virtual bool visit(NestedExpression *ast)
+    bool visit(NestedExpression *ast) override
     {
         out(ast->lparenToken);
         accept(ast->expression);
@@ -704,11 +794,11 @@ protected:
         return false;
     }
 
-    virtual bool visit(IdentifierPropertyName *ast) { out(ast->id.toString()); return true; }
-    virtual bool visit(StringLiteralPropertyName *ast) { out(ast->id.toString()); return true; }
-    virtual bool visit(NumericLiteralPropertyName *ast) { out(QString::number(ast->id)); return true; }
+    bool visit(IdentifierPropertyName *ast) override { out(ast->id.toString()); return true; }
+    bool visit(StringLiteralPropertyName *ast) override { out(ast->id.toString()); return true; }
+    bool visit(NumericLiteralPropertyName *ast) override { out(QString::number(ast->id)); return true; }
 
-    virtual bool visit(ArrayMemberExpression *ast)
+    bool visit(ArrayMemberExpression *ast) override
     {
         accept(ast->base);
         out(ast->lbracketToken);
@@ -717,7 +807,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(FieldMemberExpression *ast)
+    bool visit(FieldMemberExpression *ast) override
     {
         accept(ast->base);
         out(ast->dotToken);
@@ -725,7 +815,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(NewMemberExpression *ast)
+    bool visit(NewMemberExpression *ast) override
     {
         out("new ", ast->newToken);
         accept(ast->base);
@@ -735,14 +825,14 @@ protected:
         return false;
     }
 
-    virtual bool visit(NewExpression *ast)
+    bool visit(NewExpression *ast) override
     {
         out("new ", ast->newToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(CallExpression *ast)
+    bool visit(CallExpression *ast) override
     {
         accept(ast->base);
         out(ast->lparenToken);
@@ -752,84 +842,84 @@ protected:
         return false;
     }
 
-    virtual bool visit(PostIncrementExpression *ast)
+    bool visit(PostIncrementExpression *ast) override
     {
         accept(ast->base);
         out(ast->incrementToken);
         return false;
     }
 
-    virtual bool visit(PostDecrementExpression *ast)
+    bool visit(PostDecrementExpression *ast) override
     {
         accept(ast->base);
         out(ast->decrementToken);
         return false;
     }
 
-    virtual bool visit(PreIncrementExpression *ast)
+    bool visit(PreIncrementExpression *ast) override
     {
         out(ast->incrementToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(PreDecrementExpression *ast)
+    bool visit(PreDecrementExpression *ast) override
     {
         out(ast->decrementToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(DeleteExpression *ast)
+    bool visit(DeleteExpression *ast) override
     {
         out("delete ", ast->deleteToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(VoidExpression *ast)
+    bool visit(VoidExpression *ast) override
     {
         out("void ", ast->voidToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(TypeOfExpression *ast)
+    bool visit(TypeOfExpression *ast) override
     {
         out("typeof ", ast->typeofToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(UnaryPlusExpression *ast)
+    bool visit(UnaryPlusExpression *ast) override
     {
         out(ast->plusToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(UnaryMinusExpression *ast)
+    bool visit(UnaryMinusExpression *ast) override
     {
         out(ast->minusToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(TildeExpression *ast)
+    bool visit(TildeExpression *ast) override
     {
         out(ast->tildeToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(NotExpression *ast)
+    bool visit(NotExpression *ast) override
     {
         out(ast->notToken);
         accept(ast->expression);
         return false;
     }
 
-    virtual bool visit(BinaryExpression *ast)
+    bool visit(BinaryExpression *ast) override
     {
         ++_binaryExpDepth;
         accept(ast->left);
@@ -850,7 +940,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(ConditionalExpression *ast)
+    bool visit(ConditionalExpression *ast) override
     {
         accept(ast->expression);
         out(" ? ", ast->questionToken);
@@ -860,7 +950,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(Block *ast)
+    bool visit(Block *ast) override
     {
         out(ast->lbraceToken);
         lnAcceptIndented(ast->statements);
@@ -869,30 +959,41 @@ protected:
         return false;
     }
 
-    virtual bool visit(VariableStatement *ast)
+    bool visit(VariableStatement *ast) override
     {
-        out("var ", ast->declarationKindToken);
+        out(ast->declarationKindToken);
+        out(" ");
         accept(ast->declarations);
         return false;
     }
 
-    virtual bool visit(VariableDeclaration *ast)
+    bool visit(PatternElement *ast) override
     {
+        if (ast->isForDeclaration) {
+            if (ast->scope == VariableScope::Var) {
+                out("var ");
+            } else if (ast->scope == VariableScope::Let) {
+                out("let ");
+            } else if (ast->scope == VariableScope::Const) {
+                out("const ");
+            }
+        }
         out(ast->identifierToken);
-        if (ast->expression) {
-            out(" = ");
-            accept(ast->expression);
+        if (ast->initializer) {
+            if (ast->isVariableDeclaration())
+                out(" = ");
+            accept(ast->initializer);
         }
         return false;
     }
 
-    virtual bool visit(EmptyStatement *ast)
+    bool visit(EmptyStatement *ast) override
     {
         out(ast->semicolonToken);
         return false;
     }
 
-    virtual bool visit(IfStatement *ast)
+    bool visit(IfStatement *ast) override
     {
         out(ast->ifToken);
         out(" ");
@@ -912,7 +1013,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(DoWhileStatement *ast)
+    bool visit(DoWhileStatement *ast) override
     {
         out(ast->doToken);
         acceptBlockOrIndented(ast->statement, true);
@@ -924,7 +1025,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(WhileStatement *ast)
+    bool visit(WhileStatement *ast) override
     {
         out(ast->whileToken);
         out(" ");
@@ -935,12 +1036,17 @@ protected:
         return false;
     }
 
-    virtual bool visit(ForStatement *ast)
+    bool visit(ForStatement *ast) override
     {
         out(ast->forToken);
         out(" ");
         out(ast->lparenToken);
-        accept(ast->initialiser);
+        if (ast->initialiser) {
+            accept(ast->initialiser);
+        } else if (ast->declarations) {
+            out("var ");
+            accept(ast->declarations);
+        }
         out("; ", ast->firstSemicolonToken);
         accept(ast->condition);
         out("; ", ast->secondSemicolonToken);
@@ -950,52 +1056,20 @@ protected:
         return false;
     }
 
-    virtual bool visit(LocalForStatement *ast)
+    bool visit(ForEachStatement *ast) override
     {
         out(ast->forToken);
         out(" ");
         out(ast->lparenToken);
-        out(ast->varToken);
-        out(" ");
-        accept(ast->declarations);
-        out("; ", ast->firstSemicolonToken);
-        accept(ast->condition);
-        out("; ", ast->secondSemicolonToken);
-        accept(ast->expression);
-        out(")", ast->rparenToken);
-        acceptBlockOrIndented(ast->statement);
-        return false;
-    }
-
-    virtual bool visit(ForEachStatement *ast)
-    {
-        out(ast->forToken);
-        out(" ");
-        out(ast->lparenToken);
-        accept(ast->initialiser);
-        out(" in ", ast->inToken);
+        accept(ast->lhs);
+        out(" in ");
         accept(ast->expression);
         out(ast->rparenToken);
         acceptBlockOrIndented(ast->statement);
         return false;
     }
 
-    virtual bool visit(LocalForEachStatement *ast)
-    {
-        out(ast->forToken);
-        out(" ");
-        out(ast->lparenToken);
-        out(ast->varToken);
-        out(" ");
-        accept(ast->declaration);
-        out(" in ", ast->inToken);
-        accept(ast->expression);
-        out(ast->rparenToken);
-        acceptBlockOrIndented(ast->statement);
-        return false;
-    }
-
-    virtual bool visit(ContinueStatement *ast)
+    bool visit(ContinueStatement *ast) override
     {
         out(ast->continueToken);
         if (!ast->label.isNull()) {
@@ -1005,7 +1079,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(BreakStatement *ast)
+    bool visit(BreakStatement *ast) override
     {
         out(ast->breakToken);
         if (!ast->label.isNull()) {
@@ -1015,7 +1089,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(ReturnStatement *ast)
+    bool visit(ReturnStatement *ast) override
     {
         out(ast->returnToken);
         if (ast->expression) {
@@ -1025,7 +1099,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(ThrowStatement *ast)
+    bool visit(ThrowStatement *ast) override
     {
         out(ast->throwToken);
         if (ast->expression) {
@@ -1035,7 +1109,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(WithStatement *ast)
+    bool visit(WithStatement *ast) override
     {
         out(ast->withToken);
         out(" ");
@@ -1046,7 +1120,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(SwitchStatement *ast)
+    bool visit(SwitchStatement *ast) override
     {
         out(ast->switchToken);
         out(" ");
@@ -1058,7 +1132,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(CaseBlock *ast)
+    bool visit(CaseBlock *ast) override
     {
         out(ast->lbraceToken);
         newLine();
@@ -1074,16 +1148,17 @@ protected:
         return false;
     }
 
-    virtual bool visit(CaseClause *ast)
+    bool visit(CaseClause *ast) override
     {
         out("case ", ast->caseToken);
         accept(ast->expression);
         out(ast->colonToken);
-        lnAcceptIndented(ast->statements);
+        if (ast->statements)
+            lnAcceptIndented(ast->statements);
         return false;
     }
 
-    virtual bool visit(DefaultClause *ast)
+    bool visit(DefaultClause *ast) override
     {
         out(ast->defaultToken);
         out(ast->colonToken);
@@ -1091,7 +1166,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(LabelledStatement *ast)
+    bool visit(LabelledStatement *ast) override
     {
         out(ast->identifierToken);
         out(": ", ast->colonToken);
@@ -1099,7 +1174,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(TryStatement *ast)
+    bool visit(TryStatement *ast) override
     {
         out("try ", ast->tryToken);
         accept(ast->statement);
@@ -1114,7 +1189,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(Catch *ast)
+    bool visit(Catch *ast) override
     {
         out(ast->catchToken);
         out(" ");
@@ -1125,19 +1200,19 @@ protected:
         return false;
     }
 
-    virtual bool visit(Finally *ast)
+    bool visit(Finally *ast) override
     {
         out("finally ", ast->finallyToken);
         accept(ast->statement);
         return false;
     }
 
-    virtual bool visit(FunctionDeclaration *ast)
+    bool visit(FunctionDeclaration *ast) override
     {
         return visit(static_cast<FunctionExpression *>(ast));
     }
 
-    virtual bool visit(FunctionExpression *ast)
+    bool visit(FunctionExpression *ast) override
     {
         out("function ", ast->functionToken);
         if (!ast->name.isNull())
@@ -1156,7 +1231,7 @@ protected:
     }
 
 
-    virtual bool visit(UiHeaderItemList *ast)
+    bool visit(UiHeaderItemList *ast) override
     {
         for (UiHeaderItemList *it = ast; it; it = it->next) {
             accept(it->headerItem);
@@ -1166,7 +1241,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiObjectMemberList *ast)
+    bool visit(UiObjectMemberList *ast) override
     {
         for (UiObjectMemberList *it = ast; it; it = it->next) {
             accept(it->member);
@@ -1176,7 +1251,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiArrayMemberList *ast)
+    bool visit(UiArrayMemberList *ast) override
     {
         for (UiArrayMemberList *it = ast; it; it = it->next) {
             accept(it->member);
@@ -1188,7 +1263,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(UiQualifiedId *ast)
+    bool visit(UiQualifiedId *ast) override
     {
         for (UiQualifiedId *it = ast; it; it = it->next) {
             out(it->identifierToken);
@@ -1198,7 +1273,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(Elision *ast)
+    bool visit(Elision *ast) override
     {
         for (Elision *it = ast; it; it = it->next) {
             if (it->next)
@@ -1207,7 +1282,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(ArgumentList *ast)
+    bool visit(ArgumentList *ast) override
     {
         for (ArgumentList *it = ast; it; it = it->next) {
             accept(it->expression);
@@ -1219,7 +1294,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(StatementList *ast)
+    bool visit(StatementList *ast) override
     {
         for (StatementList *it = ast; it; it = it->next) {
             // ### work around parser bug: skip empty statements with wrong tokens
@@ -1235,17 +1310,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(SourceElements *ast)
-    {
-        for (SourceElements *it = ast; it; it = it->next) {
-            accept(it->element);
-            if (it->next)
-                newLine();
-        }
-        return false;
-    }
-
-    virtual bool visit(VariableDeclarationList *ast)
+    bool visit(VariableDeclarationList *ast) override
     {
         for (VariableDeclarationList *it = ast; it; it = it->next) {
             accept(it->declaration);
@@ -1255,7 +1320,7 @@ protected:
         return false;
     }
 
-    virtual bool visit(CaseClauses *ast)
+    bool visit(CaseClauses *ast) override
     {
         for (CaseClauses *it = ast; it; it = it->next) {
             accept(it->clause);
@@ -1265,12 +1330,12 @@ protected:
         return false;
     }
 
-    virtual bool visit(FormalParameterList *ast)
+    bool visit(FormalParameterList *ast) override
     {
         for (FormalParameterList *it = ast; it; it = it->next) {
-            if (it->commaToken.isValid())
-                out(", ", it->commaToken);
-            out(it->identifierToken);
+            out(it->element->bindingIdentifier.toString()); // TODO
+            if (it->next)
+                out(", ");
         }
         return false;
     }
@@ -1283,3 +1348,11 @@ QString QmlJS::reformat(const Document::Ptr &doc)
     Rewriter rewriter(doc);
     return rewriter(doc->ast());
 }
+
+QString QmlJS::reformat(const Document::Ptr &doc, int indentSize, int tabSize)
+{
+    Rewriter rewriter(doc);
+    rewriter.setIndentSize(indentSize);
+    rewriter.setTabSize(tabSize);
+    return rewriter(doc->ast());
+}
