@@ -34,7 +34,7 @@
 #include <QFile>
 #include <QLoggingCategory>
 
-static Q_LOGGING_CATEGORY(simpleReaderLog, "qtc.qmljs.simpleReader")
+static Q_LOGGING_CATEGORY(simpleReaderLog, "qtc.qmljs.simpleReader", QtWarningMsg)
 
 namespace QmlJS{
 
@@ -92,7 +92,7 @@ SimpleReaderNode::Ptr SimpleReaderNode::create(const QString &name, WeakPtr pare
     Ptr newNode(new SimpleReaderNode(name, parent));
     newNode->m_weakThis = newNode;
     if (parent)
-        parent.data()->m_children.append(newNode);
+        parent.toStrongRef().data()->m_children.append(newNode);
     return newNode;
 }
 
@@ -250,12 +250,12 @@ QVariant SimpleAbstractStreamReader::parsePropertyExpression(AST::ExpressionNode
 {
     Q_ASSERT(expressionNode);
 
-    AST::ArrayLiteral *arrayLiteral = AST::cast<AST::ArrayLiteral *>(expressionNode);
+    AST::ArrayPattern *arrayLiteral = AST::cast<AST::ArrayPattern *>(expressionNode);
 
     if (arrayLiteral) {
         QList<QVariant> variantList;
-        for (AST::ElementList *it = arrayLiteral->elements; it; it = it->next)
-            variantList << parsePropertyExpression(it->expression);
+        for (AST::PatternElementList *it = arrayLiteral->elements; it; it = it->next)
+            variantList << parsePropertyExpression(it->element->initializer);
         return variantList;
     }
 
@@ -323,21 +323,21 @@ void SimpleReader::elementEnd()
 {
     Q_ASSERT(m_currentNode);
 
-    qCDebug(simpleReaderLog) << "elementEnd()" << m_currentNode.data()->name();
+    qCDebug(simpleReaderLog) << "elementEnd()" << m_currentNode.toStrongRef().data()->name();
 
-    m_currentNode = m_currentNode.data()->parent();
+    m_currentNode = m_currentNode.toStrongRef().data()->parent();
 }
 
 void SimpleReader::propertyDefinition(const QString &name, const QVariant &value)
 {
     Q_ASSERT(m_currentNode);
 
-    qCDebug(simpleReaderLog) << "propertyDefinition()" << m_currentNode.data()->name() << name << value;
+    qCDebug(simpleReaderLog) << "propertyDefinition()" << m_currentNode.toStrongRef().data()->name() << name << value;
 
-    if (m_currentNode.data()->propertyNames().contains(name))
+    if (m_currentNode.toStrongRef().data()->propertyNames().contains(name))
         addError(tr("Property is defined twice."), currentSourceLocation());
 
-    m_currentNode.data()->setProperty(name, value);
+    m_currentNode.toStrongRef().data()->setProperty(name, value);
 }
 
 } // namespace QmlJS
