@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -47,7 +42,7 @@ using namespace QmlJS::AST;
 using namespace LanguageUtils;
 
 TypeDescriptionReader::TypeDescriptionReader(const QString &fileName, const QString &data)
-    : _fileName (fileName), _source(data), _objects(0)
+    : _fileName (fileName), _source(data), _objects(nullptr)
 {
 }
 
@@ -57,7 +52,8 @@ TypeDescriptionReader::~TypeDescriptionReader()
 
 bool TypeDescriptionReader::operator()(
         QHash<QString, FakeMetaObject::ConstPtr> *objects,
-        QList<ModuleApiInfo> *moduleApis)
+        QList<ModuleApiInfo> *moduleApis,
+        QStringList *dependencies)
 {
     Engine engine;
 
@@ -76,6 +72,7 @@ bool TypeDescriptionReader::operator()(
 
     _objects = objects;
     _moduleApis = moduleApis;
+    _dependencies = dependencies;
     readDocument(parser.ast());
 
     return _errorMessage.isEmpty();
@@ -113,22 +110,20 @@ void TypeDescriptionReader::readDocument(UiProgram *ast)
     const QString versionString = _source.mid(import->versionToken.offset, import->versionToken.length);
     const int dotIdx = versionString.indexOf(QLatin1Char('.'));
     if (dotIdx != -1) {
-        version = ComponentVersion(versionString.left(dotIdx).toInt(),
-                                   versionString.mid(dotIdx + 1).toInt());
+        version = ComponentVersion(versionString.leftRef(dotIdx).toInt(),
+                                   versionString.midRef(dotIdx + 1).toInt());
     }
     if (version.majorVersion() != 1) {
         addError(import->versionToken, tr("Major version different from 1 not supported."));
         return;
     }
-    if (version.minorVersion() > 1)
-        addWarning(import->versionToken, tr("Reading only version 1.1 parts."));
 
     if (!ast->members || !ast->members->member || ast->members->next) {
         addError(SourceLocation(), tr("Expected document to contain a single object definition."));
         return;
     }
 
-    UiObjectDefinition *module = dynamic_cast<UiObjectDefinition *>(ast->members->member);
+    UiObjectDefinition *module = AST::cast<UiObjectDefinition *>(ast->members->member);
     if (!module) {
         addError(SourceLocation(), tr("Expected document to contain a single object definition."));
         return;
@@ -146,15 +141,19 @@ void TypeDescriptionReader::readModule(UiObjectDefinition *ast)
 {
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiObjectDefinition *component = dynamic_cast<UiObjectDefinition *>(member);
+        UiObjectDefinition *component = AST::cast<UiObjectDefinition *>(member);
+
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
+        if (script && (toString(script->qualifiedId) == QStringLiteral("dependencies"))) {
+            readDependencies(script);
+            continue;
+        }
 
         QString typeName;
         if (component)
             typeName = toString(component->qualifiedTypeNameId);
 
         if (!component || (typeName != QLatin1String("Component") && typeName != QLatin1String("ModuleApi"))) {
-            addWarning(member->firstSourceLocation(),
-                       tr("Expected only Component and ModuleApi object definitions."));
             continue;
         }
 
@@ -183,14 +182,33 @@ void TypeDescriptionReader::addWarning(const SourceLocation &loc, const QString
                 message);
 }
 
+void TypeDescriptionReader::readDependencies(UiScriptBinding *ast)
+{
+    ExpressionStatement *stmt = AST::cast<ExpressionStatement*>(ast->statement);
+    if (!stmt) {
+        addError(ast->statement->firstSourceLocation(), tr("Expected dependency definitions"));
+        return;
+    }
+    ArrayPattern *exp = AST::cast<ArrayPattern *>(stmt->expression);
+    if (!exp) {
+        addError(stmt->expression->firstSourceLocation(), tr("Expected dependency definitions"));
+        return;
+    }
+    for (PatternElementList *l = exp->elements; l; l = l->next) {
+        //StringLiteral *str = AST::cast<StringLiteral *>(l->element->initializer);
+        StringLiteral *str = AST::cast<StringLiteral *>(l->element->initializer);
+        *_dependencies << str->value.toString();
+    }
+}
+
 void TypeDescriptionReader::readComponent(UiObjectDefinition *ast)
 {
     FakeMetaObject::Ptr fmo(new FakeMetaObject);
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiObjectDefinition *component = dynamic_cast<UiObjectDefinition *>(member);
-        UiScriptBinding *script = dynamic_cast<UiScriptBinding *>(member);
+        UiObjectDefinition *component = AST::cast<UiObjectDefinition *>(member);
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
         if (component) {
             QString name = toString(component->qualifiedTypeNameId);
             if (name == QLatin1String("Property"))
@@ -251,7 +269,7 @@ void TypeDescriptionReader::readModuleApi(UiObjectDefinition *ast)
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiScriptBinding *script = dynamic_cast<UiScriptBinding *>(member);
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
 
         if (script) {
             const QString name = toString(script->qualifiedId);
@@ -290,8 +308,8 @@ void TypeDescriptionReader::readSignalOrMethod(UiObjectDefinition *ast, bool isM
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiObjectDefinition *component = dynamic_cast<UiObjectDefinition *>(member);
-        UiScriptBinding *script = dynamic_cast<UiScriptBinding *>(member);
+        UiObjectDefinition *component = AST::cast<UiObjectDefinition *>(member);
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
         if (component) {
             QString name = toString(component->qualifiedTypeNameId);
             if (name == QLatin1String("Parameter"))
@@ -333,7 +351,7 @@ void TypeDescriptionReader::readProperty(UiObjectDefinition *ast, FakeMetaObject
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiScriptBinding *script = dynamic_cast<UiScriptBinding *>(member);
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
         if (!script) {
             addWarning(member->firstSourceLocation(), tr("Expected script binding."));
             continue;
@@ -370,7 +388,7 @@ void TypeDescriptionReader::readEnum(UiObjectDefinition *ast, FakeMetaObject::Pt
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiScriptBinding *script = dynamic_cast<UiScriptBinding *>(member);
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
         if (!script) {
             addWarning(member->firstSourceLocation(), tr("Expected script binding."));
             continue;
@@ -395,7 +413,7 @@ void TypeDescriptionReader::readParameter(UiObjectDefinition *ast, FakeMetaMetho
 
     for (UiObjectMemberList *it = ast->initializer->members; it; it = it->next) {
         UiObjectMember *member = it->member;
-        UiScriptBinding *script = dynamic_cast<UiScriptBinding *>(member);
+        UiScriptBinding *script = AST::cast<UiScriptBinding *>(member);
         if (!script) {
             addWarning(member->firstSourceLocation(), tr("Expected script binding."));
             continue;
@@ -429,13 +447,13 @@ QString TypeDescriptionReader::readStringBinding(UiScriptBinding *ast)
         return QString();
     }
 
-    ExpressionStatement *expStmt = dynamic_cast<ExpressionStatement *>(ast->statement);
+    ExpressionStatement *expStmt = AST::cast<ExpressionStatement *>(ast->statement);
     if (!expStmt) {
         addError(ast->statement->firstSourceLocation(), tr("Expected string after colon."));
         return QString();
     }
 
-    StringLiteral *stringLit = dynamic_cast<StringLiteral *>(expStmt->expression);
+    StringLiteral *stringLit = AST::cast<StringLiteral *>(expStmt->expression);
     if (!stringLit) {
         addError(expStmt->firstSourceLocation(), tr("Expected string after colon."));
         return QString();
@@ -453,14 +471,14 @@ bool TypeDescriptionReader::readBoolBinding(AST::UiScriptBinding *ast)
         return false;
     }
 
-    ExpressionStatement *expStmt = dynamic_cast<ExpressionStatement *>(ast->statement);
+    ExpressionStatement *expStmt = AST::cast<ExpressionStatement *>(ast->statement);
     if (!expStmt) {
         addError(ast->statement->firstSourceLocation(), tr("Expected boolean after colon."));
         return false;
     }
 
-    TrueLiteral *trueLit = dynamic_cast<TrueLiteral *>(expStmt->expression);
-    FalseLiteral *falseLit = dynamic_cast<FalseLiteral *>(expStmt->expression);
+    TrueLiteral *trueLit = AST::cast<TrueLiteral *>(expStmt->expression);
+    FalseLiteral *falseLit = AST::cast<FalseLiteral *>(expStmt->expression);
     if (!trueLit && !falseLit) {
         addError(expStmt->firstSourceLocation(), tr("Expected true or false after colon."));
         return false;
@@ -539,20 +557,20 @@ void TypeDescriptionReader::readExports(UiScriptBinding *ast, FakeMetaObject::Pt
         return;
     }
 
-    ExpressionStatement *expStmt = dynamic_cast<ExpressionStatement *>(ast->statement);
+    ExpressionStatement *expStmt = AST::cast<ExpressionStatement *>(ast->statement);
     if (!expStmt) {
         addError(ast->statement->firstSourceLocation(), tr("Expected array of strings after colon."));
         return;
     }
 
-    ArrayLiteral *arrayLit = dynamic_cast<ArrayLiteral *>(expStmt->expression);
+    ArrayPattern *arrayLit = AST::cast<ArrayPattern *>(expStmt->expression);
     if (!arrayLit) {
         addError(expStmt->firstSourceLocation(), tr("Expected array of strings after colon."));
         return;
     }
 
-    for (ElementList *it = arrayLit->elements; it; it = it->next) {
-        StringLiteral *stringLit = dynamic_cast<StringLiteral *>(it->expression);
+    for (PatternElementList *it = arrayLit->elements; it; it = it->next) {
+        StringLiteral *stringLit = AST::cast<StringLiteral *>(it->element->initializer);
         if (!stringLit) {
             addError(arrayLit->firstSourceLocation(), tr("Expected array literal with only string literal members."));
             return;
@@ -585,13 +603,13 @@ void TypeDescriptionReader::readMetaObjectRevisions(UiScriptBinding *ast, FakeMe
         return;
     }
 
-    ExpressionStatement *expStmt = dynamic_cast<ExpressionStatement *>(ast->statement);
+    ExpressionStatement *expStmt = AST::cast<ExpressionStatement *>(ast->statement);
     if (!expStmt) {
         addError(ast->statement->firstSourceLocation(), tr("Expected array of numbers after colon."));
         return;
     }
 
-    ArrayLiteral *arrayLit = dynamic_cast<ArrayLiteral *>(expStmt->expression);
+    ArrayPattern *arrayLit = AST::cast<ArrayPattern *>(expStmt->expression);
     if (!arrayLit) {
         addError(expStmt->firstSourceLocation(), tr("Expected array of numbers after colon."));
         return;
@@ -599,8 +617,8 @@ void TypeDescriptionReader::readMetaObjectRevisions(UiScriptBinding *ast, FakeMe
 
     int exportIndex = 0;
     const int exportCount = fmo->exports().size();
-    for (ElementList *it = arrayLit->elements; it; it = it->next, ++exportIndex) {
-        NumericLiteral *numberLit = cast<NumericLiteral *>(it->expression);
+    for (PatternElementList *it = arrayLit->elements; it; it = it->next, ++exportIndex) {
+        NumericLiteral *numberLit = cast<NumericLiteral *>(it->element->initializer);
         if (!numberLit) {
             addError(arrayLit->firstSourceLocation(), tr("Expected array literal with only number literal members."));
             return;
@@ -624,44 +642,41 @@ void TypeDescriptionReader::readMetaObjectRevisions(UiScriptBinding *ast, FakeMe
 
 void TypeDescriptionReader::readEnumValues(AST::UiScriptBinding *ast, LanguageUtils::FakeMetaEnum *fme)
 {
-    if (!ast || !ast->statement) {
+    if (!ast)
+        return;
+    if (!ast->statement) {
         addError(ast->colonToken, tr("Expected object literal after colon."));
         return;
     }
 
-    ExpressionStatement *expStmt = dynamic_cast<ExpressionStatement *>(ast->statement);
+    auto *expStmt = AST::cast<ExpressionStatement *>(ast->statement);
     if (!expStmt) {
-        addError(ast->statement->firstSourceLocation(), tr("Expected object literal after colon."));
+        addError(ast->statement->firstSourceLocation(), tr("Expected expression after colon."));
         return;
     }
 
-    ObjectLiteral *objectLit = dynamic_cast<ObjectLiteral *>(expStmt->expression);
-    if (!objectLit) {
-        addError(expStmt->firstSourceLocation(), tr("Expected object literal after colon."));
-        return;
-    }
-
-    for (PropertyAssignmentList *it = objectLit->properties; it; it = it->next) {
-        PropertyNameAndValue *assignement = AST::cast<PropertyNameAndValue *>(it->assignment);
-        if (assignement) {
-            StringLiteralPropertyName *propName = dynamic_cast<StringLiteralPropertyName *>(assignement->name);
-            NumericLiteral *value = dynamic_cast<NumericLiteral *>(assignement->value);
-            UnaryMinusExpression *minus = dynamic_cast<UnaryMinusExpression *>(assignement->value);
-            if (minus)
-                value = dynamic_cast<NumericLiteral *>(minus->expression);
-            if (!propName || !value) {
-                addError(objectLit->firstSourceLocation(), tr("Expected object literal to contain only 'string: number' elements."));
-                continue;
+    if (auto *objectLit = AST::cast<ObjectPattern *>(expStmt->expression)) {
+        for (PatternPropertyList *it = objectLit->properties; it; it = it->next) {
+            if (PatternProperty *assignement = it->property) {
+                if (auto *name = AST::cast<StringLiteralPropertyName *>(assignement->name)) {
+                    fme->addKey(name->id.toString());
+                    continue;
+                }
             }
-
-            double v = value->value;
-            if (minus)
-                v = -v;
-            fme->addKey(propName->id.toString(), v);
-            continue;
+            addError(it->firstSourceLocation(), tr("Expected strings as enum keys."));
         }
-        PropertyGetterSetter *getterSetter = AST::cast<PropertyGetterSetter *>(it->assignment);
-        if (getterSetter)
-            addError(objectLit->firstSourceLocation(), tr("Enum should not contain getter and setters, but only 'string: number' elements."));
+    } else if (auto *arrayLit = AST::cast<ArrayPattern *>(expStmt->expression)) {
+        for (PatternElementList *it = arrayLit->elements; it; it = it->next) {
+            if (PatternElement *element = it->element) {
+                if (auto *name = AST::cast<StringLiteral *>(element->initializer)) {
+                    fme->addKey(name->value.toString());
+                    continue;
+                }
+            }
+            addError(it->firstSourceLocation(), tr("Expected strings as enum keys."));
+        }
+    } else {
+        addError(ast->statement->firstSourceLocation(),
+                 tr("Expected either array or object literal as enum definition."));
     }
 }
