@@ -220,12 +220,17 @@ static PyObject *cdbext_listOfLocals(PyObject *, PyObject *args) // -> [ Value ]
         ++currentPartialIname; // skip "local" part
 
         ULONG symbolGroupIndex = 0;
+        ULONG childEndIndex = 0;
         for (;symbolGroupIndex < scopeEnd; ++symbolGroupIndex) {
-            const PyValue value(symbolGroupIndex, symbolGroup);
-            if (value.name() == *currentPartialIname) {
-                PyList_Append(locals, createPythonObject(value));
-                return locals;
+            PyValue value(symbolGroupIndex, symbolGroup);
+            if (childEndIndex <= symbolGroupIndex) { // do not return a child value
+                if (value.name() == *currentPartialIname) {
+                    PyList_Append(locals, createPythonObject(value));
+                    return locals;
+                }
+                ++childEndIndex;
             }
+            childEndIndex += ULONG(value.childCount());
         }
     }
 
@@ -352,7 +357,7 @@ static PyObject *cdbext_reportResult(PyObject *, PyObject *args)
 
 static PyMethodDef cdbextMethods[] = {
     {"parseAndEvaluate",    cdbext_parseAndEvaluate,    METH_VARARGS,
-     "Returns value of expression or None if the expression can not be resolved"},
+     "Returns value of expression or None if the expression cannot be resolved"},
     {"resolveSymbol",       cdbext_resolveSymbol,       METH_VARARGS,
      "Returns a list of symbol names matching the given pattern"},
     {"getNameByAddress",    cdbext_getNameByAddress,    METH_VARARGS,
@@ -360,7 +365,7 @@ static PyMethodDef cdbextMethods[] = {
     {"getAddressByName",    cdbext_getAddressByName,    METH_VARARGS,
      "Returns the address of the symbol with the given name"},
     {"lookupType",          cdbext_lookupType,          METH_VARARGS,
-     "Returns type object or None if the type can not be resolved"},
+     "Returns type object or None if the type cannot be resolved"},
     {"listOfLocals",        cdbext_listOfLocals,        METH_VARARGS,
      "Returns list of values that are currently in scope"},
     {"listOfModules",       cdbext_listOfModules,       METH_NOARGS,
