@@ -700,7 +700,8 @@ static std::string dumplocalHelper(ExtensionCommandContext &exc,PCSTR args, int
         return std::string();
     }
     std::wstring value;
-    if (!dumpSimpleType(n->asSymbolGroupNode(), SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()), &value, &std::string())) {
+    std::string tmp;
+    if (!dumpSimpleType(n->asSymbolGroupNode(), SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()), &value, &tmp)) {
         *errorMessage = "Cannot dump " + iname;
         return std::string();
     }
@@ -819,7 +820,7 @@ extern "C" HRESULT CALLBACK assign(CIDebugClient *client, PCSTR argsIn)
 
     std::string errorMessage;
     bool success = false;
-    AssignEncoding enc = AssignPlainValue;
+    bool encoded = false;
     int token = 0;
     do {
         StringList tokens = commandTokens<StringList>(argsIn, &token);
@@ -829,10 +830,7 @@ extern "C" HRESULT CALLBACK assign(CIDebugClient *client, PCSTR argsIn)
         }
 
         if (tokens.front() == "-h") {
-            enc = AssignHexEncoded;
-            tokens.pop_front();
-        } else if (tokens.front() == "-u") {
-            enc = AssignHexEncodedUtf16;
+            encoded = true;
             tokens.pop_front();
         }
 
@@ -863,7 +861,7 @@ extern "C" HRESULT CALLBACK assign(CIDebugClient *client, PCSTR argsIn)
         SymbolGroup *symGroup = ExtensionContext::instance().symbolGroup(exc.symbols(), exc.threadId(), currentFrame, &errorMessage);
         if (!symGroup)
             break;
-        success = symGroup->assign(iname, enc, value,
+        success = symGroup->assign(iname, encoded ? stringFromHex(value) : value,
                                    SymbolGroupValueContext(exc.dataSpaces(), exc.symbols()),
                                    &errorMessage);
     } while (false);
@@ -964,6 +962,12 @@ extern "C" HRESULT CALLBACK setparameter(CIDebugClient *, PCSTR args)
             } else if (!token.compare(0, equalsPos, "maxArraySize")) {
                 if (integerFromString(value, &ExtensionContext::instance().parameters().maxArraySize))
                     ++success;
+            } else if (!token.compare(0, equalsPos, "firstChance")) {
+                if (integerFromString(value, &ExtensionContext::instance().parameters().firstChanceException))
+                    ++success;
+            } else if (!token.compare(0, equalsPos, "secondChance")) {
+                if (integerFromString(value, &ExtensionContext::instance().parameters().secondChanceException))
+                    ++success;
             } else if (!token.compare(0, equalsPos, "maxStackDepth")) {
                 if (integerFromString(value, &ExtensionContext::instance().parameters().maxStackDepth))
                     ++success;
