@@ -32,6 +32,7 @@
 #include <QLabel>
 #include <QPushButton>
 #include <QSettings>
+#include <QStyle>
 
 /*!
     \class Utils::CheckableMessageBox
@@ -107,6 +108,7 @@ public:
     QCheckBox *checkBox = nullptr;
     QDialogButtonBox *buttonBox = nullptr;
     QAbstractButton *clickedButton = nullptr;
+    QMessageBox::Icon icon = QMessageBox::NoIcon;
 };
 
 CheckableMessageBox::CheckableMessageBox(QWidget *parent) :
@@ -148,17 +150,53 @@ void CheckableMessageBox::setText(const QString &t)
     d->messageLabel->setText(t);
 }
 
-QPixmap CheckableMessageBox::iconPixmap() const
+QMessageBox::Icon CheckableMessageBox::icon() const
 {
-    if (const QPixmap *p = d->pixmapLabel->pixmap())
-        return QPixmap(*p);
+    return d->icon;
+}
+
+// See QMessageBoxPrivate::standardIcon
+static QPixmap pixmapForIcon(QMessageBox::Icon icon, QWidget *w)
+{
+    const QStyle *style = w ? w->style() : QApplication::style();
+    const int iconSize = style->pixelMetric(QStyle::PM_MessageBoxIconSize, nullptr, w);
+    QIcon tmpIcon;
+    switch (icon) {
+    case QMessageBox::Information:
+        tmpIcon = style->standardIcon(QStyle::SP_MessageBoxInformation, nullptr, w);
+        break;
+    case QMessageBox::Warning:
+        tmpIcon = style->standardIcon(QStyle::SP_MessageBoxWarning, nullptr, w);
+        break;
+    case QMessageBox::Critical:
+        tmpIcon = style->standardIcon(QStyle::SP_MessageBoxCritical, nullptr, w);
+        break;
+    case QMessageBox::Question:
+        tmpIcon = style->standardIcon(QStyle::SP_MessageBoxQuestion, nullptr, w);
+        break;
+    default:
+        break;
+    }
+    if (!tmpIcon.isNull()) {
+        QWindow *window = nullptr;
+        if (w) {
+            window = w->windowHandle();
+            if (!window) {
+                if (const QWidget *nativeParent = w->nativeParentWidget())
+                    window = nativeParent->windowHandle();
+            }
+        }
+        return tmpIcon.pixmap(window, QSize(iconSize, iconSize));
+    }
     return QPixmap();
 }
 
-void CheckableMessageBox::setIconPixmap(const QPixmap &p)
+void CheckableMessageBox::setIcon(QMessageBox::Icon icon)
 {
-    d->pixmapLabel->setPixmap(p);
-    d->pixmapLabel->setVisible(!p.isNull());
+    d->icon = icon;
+    const QPixmap pixmap = pixmapForIcon(icon, this);
+    d->pixmapLabel->setPixmap(pixmap);
+    d->pixmapLabel->setVisible(!pixmap.isNull());
 }
 
 bool CheckableMessageBox::isChecked() const
@@ -239,7 +277,7 @@ CheckableMessageBox::question(QWidget *parent,
 {
     CheckableMessageBox mb(parent);
     mb.setWindowTitle(title);
-    mb.setIconPixmap(QMessageBox::standardIcon(QMessageBox::Question));
+    mb.setIcon(QMessageBox::Question);
     mb.setText(question);
     mb.setCheckBoxText(checkBoxText);
     mb.setChecked(*checkBoxSetting);
@@ -261,7 +299,7 @@ CheckableMessageBox::information(QWidget *parent,
 {
     CheckableMessageBox mb(parent);
     mb.setWindowTitle(title);
-    mb.setIconPixmap(QMessageBox::standardIcon(QMessageBox::Information));
+    mb.setIcon(QMessageBox::Information);
     mb.setText(text);
     mb.setCheckBoxText(checkBoxText);
     mb.setChecked(*checkBoxSetting);
@@ -277,7 +315,7 @@ QMessageBox::StandardButton CheckableMessageBox::dialogButtonBoxToMessageBoxButt
     return static_cast<QMessageBox::StandardButton>(int(db));
 }
 
-bool askAgain(QSettings *settings, const QString &settingsSubKey)
+bool CheckableMessageBox::shouldAskAgain(QSettings *settings, const QString &settingsSubKey)
 {
     if (QTC_GUARD(settings)) {
         settings->beginGroup(QLatin1String(kDoNotAskAgainKey));
@@ -297,9 +335,7 @@ void initDoNotAskAgainMessageBox(CheckableMessageBox &messageBox, const QString
                                  DoNotAskAgainType type)
 {
     messageBox.setWindowTitle(title);
-    messageBox.setIconPixmap(QMessageBox::standardIcon(type == Information
-                                               ? QMessageBox::Information
-                                               : QMessageBox::Question));
+    messageBox.setIcon(type == Information ? QMessageBox::Information : QMessageBox::Question);
     messageBox.setText(text);
     messageBox.setCheckBoxVisible(true);
     messageBox.setCheckBoxText(type == Information ? CheckableMessageBox::msgDoNotShowAgain()
@@ -309,7 +345,7 @@ void initDoNotAskAgainMessageBox(CheckableMessageBox &messageBox, const QString
     messageBox.setDefaultButton(defaultButton);
 }
 
-void doNotAskAgain(QSettings *settings, const QString &settingsSubKey)
+void CheckableMessageBox::doNotAskAgain(QSettings *settings, const QString &settingsSubKey)
 {
     if (!settings)
         return;
@@ -337,7 +373,7 @@ CheckableMessageBox::doNotAskAgainQuestion(QWidget *parent, const QString &title
                                            QDialogButtonBox::StandardButton acceptButton)
 
 {
-    if (!askAgain(settings, settingsSubKey))
+    if (!shouldAskAgain(settings, settingsSubKey))
         return acceptButton;
 
     CheckableMessageBox messageBox(parent);
@@ -365,7 +401,7 @@ CheckableMessageBox::doNotShowAgainInformation(QWidget *parent, const QString &t
                                            QDialogButtonBox::StandardButton defaultButton)
 
 {
-    if (!askAgain(settings, settingsSubKey))
+    if (!shouldAskAgain(settings, settingsSubKey))
             return defaultButton;
 
     CheckableMessageBox messageBox(parent);
