@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,33 +9,30 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
+#include "algorithm.h"
 #include "filesearch.h"
-#include "runextensions.h"
+#include "fileutils.h"
+#include "mapreduce.h"
 
 #include <QCoreApplication>
 #include <QMutex>
+#include <QRegExp>
 #include <QRegularExpression>
 #include <QTextCodec>
-#include <QtConcurrentMap>
 
 #include <cctype>
 
@@ -45,14 +42,14 @@ static inline QString msgCanceled(const QString &searchTerm, int numMatches, int
 {
     return QCoreApplication::translate("Utils::FileSearch",
                                        "%1: canceled. %n occurrences found in %2 files.",
-                                       0, numMatches).arg(searchTerm).arg(numFilesSearched);
+                                       nullptr, numMatches).arg(searchTerm).arg(numFilesSearched);
 }
 
 static inline QString msgFound(const QString &searchTerm, int numMatches, int numFilesSearched)
 {
     return QCoreApplication::translate("Utils::FileSearch",
                                        "%1: %n occurrences found in %2 files.",
-                                       0, numMatches).arg(searchTerm).arg(numFilesSearched);
+                                       nullptr, numMatches).arg(searchTerm).arg(numFilesSearched);
 }
 
 namespace {
@@ -88,13 +85,12 @@ class FileSearch
 {
 public:
     FileSearch(const QString &searchTerm, QTextDocument::FindFlags flags,
-               QMap<QString, QString> fileToContentsMap,
-               QFutureInterface<FileSearchResultList> *futureInterface);
-    const FileSearchResultList operator()(const FileIterator::Item &item) const;
+               const QMap<QString, QString> &fileToContentsMap);
+    void operator()(QFutureInterface<FileSearchResultList> &futureInterface,
+                    const FileIterator::Item &item) const;
 
 private:
     QMap<QString, QString> fileToContentsMap;
-    QFutureInterface<FileSearchResultList> *future;
     QString searchTermLower;
     QString searchTermUpper;
     int termMaxIndex;
@@ -109,27 +105,25 @@ class FileSearchRegExp
 {
 public:
     FileSearchRegExp(const QString &searchTerm, QTextDocument::FindFlags flags,
-                     QMap<QString, QString> fileToContentsMap,
-                     QFutureInterface<FileSearchResultList> *futureInterface);
-    const FileSearchResultList operator()(const FileIterator::Item &item) const;
+                     const QMap<QString, QString> &fileToContentsMap);
+    FileSearchRegExp(const FileSearchRegExp &other);
+    void operator()(QFutureInterface<FileSearchResultList> &futureInterface,
+                    const FileIterator::Item &item) const;
 
 private:
     QRegularExpressionMatch doGuardedMatch(const QString &line, int offset) const;
 
     QMap<QString, QString> fileToContentsMap;
-    QFutureInterface<FileSearchResultList> *future;
     QRegularExpression expression;
     mutable QMutex mutex;
 };
 
 FileSearch::FileSearch(const QString &searchTerm, QTextDocument::FindFlags flags,
-                       QMap<QString, QString> fileToContentsMap,
-                       QFutureInterface<FileSearchResultList> *futureInterface)
+                       const QMap<QString, QString> &fileToContentsMap)
 {
     this->fileToContentsMap = fileToContentsMap;
     caseSensitive = (flags & QTextDocument::FindCaseSensitively);
     wholeWord = (flags & QTextDocument::FindWholeWords);
-    future = futureInterface;
     searchTermLower = searchTerm.toLower();
     searchTermUpper = searchTerm.toUpper();
     termMaxIndex = searchTerm.length() - 1;
@@ -138,16 +132,21 @@ FileSearch::FileSearch(const QString &searchTerm, QTextDocument::FindFlags flags
     termDataUpper = searchTermUpper.constData();
 }
 
-const FileSearchResultList FileSearch::operator()(const FileIterator::Item &item) const
+void FileSearch::operator()(QFutureInterface<FileSearchResultList> &futureInterface,
+                            const FileIterator::Item &item) const
 {
+    if (futureInterface.isCanceled())
+        return;
+    futureInterface.setProgressRange(0, 1);
+    futureInterface.setProgressValue(0);
     FileSearchResultList results;
-    if (future->isCanceled())
-        return results;
     QFile file;
     QTextStream stream;
     QString tempString;
-    if (!openStream(item.filePath, item.encoding, &stream, &file, &tempString, fileToContentsMap))
-        return results;
+    if (!openStream(item.filePath, item.encoding, &stream, &file, &tempString, fileToContentsMap)) {
+        futureInterface.cancel(); // failure
+        return;
+    }
     int lineNr = 0;
 
     while (!stream.atEnd()) {
@@ -212,22 +211,23 @@ const FileSearchResultList FileSearch::operator()(const FileIterator::Item &item
                 }
             }
         }
-        if (future->isPaused())
-            future->waitForResume();
-        if (future->isCanceled())
+        if (futureInterface.isPaused())
+            futureInterface.waitForResume();
+        if (futureInterface.isCanceled())
             break;
     }
     if (file.isOpen())
         file.close();
-    return results;
+    if (!futureInterface.isCanceled()) {
+        futureInterface.reportResult(results);
+        futureInterface.setProgressValue(1);
+    }
 }
 
 FileSearchRegExp::FileSearchRegExp(const QString &searchTerm, QTextDocument::FindFlags flags,
-                                   QMap<QString, QString> fileToContentsMap,
-                                   QFutureInterface<FileSearchResultList> *futureInterface)
+                                   const QMap<QString, QString> &fileToContentsMap)
 {
     this->fileToContentsMap = fileToContentsMap;
-    future = futureInterface;
     QString term = searchTerm;
     if (flags & QTextDocument::FindWholeWords)
         term = QString::fromLatin1("\\b%1\\b").arg(term);
@@ -236,22 +236,33 @@ FileSearchRegExp::FileSearchRegExp(const QString &searchTerm, QTextDocument::Fin
     expression = QRegularExpression(term, patternOptions);
 }
 
+FileSearchRegExp::FileSearchRegExp(const FileSearchRegExp &other)
+    : fileToContentsMap(other.fileToContentsMap),
+      expression(other.expression)
+{
+}
+
 QRegularExpressionMatch FileSearchRegExp::doGuardedMatch(const QString &line, int offset) const
 {
     QMutexLocker lock(&mutex);
     return expression.match(line, offset);
 }
 
-const FileSearchResultList FileSearchRegExp::operator()(const FileIterator::Item &item) const
+void FileSearchRegExp::operator()(QFutureInterface<FileSearchResultList> &futureInterface,
+                                  const FileIterator::Item &item) const
 {
+    if (futureInterface.isCanceled())
+        return;
+    futureInterface.setProgressRange(0, 1);
+    futureInterface.setProgressValue(0);
     FileSearchResultList results;
-    if (future->isCanceled())
-        return results;
     QFile file;
     QTextStream stream;
     QString tempString;
-    if (!openStream(item.filePath, item.encoding, &stream, &file, &tempString, fileToContentsMap))
-        return results;
+    if (!openStream(item.filePath, item.encoding, &stream, &file, &tempString, fileToContentsMap)) {
+        futureInterface.cancel(); // failure
+        return;
+    }
     int lineNr = 0;
 
     QString line;
@@ -273,160 +284,114 @@ const FileSearchResultList FileSearchRegExp::operator()(const FileIterator::Item
             if (pos >= lengthOfLine)
                 break;
         }
-        if (future->isPaused())
-            future->waitForResume();
-        if (future->isCanceled())
+        if (futureInterface.isPaused())
+            futureInterface.waitForResume();
+        if (futureInterface.isCanceled())
             break;
     }
     if (file.isOpen())
         file.close();
-    return results;
+    if (!futureInterface.isCanceled()) {
+        futureInterface.reportResult(results);
+        futureInterface.setProgressValue(1);
+    }
 }
 
-class RunFileSearch
+struct SearchState
 {
-public:
-    RunFileSearch(QFutureInterface<FileSearchResultList> &future,
-                  const QString &searchTerm,
-                  FileIterator *files,
-                  const std::function<FileSearchResultList(FileIterator::Item)> &searchFunction);
-
-    void run();
-    void collect(const FileSearchResultList &results);
-
-private:
-    QFutureInterface<FileSearchResultList> &m_future;
-    QString m_searchTerm;
-    FileIterator *m_files;
-    std::function<FileSearchResultList(FileIterator::Item)> m_searchFunction;
-
-    int m_numFilesSearched;
-    int m_numMatches;
-    FileSearchResultList m_results;
-    bool m_canceled;
+    SearchState(const QString &term, FileIterator *iterator) : searchTerm(term), files(iterator) {}
+    QString searchTerm;
+    FileIterator *files = nullptr;
+    FileSearchResultList cachedResults;
+    int numFilesSearched = 0;
+    int numMatches = 0;
 };
 
-RunFileSearch::RunFileSearch(QFutureInterface<FileSearchResultList> &future,
-                             const QString &searchTerm, FileIterator *files,
-                             const std::function<FileSearchResultList (FileIterator::Item)> &searchFunction)
-    : m_future(future),
-      m_searchTerm(searchTerm),
-      m_files(files),
-      m_searchFunction(searchFunction),
-      m_numFilesSearched(0),
-      m_numMatches(0),
-      m_canceled(false)
+SearchState initFileSearch(QFutureInterface<FileSearchResultList> &futureInterface,
+                           const QString &searchTerm, FileIterator *files)
 {
-    m_future.setProgressRange(0, m_files->maxProgress());
-    m_future.setProgressValueAndText(m_files->currentProgress(), msgFound(m_searchTerm,
-                                                                          m_numMatches,
-                                                                          m_numFilesSearched));
+    futureInterface.setProgressRange(0, files->maxProgress());
+    futureInterface.setProgressValueAndText(files->currentProgress(), msgFound(searchTerm, 0, 0));
+    return SearchState(searchTerm, files);
 }
 
-void RunFileSearch::run()
+void collectSearchResults(QFutureInterface<FileSearchResultList> &futureInterface,
+                          SearchState &state,
+                          const FileSearchResultList &results)
 {
-    // This thread waits for blockingMappedReduced to finish, so reduce the pool's used thread count
-    // so the blockingMappedReduced can use one more thread, and increase it again afterwards.
-    QThreadPool::globalInstance()->releaseThread();
-    QtConcurrent::blockingMappedReduced<FileSearchResultList>(m_files->begin(), m_files->end(),
-                                                  m_searchFunction,
-                                                  [this](FileSearchResultList &, const FileSearchResultList &results) {
-                                                      collect(results);
-                                                  },
-                                                  QtConcurrent::OrderedReduce | QtConcurrent::SequentialReduce);
-    QThreadPool::globalInstance()->reserveThread();
-    if (!m_results.isEmpty()) {
-        m_future.reportResult(m_results);
-        m_results.clear();
-    }
-    if (!m_future.isCanceled())
-        m_future.setProgressValueAndText(m_files->currentProgress(), msgFound(m_searchTerm,
-                                                                              m_numMatches,
-                                                                              m_numFilesSearched));
-    delete m_files;
-    if (m_future.isPaused())
-        m_future.waitForResume();
-}
-
-void RunFileSearch::collect(const FileSearchResultList &results)
-{
-    if (m_future.isCanceled()) {
-        if (!m_canceled) {
-            m_future.setProgressValueAndText(m_files->currentProgress(),
-                                             msgCanceled(m_searchTerm,
-                                                         m_numMatches,
-                                                         m_numFilesSearched));
-            m_canceled = true;
+    state.numMatches += results.size();
+    state.cachedResults << results;
+    state.numFilesSearched += 1;
+    if (futureInterface.isProgressUpdateNeeded()
+            || futureInterface.progressValue() == 0 /*workaround for regression in Qt*/) {
+        if (!state.cachedResults.isEmpty()) {
+            futureInterface.reportResult(state.cachedResults);
+            state.cachedResults.clear();
         }
-        return;
-    }
-    m_numMatches += results.size();
-    m_results << results;
-    ++m_numFilesSearched;
-    if (m_future.isProgressUpdateNeeded()
-            || m_future.progressValue() == 0 /*workaround for regression in Qt*/) {
-        if (!m_results.isEmpty()) {
-            m_future.reportResult(m_results);
-            m_results.clear();
-        }
-        m_future.setProgressRange(0, m_files->maxProgress());
-        m_future.setProgressValueAndText(m_files->currentProgress(), msgFound(m_searchTerm,
-                                                                              m_numMatches,
-                                                                              m_numFilesSearched));
+        futureInterface.setProgressRange(0, state.files->maxProgress());
+        futureInterface.setProgressValueAndText(state.files->currentProgress(),
+                                                 msgFound(state.searchTerm,
+                                                          state.numMatches,
+                                                          state.numFilesSearched));
     }
 }
 
-void runFileSearch(QFutureInterface<FileSearchResultList> &future,
-                   QString searchTerm,
-                   FileIterator *files,
-                   QTextDocument::FindFlags flags,
-                   QMap<QString, QString> fileToContentsMap)
+void cleanUpFileSearch(QFutureInterface<FileSearchResultList> &futureInterface,
+                       SearchState &state)
 {
-    FileSearch searchFunction(searchTerm, flags, fileToContentsMap, &future);
-    RunFileSearch search(future, searchTerm, files, std::bind(&FileSearch::operator(),
-                                                              &searchFunction,
-                                                              std::placeholders::_1));
-    search.run();
-}
-
-void runFileSearchRegExp(QFutureInterface<FileSearchResultList> &future,
-                   QString searchTerm,
-                   FileIterator *files,
-                   QTextDocument::FindFlags flags,
-                   QMap<QString, QString> fileToContentsMap)
-{
-    FileSearchRegExp searchFunction(searchTerm, flags, fileToContentsMap, &future);
-    RunFileSearch search(future, searchTerm, files, std::bind(&FileSearchRegExp::operator(),
-                                                              &searchFunction,
-                                                              std::placeholders::_1));
-    search.run();
+    if (!state.cachedResults.isEmpty()) {
+        futureInterface.reportResult(state.cachedResults);
+        state.cachedResults.clear();
+    }
+    if (futureInterface.isCanceled()) {
+        futureInterface.setProgressValueAndText(state.files->currentProgress(),
+                                                msgCanceled(state.searchTerm,
+                                                            state.numMatches,
+                                                            state.numFilesSearched));
+    } else {
+        futureInterface.setProgressValueAndText(state.files->currentProgress(),
+                                                msgFound(state.searchTerm,
+                                                         state.numMatches,
+                                                         state.numFilesSearched));
+    }
+    delete state.files;
 }
 
 } // namespace
 
-
 QFuture<FileSearchResultList> Utils::findInFiles(const QString &searchTerm, FileIterator *files,
-    QTextDocument::FindFlags flags, QMap<QString, QString> fileToContentsMap)
+    QTextDocument::FindFlags flags, const QMap<QString, QString> &fileToContentsMap)
 {
-    return QtConcurrent::run<FileSearchResultList, QString, FileIterator *, QTextDocument::FindFlags, QMap<QString, QString> >
-            (runFileSearch, searchTerm, files, flags, fileToContentsMap);
+    return mapReduce(files->begin(), files->end(),
+                     [searchTerm, files](QFutureInterface<FileSearchResultList> &futureInterface) {
+                         return initFileSearch(futureInterface, searchTerm, files);
+                     },
+                     FileSearch(searchTerm, flags, fileToContentsMap),
+                     &collectSearchResults,
+                     &cleanUpFileSearch);
 }
 
 QFuture<FileSearchResultList> Utils::findInFilesRegExp(const QString &searchTerm, FileIterator *files,
-    QTextDocument::FindFlags flags, QMap<QString, QString> fileToContentsMap)
+    QTextDocument::FindFlags flags, const QMap<QString, QString> &fileToContentsMap)
 {
-    return QtConcurrent::run<FileSearchResultList, QString, FileIterator *, QTextDocument::FindFlags, QMap<QString, QString> >
-            (runFileSearchRegExp, searchTerm, files, flags, fileToContentsMap);
+    return mapReduce(files->begin(), files->end(),
+                     [searchTerm, files](QFutureInterface<FileSearchResultList> &futureInterface) {
+                         return initFileSearch(futureInterface, searchTerm, files);
+                     },
+                     FileSearchRegExp(searchTerm, flags, fileToContentsMap),
+                     &collectSearchResults,
+                     &cleanUpFileSearch);
 }
 
 QString Utils::expandRegExpReplacement(const QString &replaceText, const QStringList &capturedTexts)
 {
-    // handles \1 \\ \& & \t \n
+    // handles \1 \\ \& \t \n $1 $$ $&
     QString result;
     const int numCaptures = capturedTexts.size() - 1;
-    for (int i = 0; i < replaceText.length(); ++i) {
+    const int replaceLength = replaceText.length();
+    for (int i = 0; i < replaceLength; ++i) {
         QChar c = replaceText.at(i);
-        if (c == QLatin1Char('\\') && i < replaceText.length() - 1) {
+        if (c == QLatin1Char('\\') && i < replaceLength - 1) {
             c = replaceText.at(++i);
             if (c == QLatin1Char('\\')) {
                 result += QLatin1Char('\\');
@@ -440,16 +405,26 @@ QString Utils::expandRegExpReplacement(const QString &replaceText, const QString
                 int index = c.unicode()-'1';
                 if (index < numCaptures) {
                     result += capturedTexts.at(index+1);
-                } else {
-                    result += QLatin1Char('\\');
-                    result += c;
-                }
+                } // else add nothing
             } else {
                 result += QLatin1Char('\\');
                 result += c;
             }
-        } else if (c == QLatin1Char('&')) {
-            result += capturedTexts.at(0);
+        } else if (c== '$' && i < replaceLength - 1) {
+            c = replaceText.at(++i);
+            if (c == '$') {
+                result += '$';
+            } else if (c == '&') {
+                result += capturedTexts.at(0);
+            } else if (c.isDigit()) {
+                int index = c.unicode()-'1';
+                if (index < numCaptures) {
+                    result += capturedTexts.at(index+1);
+                } // else add nothing
+            } else {
+                result += '$';
+                result += c;
+            }
         } else {
             result += c;
         }
@@ -477,7 +452,7 @@ QString matchCaseReplacement(const QString &originalText, const QString &replace
             restIsUpperCase = false;
 
         if (!restIsLowerCase && !restIsUpperCase)
-            return replaceText; // mixed
+            break;
     }
 
     if (restIsLowerCase) {
@@ -485,17 +460,86 @@ QString matchCaseReplacement(const QString &originalText, const QString &replace
         if (firstIsUpperCase)
             res.replace(0, 1, res.at(0).toUpper());
         return res;
-    }
-
-    if (restIsUpperCase) {
+    } else if (restIsUpperCase) {
         QString res = replaceText.toUpper();
         if (firstIsLowerCase)
             res.replace(0, 1, res.at(0).toLower());
         return res;
+    } else {
+        return replaceText; // mixed
     }
-
-    return replaceText;         // mixed
 }
+} // namespace
+
+static QList<QRegExp> filtersToRegExps(const QStringList &filters)
+{
+    return Utils::transform(filters, [](const QString &filter) {
+        return QRegExp(filter, Qt::CaseInsensitive, QRegExp::Wildcard);
+    });
+}
+
+static bool matches(const QList<QRegExp> &exprList, const QString &filePath)
+{
+    return Utils::anyOf(exprList, [&filePath](QRegExp reg) {
+        return (reg.exactMatch(filePath)
+                || reg.exactMatch(FilePath::fromString(filePath).fileName()));
+    });
+}
+
+static bool isFileIncluded(const QList<QRegExp> &filterRegs, const QList<QRegExp> &exclusionRegs,
+                           const QString &filePath)
+{
+    const bool isIncluded = filterRegs.isEmpty() || matches(filterRegs, filePath);
+    return isIncluded && (exclusionRegs.isEmpty() || !matches(exclusionRegs, filePath));
+}
+
+std::function<bool(const QString &)>
+filterFileFunction(const QStringList &filters, const QStringList &exclusionFilters)
+{
+    const QList<QRegExp> filterRegs = filtersToRegExps(filters);
+    const QList<QRegExp> exclusionRegs = filtersToRegExps(exclusionFilters);
+    return [filterRegs, exclusionRegs](const QString &filePath) {
+        return isFileIncluded(filterRegs, exclusionRegs, filePath);
+    };
+}
+
+std::function<QStringList(const QStringList &)>
+filterFilesFunction(const QStringList &filters, const QStringList &exclusionFilters)
+{
+    const QList<QRegExp> filterRegs = filtersToRegExps(filters);
+    const QList<QRegExp> exclusionRegs = filtersToRegExps(exclusionFilters);
+    return [filterRegs, exclusionRegs](const QStringList &filePaths) {
+        return Utils::filtered(filePaths, [&filterRegs, &exclusionRegs](const QString &filePath) {
+            return isFileIncluded(filterRegs, exclusionRegs, filePath);
+        });
+    };
+}
+
+QStringList splitFilterUiText(const QString &text)
+{
+    const QStringList parts = text.split(',');
+    const QStringList trimmedPortableParts = Utils::transform(parts, [](const QString &s) {
+        return QDir::fromNativeSeparators(s.trimmed());
+    });
+    return Utils::filtered(trimmedPortableParts, [](const QString &s) { return !s.isEmpty(); });
+}
+
+
+QString msgFilePatternLabel()
+{
+    return QCoreApplication::translate("Utils::FileSearch", "Fi&le pattern:");
+}
+
+QString msgExclusionPatternLabel()
+{
+    return QCoreApplication::translate("Utils::FileSearch", "Excl&usion pattern:");
+}
+
+QString msgFilePatternToolTip()
+{
+    return QCoreApplication::translate("Utils::FileSearch",
+                                       "List of comma separated wildcard filters. "
+                                       "Files with file name or full file path matching any filter are included.");
 }
 
 QString matchCaseReplacement(const QString &originalText, const QString &replaceText)
@@ -527,46 +571,45 @@ QString matchCaseReplacement(const QString &originalText, const QString &replace
 
 // #pragma mark -- FileIterator
 
-void FileIterator::next(FileIterator::const_iterator *it)
+void FileIterator::advance(FileIterator::const_iterator *it) const
 {
     if (it->m_index < 0) // == end
         return;
     ++it->m_index;
-    update(it->m_index);
-    if (it->m_index < currentFileCount()) {
-        it->m_item.filePath = fileAt(it->m_index);
-        it->m_item.encoding = codecAt(it->m_index);
-    } else {
+    const_cast<FileIterator *>(this)->update(it->m_index);
+    if (it->m_index >= currentFileCount())
         it->m_index = -1; // == end
-        it->m_item.filePath.clear();
-        it->m_item.encoding = 0;
-    }
 }
 
-FileIterator::const_iterator FileIterator::begin()
+FileIterator::const_iterator FileIterator::begin() const
 {
-    update(0);
+    const_cast<FileIterator *>(this)->update(0);
     if (currentFileCount() == 0)
         return end();
-    return FileIterator::const_iterator(this,
-                                        FileIterator::Item(fileAt(0), codecAt(0)),
-                                        0/*index*/);
+    return FileIterator::const_iterator(this, 0/*index*/);
 }
 
-FileIterator::const_iterator FileIterator::end()
+FileIterator::const_iterator FileIterator::end() const
 {
-    return FileIterator::const_iterator(this, FileIterator::Item(QString(), 0),
-                                        -1/*end*/);
+    return FileIterator::const_iterator(this, -1/*end*/);
 }
 
 // #pragma mark -- FileListIterator
 
+QTextCodec *encodingAt(const QList<QTextCodec *> &encodings, int index)
+{
+    if (index >= 0 && index < encodings.size())
+        return encodings.at(index);
+    return QTextCodec::codecForLocale();
+}
+
 FileListIterator::FileListIterator(const QStringList &fileList,
                                    const QList<QTextCodec *> encodings)
-    : m_files(fileList),
-      m_encodings(encodings),
-      m_maxIndex(-1)
+    : m_maxIndex(-1)
 {
+    m_items.reserve(fileList.size());
+    for (int i = 0; i < fileList.size(); ++i)
+        m_items.append(Item(fileList.at(i), encodingAt(encodings, i)));
 }
 
 void FileListIterator::update(int requestedIndex)
@@ -577,22 +620,17 @@ void FileListIterator::update(int requestedIndex)
 
 int FileListIterator::currentFileCount() const
 {
-    return m_files.size();
+    return m_items.size();
 }
 
-QString FileListIterator::fileAt(int index) const
+const FileIterator::Item &FileListIterator::itemAt(int index) const
 {
-    return m_files.at(index);
-}
-
-QTextCodec *FileListIterator::codecAt(int index) const
-{
-    return m_encodings.at(index);
+    return m_items.at(index);
 }
 
 int FileListIterator::maxProgress() const
 {
-    return m_files.size();
+    return m_items.size();
 }
 
 int FileListIterator::currentProgress() const
@@ -600,13 +638,6 @@ int FileListIterator::currentProgress() const
     return m_maxIndex + 1;
 }
 
-QTextCodec *FileListIterator::encodingAt(int index) const
-{
-    if (index >= 0 && index < m_encodings.size())
-        return m_encodings.at(index);
-    return QTextCodec::codecForLocale();
-}
-
 // #pragma mark -- SubDirFileIterator
 
 namespace {
@@ -614,79 +645,94 @@ namespace {
 }
 
 SubDirFileIterator::SubDirFileIterator(const QStringList &directories, const QStringList &filters,
-                                       QTextCodec *encoding)
-    : m_filters(filters), m_progress(0)
+                                       const QStringList &exclusionFilters, QTextCodec *encoding)
+    : m_filterFiles(filterFilesFunction(filters, exclusionFilters)),
+      m_progress(0)
 {
-    m_encoding = (encoding == 0 ? QTextCodec::codecForLocale() : encoding);
+    m_encoding = (encoding == nullptr ? QTextCodec::codecForLocale() : encoding);
     qreal maxPer = qreal(MAX_PROGRESS) / directories.count();
-    foreach (const QString &directoryEntry, directories) {
+    for (const QString &directoryEntry : directories) {
         if (!directoryEntry.isEmpty()) {
-            m_dirs.push(QDir(directoryEntry));
-            m_progressValues.push(maxPer);
-            m_processedValues.push(false);
+            const QDir dir(directoryEntry);
+            const QString canonicalPath = dir.canonicalPath();
+            if (!canonicalPath.isEmpty() && dir.exists()) {
+                m_dirs.push(dir);
+                m_knownDirs.insert(canonicalPath);
+                m_progressValues.push(maxPer);
+                m_processedValues.push(false);
+            }
         }
     }
 }
 
+SubDirFileIterator::~SubDirFileIterator()
+{
+    qDeleteAll(m_items);
+}
+
 void SubDirFileIterator::update(int index)
 {
-    if (index < m_files.size())
+    if (index < m_items.size())
         return;
     // collect files from the directories until we have enough for the given index
-    while (!m_dirs.isEmpty() && index >= m_files.size()) {
+    while (!m_dirs.isEmpty() && index >= m_items.size()) {
         QDir dir = m_dirs.pop();
         const qreal dirProgressMax = m_progressValues.pop();
         const bool processed = m_processedValues.pop();
         if (dir.exists()) {
-            QStringList subDirs;
-            if (!processed)
-                subDirs = dir.entryList(QDir::Dirs|QDir::Hidden|QDir::NoDotAndDotDot);
-            if (subDirs.isEmpty()) {
-                QStringList fileEntries = dir.entryList(m_filters,
-                    QDir::Files|QDir::Hidden);
-                QStringListIterator it(fileEntries);
-                it.toBack();
-                while (it.hasPrevious()) {
-                    const QString &file = it.previous();
-                    m_files.append(dir.path()+ QLatin1Char('/') +file);
+            const QString dirPath = dir.path();
+            using Dir = QString;
+            using CanonicalDir = QString;
+            std::vector<std::pair<Dir, CanonicalDir>> subDirs;
+            if (!processed) {
+                for (const QFileInfo &info :
+                     dir.entryInfoList(QDir::Dirs | QDir::Hidden | QDir::NoDotAndDotDot)) {
+                    const QString canonicalDir = info.canonicalFilePath();
+                    if (!m_knownDirs.contains(canonicalDir))
+                        subDirs.emplace_back(info.filePath(), canonicalDir);
                 }
+            }
+            if (subDirs.empty()) {
+                const QStringList allFileEntries = dir.entryList(QDir::Files|QDir::Hidden);
+                const QStringList allFilePaths = Utils::transform(allFileEntries,
+                                                                  [&dirPath](const QString &entry) {
+                    return QString(dirPath + '/' + entry);
+                });
+                const QStringList filePaths = m_filterFiles(allFilePaths);
+                m_items.reserve(m_items.size() + filePaths.size());
+                Utils::reverseForeach(filePaths, [this](const QString &file) {
+                    m_items.append(new Item(file, m_encoding));
+                });
                 m_progress += dirProgressMax;
             } else {
                 qreal subProgress = dirProgressMax/(subDirs.size()+1);
                 m_dirs.push(dir);
                 m_progressValues.push(subProgress);
                 m_processedValues.push(true);
-                QStringListIterator it(subDirs);
-                it.toBack();
-                while (it.hasPrevious()) {
-                    const QString &directory = it.previous();
-                    m_dirs.push(QDir(dir.path()+ QLatin1Char('/') + directory));
-                    m_progressValues.push(subProgress);
-                    m_processedValues.push(false);
-                }
+                Utils::reverseForeach(subDirs,
+                                      [this, subProgress](const std::pair<Dir, CanonicalDir> &dir) {
+                                          m_dirs.push(QDir(dir.first));
+                                          m_knownDirs.insert(dir.second);
+                                          m_progressValues.push(subProgress);
+                                          m_processedValues.push(false);
+                                      });
             }
         } else {
             m_progress += dirProgressMax;
         }
     }
-    if (index >= m_files.size())
+    if (index >= m_items.size())
         m_progress = MAX_PROGRESS;
 }
 
 int SubDirFileIterator::currentFileCount() const
 {
-    return m_files.size();
+    return m_items.size();
 }
 
-QString SubDirFileIterator::fileAt(int index) const
+const FileIterator::Item &SubDirFileIterator::itemAt(int index) const
 {
-    return m_files.at(index);
-}
-
-QTextCodec *SubDirFileIterator::codecAt(int index) const
-{
-    Q_UNUSED(index)
-    return m_encoding;
+    return *m_items.at(index);
 }
 
 int SubDirFileIterator::maxProgress() const
