@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,25 +9,22 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "filesystemwatcher.h"
+#include "globalfilechangeblocker.h"
 
 #include <QDebug>
 #include <QDir>
@@ -91,32 +88,32 @@ class FileSystemWatcherStaticData
 {
 public:
     FileSystemWatcherStaticData() :
-        maxFileOpen(getFileLimit()) , m_objectCount(0), m_watcher(0) {}
+        maxFileOpen(getFileLimit()) {}
 
     quint64 maxFileOpen;
-    int m_objectCount;
+    int m_objectCount = 0;
     QHash<QString, int> m_fileCount;
     QHash<QString, int> m_directoryCount;
-    QFileSystemWatcher *m_watcher;
+    QFileSystemWatcher *m_watcher = nullptr;
 };
 
-typedef QMap<int, FileSystemWatcherStaticData> FileSystemWatcherStaticDataMap;
+using FileSystemWatcherStaticDataMap = QMap<int, FileSystemWatcherStaticData>;
 
 Q_GLOBAL_STATIC(FileSystemWatcherStaticDataMap, fileSystemWatcherStaticDataMap)
 
 class WatchEntry
 {
 public:
-    typedef FileSystemWatcher::WatchMode WatchMode;
+    using WatchMode = FileSystemWatcher::WatchMode;
 
     explicit WatchEntry(const QString &file, WatchMode wm) :
         watchMode(wm), modifiedTime(QFileInfo(file).lastModified()) {}
 
-    WatchEntry() : watchMode(FileSystemWatcher::WatchAllChanges) {}
+    WatchEntry() = default;
 
     bool trigger(const QString &fileName);
 
-    WatchMode watchMode;
+    WatchMode watchMode = FileSystemWatcher::WatchAllChanges;
     QDateTime modifiedTime;
 };
 
@@ -135,21 +132,37 @@ bool WatchEntry::trigger(const QString &fileName)
     return false;
 }
 
-typedef QHash<QString, WatchEntry> WatchEntryMap;
-typedef WatchEntryMap::iterator WatchEntryMapIterator;
+using WatchEntryMap = QHash<QString, WatchEntry>;
+using WatchEntryMapIterator = WatchEntryMap::iterator;
 
 class FileSystemWatcherPrivate
 {
 public:
-    explicit FileSystemWatcherPrivate(int id) : m_id(id), m_staticData(0) {}
+    explicit FileSystemWatcherPrivate(FileSystemWatcher *q, int id) : m_id(id), q(q)
+    {
+        QObject::connect(Utils::GlobalFileChangeBlocker::instance(),
+                         &Utils::GlobalFileChangeBlocker::stateChanged,
+                         q,
+                         [this](bool blocked) { autoReloadPostponed(blocked); });
+    }
 
     WatchEntryMap m_files;
     WatchEntryMap m_directories;
 
+    QSet<QString> m_postponedFiles;
+    QSet<QString> m_postponedDirectories;
+
     bool checkLimit() const;
+    void fileChanged(const QString &path);
+    void directoryChanged(const QString &path);
 
     const int m_id;
-    FileSystemWatcherStaticData *m_staticData;
+    FileSystemWatcherStaticData *m_staticData = nullptr;
+
+private:
+    void autoReloadPostponed(bool postponed);
+    bool m_postponed = false;
+    FileSystemWatcher *q;
 };
 
 bool FileSystemWatcherPrivate::checkLimit() const
@@ -162,12 +175,43 @@ bool FileSystemWatcherPrivate::checkLimit() const
            (m_staticData->maxFileOpen / 2);
 }
 
+void FileSystemWatcherPrivate::fileChanged(const QString &path)
+{
+    if (m_postponed)
+        m_postponedFiles.insert(path);
+    else
+        emit q->fileChanged(path);
+}
+
+void FileSystemWatcherPrivate::directoryChanged(const QString &path)
+{
+    if (m_postponed)
+        m_postponedDirectories.insert(path);
+    else
+        emit q->directoryChanged(path);
+}
+
+void FileSystemWatcherPrivate::autoReloadPostponed(bool postponed)
+{
+    if (m_postponed == postponed)
+        return;
+    m_postponed = postponed;
+    if (!postponed) {
+        for (const QString &file : qAsConst(m_postponedFiles))
+            emit q->fileChanged(file);
+        m_postponedFiles.clear();
+        for (const QString &directory : qAsConst(m_postponedDirectories))
+            emit q->directoryChanged(directory);
+        m_postponedDirectories.clear();
+    }
+}
+
 /*!
     Adds directories to watcher 0.
 */
 
 FileSystemWatcher::FileSystemWatcher(QObject *parent) :
-    QObject(parent), d(new FileSystemWatcherPrivate(0))
+    QObject(parent), d(new FileSystemWatcherPrivate(this, 0))
 {
     init();
 }
@@ -177,7 +221,7 @@ FileSystemWatcher::FileSystemWatcher(QObject *parent) :
 */
 
 FileSystemWatcher::FileSystemWatcher(int id, QObject *parent) :
-    QObject(parent), d(new FileSystemWatcherPrivate(id))
+    QObject(parent), d(new FileSystemWatcherPrivate(this, id))
 {
     init();
 }
@@ -197,10 +241,10 @@ void FileSystemWatcher::init()
             qDebug() << this << "Created watcher for id " << d->m_id;
     }
     ++(d->m_staticData->m_objectCount);
-    connect(d->m_staticData->m_watcher, SIGNAL(fileChanged(QString)),
-            this, SLOT(slotFileChanged(QString)));
-    connect(d->m_staticData->m_watcher, SIGNAL(directoryChanged(QString)),
-            this, SLOT(slotDirectoryChanged(QString)));
+    connect(d->m_staticData->m_watcher, &QFileSystemWatcher::fileChanged,
+            this, &FileSystemWatcher::slotFileChanged);
+    connect(d->m_staticData->m_watcher, &QFileSystemWatcher::directoryChanged,
+            this, &FileSystemWatcher::slotDirectoryChanged);
 }
 
 FileSystemWatcher::~FileSystemWatcher()
@@ -210,9 +254,9 @@ FileSystemWatcher::~FileSystemWatcher()
     if (!d->m_directories.isEmpty())
         removeDirectories(directories());
 
-    if (--(d->m_staticData->m_objectCount) == 0) {
+    if (!fileSystemWatcherStaticDataMap.isDestroyed() && --(d->m_staticData->m_objectCount) == 0) {
         delete d->m_staticData->m_watcher;
-        d->m_staticData->m_watcher = 0;
+        d->m_staticData->m_watcher = nullptr;
         d->m_staticData->m_fileCount.clear();
         d->m_staticData->m_directoryCount.clear();
         if (debug)
@@ -257,6 +301,13 @@ void FileSystemWatcher::addFiles(const QStringList &files, WatchMode wm)
 
         if (count == 1)
             toAdd << file;
+
+        const QString directory = QFileInfo(file).path();
+        const int dirCount = ++d->m_staticData->m_directoryCount[directory];
+        Q_ASSERT(dirCount > 0);
+
+        if (dirCount == 1)
+            toAdd << directory;
     }
 
     if (!toAdd.isEmpty())
@@ -286,6 +337,13 @@ void FileSystemWatcher::removeFiles(const QStringList &files)
 
         if (!count)
             toRemove << file;
+
+        const QString directory = QFileInfo(file).path();
+        const int dirCount = --d->m_staticData->m_directoryCount[directory];
+        Q_ASSERT(dirCount >= 0);
+
+        if (!dirCount)
+            toRemove << directory;
     }
 
     if (!toRemove.isEmpty())
@@ -381,7 +439,7 @@ void FileSystemWatcher::slotFileChanged(const QString &path)
             qDebug() << this << "triggers on file " << path
                      << it.value().watchMode
                      << it.value().modifiedTime.toString(Qt::ISODate);
-        emit fileChanged(path);
+        d->fileChanged(path);
     }
 }
 
@@ -393,7 +451,24 @@ void FileSystemWatcher::slotDirectoryChanged(const QString &path)
             qDebug() << this << "triggers on dir " << path
                      << it.value().watchMode
                      << it.value().modifiedTime.toString(Qt::ISODate);
-        emit directoryChanged(path);
+        d->directoryChanged(path);
+    }
+
+    QStringList toReadd;
+    const QDir dir(path);
+    for (const QFileInfo &entry : dir.entryInfoList(QDir::Files)) {
+        const QString file = entry.filePath();
+        if (d->m_files.contains(file))
+            toReadd.append(file);
+    }
+
+    if (!toReadd.isEmpty()) {
+        for (const QString &rejected : d->m_staticData->m_watcher->addPaths(toReadd))
+            toReadd.removeOne(rejected);
+
+        // If we've successfully added the file, that means it was deleted and replaced.
+        for (const QString &reAdded : toReadd)
+            d->fileChanged(reAdded);
     }
 }
 
