@@ -37,12 +37,13 @@ class QTCREATOR_UTILS_EXPORT NetworkAccessManager : public QNetworkAccessManager
 {
     Q_OBJECT
 public:
-    NetworkAccessManager(QObject *parent = 0);
+    NetworkAccessManager(QObject *parent = nullptr);
 
     static NetworkAccessManager *instance();
 
 protected:
-    virtual QNetworkReply* createRequest(Operation op, const QNetworkRequest &request, QIODevice *outgoingData);
+    QNetworkReply* createRequest(Operation op, const QNetworkRequest &request,
+                                 QIODevice *outgoingData) override;
 };
 
 
