@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -73,10 +68,7 @@ QDebug operator<<(QDebug d, const TextFileFormat &format)
     as strings or string lists and to write out files.
 */
 
-TextFileFormat::TextFileFormat() :
-    lineTerminationMode(NativeLineTerminator), hasUtf8Bom(false), codec(0)
-{
-}
+TextFileFormat::TextFileFormat() = default;
 
 /*!
     Detects the format of text data.
@@ -88,7 +80,7 @@ TextFileFormat TextFileFormat::detect(const QByteArray &data)
     if (data.isEmpty())
         return result;
     const int bytesRead = data.size();
-    const unsigned char *buf = reinterpret_cast<const unsigned char *>(data.constData());
+    const auto buf = reinterpret_cast<const unsigned char *>(data.constData());
     // code taken from qtextstream
     if (bytesRead >= 4 && ((buf[0] == 0xff && buf[1] == 0xfe && buf[2] == 0 && buf[3] == 0)
                            || (buf[0] == 0 && buf[1] == 0 && buf[2] == 0xfe && buf[3] == 0xff))) {
@@ -207,7 +199,7 @@ bool TextFileFormat::decode(const QByteArray &data, QStringList *target) const
 template <class Target>
 TextFileFormat::ReadResult readTextFile(const QString &fileName, const QTextCodec *defaultCodec,
                                         Target *target, TextFileFormat *format, QString *errorString,
-                                        QByteArray *decodingErrorSampleIn = 0)
+                                        QByteArray *decodingErrorSampleIn = nullptr)
 {
     if (decodingErrorSampleIn)
         decodingErrorSampleIn->clear();
@@ -310,14 +302,11 @@ bool TextFileFormat::writeFile(const QString &fileName, QString plainText, QStri
     QTC_ASSERT(codec, return false);
 
     // Does the user want CRLF? If that is native,
-    // let QFile do the work, else manually add.
+    // do net let QFile do the work, because it replaces the line ending after the text was encoded,
+    // and this could lead to undecodable file contents.
     QIODevice::OpenMode fileMode = QIODevice::NotOpen;
-    if (lineTerminationMode == CRLFLineTerminator) {
-        if (NativeLineTerminator == CRLFLineTerminator)
-            fileMode |= QIODevice::Text;
-        else
-            plainText.replace(QLatin1Char('\n'), QLatin1String("\r\n"));
-    }
+    if (lineTerminationMode == CRLFLineTerminator)
+        plainText.replace(QLatin1Char('\n'), QLatin1String("\r\n"));
 
     FileSaver saver(fileName, fileMode);
     if (!saver.hasError()) {
