@@ -26,26 +26,24 @@
 #include "tooltip.h"
 #include "tips.h"
 #include "effects.h"
-#include "reuse.h"
 
 #include <utils/faketooltip.h>
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 
-#include <QString>
+#include <QApplication>
 #include <QColor>
-#include <QGuiApplication>
+#include <QDebug>
+#include <QDesktopWidget>
 #include <QKeyEvent>
+#include <QMenu>
 #include <QMouseEvent>
 #include <QWidget>
-#include <QMenu>
-
-#include <QDebug>
 
 using namespace Utils;
 using namespace Internal;
 
-ToolTip::ToolTip() : m_tip(0), m_widget(0)
+ToolTip::ToolTip() : m_tip(nullptr), m_widget(nullptr)
 {
     connect(&m_showTimer, &QTimer::timeout, this, &ToolTip::hideTipImmediately);
     connect(&m_hideDelayTimer, &QTimer::timeout, this, &ToolTip::hideTipImmediately);
@@ -53,7 +51,7 @@ ToolTip::ToolTip() : m_tip(0), m_widget(0)
 
 ToolTip::~ToolTip()
 {
-    m_tip = 0;
+    m_tip = nullptr;
 }
 
 ToolTip *ToolTip::instance()
@@ -62,36 +60,36 @@ ToolTip *ToolTip::instance()
     return &tooltip;
 }
 
-void ToolTip::show(const QPoint &pos, const QString &content, QWidget *w, const QString &helpId, const QRect &rect)
+void ToolTip::show(const QPoint &pos, const QString &content, QWidget *w, const QVariant &contextHelp, const QRect &rect)
 {
     if (content.isEmpty())
         instance()->hideTipWithDelay();
     else
-        instance()->showInternal(pos, QVariant(content), TextContent, w, helpId, rect);
+        instance()->showInternal(pos, QVariant(content), TextContent, w, contextHelp, rect);
 }
 
-void ToolTip::show(const QPoint &pos, const QColor &color, QWidget *w, const QString &helpId, const QRect &rect)
+void ToolTip::show(const QPoint &pos, const QColor &color, QWidget *w, const QVariant &contextHelp, const QRect &rect)
 {
     if (!color.isValid())
         instance()->hideTipWithDelay();
     else
-        instance()->showInternal(pos, QVariant(color), ColorContent, w, helpId, rect);
+        instance()->showInternal(pos, QVariant(color), ColorContent, w, contextHelp, rect);
 }
 
-void ToolTip::show(const QPoint &pos, QWidget *content, QWidget *w, const QString &helpId, const QRect &rect)
+void ToolTip::show(const QPoint &pos, QWidget *content, QWidget *w, const QVariant &contextHelp, const QRect &rect)
 {
     if (!content)
         instance()->hideTipWithDelay();
     else
-        instance()->showInternal(pos, QVariant::fromValue(content), WidgetContent, w, helpId, rect);
+        instance()->showInternal(pos, QVariant::fromValue(content), WidgetContent, w, contextHelp, rect);
 }
 
-void ToolTip::show(const QPoint &pos, QLayout *content, QWidget *w, const QString &helpId, const QRect &rect)
+void ToolTip::show(const QPoint &pos, QLayout *content, QWidget *w, const QVariant &contextHelp, const QRect &rect)
 {
     if (content && content->count()) {
         auto tooltipWidget = new FakeToolTip;
         tooltipWidget->setLayout(content);
-        instance()->showInternal(pos, QVariant::fromValue(tooltipWidget), WidgetContent, w, helpId, rect);
+        instance()->showInternal(pos, QVariant::fromValue(tooltipWidget), WidgetContent, w, contextHelp, rect);
     } else {
         instance()->hideTipWithDelay();
     }
@@ -109,7 +107,7 @@ bool ToolTip::pinToolTip(QWidget *w, QWidget *parent)
     // Find the parent WidgetTip, tell it to pin/release the
     // widget and close.
     for (QWidget *p = w->parentWidget(); p ; p = p->parentWidget()) {
-        if (WidgetTip *wt = qobject_cast<WidgetTip *>(p)) {
+        if (auto wt = qobject_cast<WidgetTip *>(p)) {
             wt->pinToolTipWidget(parent);
             ToolTip::hide();
             return true;
@@ -118,15 +116,15 @@ bool ToolTip::pinToolTip(QWidget *w, QWidget *parent)
     return false;
 }
 
-QString ToolTip::contextHelpId()
+QVariant ToolTip::contextHelp()
 {
-    return instance()->m_tip ? instance()->m_tip->helpId() : QString();
+    return instance()->m_tip ? instance()->m_tip->contextHelp() : QVariant();
 }
 
 bool ToolTip::acceptShow(const QVariant &content,
                          int typeId,
                          const QPoint &pos,
-                         QWidget *w, const QString &helpId,
+                         QWidget *w, const QVariant &contextHelp,
                          const QRect &rect)
 {
     if (isVisible()) {
@@ -135,9 +133,9 @@ bool ToolTip::acceptShow(const QVariant &content,
             QPoint localPos = pos;
             if (w)
                 localPos = w->mapFromGlobal(pos);
-            if (tipChanged(localPos, content, typeId, w, helpId)) {
+            if (tipChanged(localPos, content, typeId, w, contextHelp)) {
                 m_tip->setContent(content);
-                m_tip->setHelpId(helpId);
+                m_tip->setContextHelp(contextHelp);
                 setUp(pos, w, rect);
             }
             return false;
@@ -169,9 +167,9 @@ void ToolTip::setUp(const QPoint &pos, QWidget *w, const QRect &rect)
 }
 
 bool ToolTip::tipChanged(const QPoint &pos, const QVariant &content, int typeId, QWidget *w,
-                         const QString &helpId) const
+                         const QVariant &contextHelp) const
 {
-    if (!m_tip->equals(typeId, content, helpId) || m_widget != w)
+    if (!m_tip->equals(typeId, content, contextHelp) || m_widget != w)
         return true;
     if (!m_rect.isNull())
         return !m_rect.contains(pos);
@@ -234,7 +232,7 @@ void ToolTip::hideTipImmediately()
     if (m_tip) {
         m_tip->close();
         m_tip->deleteLater();
-        m_tip = 0;
+        m_tip = nullptr;
     }
     m_showTimer.stop();
     m_hideDelayTimer.stop();
@@ -243,10 +241,10 @@ void ToolTip::hideTipImmediately()
 }
 
 void ToolTip::showInternal(const QPoint &pos, const QVariant &content,
-                           int typeId, QWidget *w, const QString &helpId, const QRect &rect)
+                           int typeId, QWidget *w, const QVariant &contextHelp, const QRect &rect)
 {
-    if (acceptShow(content, typeId, pos, w, helpId, rect)) {
-        QWidget *target = 0;
+    if (acceptShow(content, typeId, pos, w, contextHelp, rect)) {
+        QWidget *target = nullptr;
         if (HostOsInfo::isWindowsHost())
             target = QApplication::desktop()->screen(Internal::screenNumber(pos, w));
         else
@@ -263,8 +261,9 @@ void ToolTip::showInternal(const QPoint &pos, const QVariant &content,
                 m_tip = new WidgetTip(target);
                 break;
         }
+        m_tip->setObjectName("qcToolTip");
         m_tip->setContent(content);
-        m_tip->setHelpId(helpId);
+        m_tip->setContextHelp(contextHelp);
         setUp(pos, w, rect);
         qApp->installEventFilter(this);
         showTip();
@@ -355,6 +354,7 @@ bool ToolTip::eventFilter(QObject *o, QEvent *event)
             !m_rect.contains(static_cast<QMouseEvent*>(event)->pos())) {
             hideTipWithDelay();
         }
+        break;
     default:
         break;
     }
