@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -62,7 +57,7 @@ class ProgressItemWidget : public QWidget
 {
     Q_OBJECT
 public:
-    ProgressItemWidget(const QPixmap &indicatorPixmap, const QString &title, QWidget *parent = 0)
+    ProgressItemWidget(const QPixmap &indicatorPixmap, const QString &title, QWidget *parent = nullptr)
         : QWidget(parent),
         m_indicatorVisible(false),
         m_indicatorPixmap(indicatorPixmap)
@@ -70,8 +65,8 @@ public:
         m_indicatorLabel = new QLabel(this);
         m_indicatorLabel->setFixedSize(m_indicatorPixmap.size());
         m_titleLabel = new QLabel(title, this);
-        QHBoxLayout *l = new QHBoxLayout(this);
-        l->setMargin(0);
+        auto l = new QHBoxLayout(this);
+        l->setContentsMargins(0, 0, 0, 0);
         l->addWidget(m_indicatorLabel);
         l->addWidget(m_titleLabel);
     }
@@ -102,9 +97,9 @@ class LinearProgressWidget : public QWidget
 {
     Q_OBJECT
 public:
-    LinearProgressWidget(WizardProgress *progress, QWidget *parent = 0);
+    LinearProgressWidget(WizardProgress *progress, QWidget *parent = nullptr);
 
-private slots:
+private:
     void slotItemAdded(WizardProgressItem *item);
     void slotItemRemoved(WizardProgressItem *item);
     void slotItemChanged(WizardProgressItem *item);
@@ -112,8 +107,6 @@ private slots:
     void slotNextShownItemChanged(WizardProgressItem *item, WizardProgressItem *nextItem);
     void slotStartItemChanged(WizardProgressItem *item);
     void slotCurrentItemChanged(WizardProgressItem *item);
-
-private:
     void recreateLayout();
     void updateProgress();
     void disableUpdates();
@@ -133,14 +126,14 @@ private:
 LinearProgressWidget::LinearProgressWidget(WizardProgress *progress, QWidget *parent)
     :
     QWidget(parent),
-    m_dotsItemWidget(0),
+    m_dotsItemWidget(nullptr),
     m_disableUpdatesCount(0)
 {
     m_indicatorPixmap = QIcon::fromTheme(QLatin1String("go-next"), QIcon(QLatin1String(":/utils/images/arrow.png"))).pixmap(16);
     m_wizardProgress = progress;
     m_mainLayout = new QVBoxLayout(this);
     m_itemWidgetLayout = new QVBoxLayout();
-    QSpacerItem *spacer = new QSpacerItem(0, 0, QSizePolicy::Ignored, QSizePolicy::Expanding);
+    auto spacer = new QSpacerItem(0, 0, QSizePolicy::Ignored, QSizePolicy::Expanding);
     m_mainLayout->addLayout(m_itemWidgetLayout);
     m_mainLayout->addSpacerItem(spacer);
 
@@ -308,25 +301,15 @@ void LinearProgressWidget::enableUpdates()
 
 class WizardPrivate
 {
-    Wizard *q_ptr;
-    Q_DECLARE_PUBLIC(Wizard)
-
 public:
-    WizardPrivate()
-        :
-        m_automaticProgressCreation(true),
-        m_wizardProgress(0)
-    {
-    }
-    bool m_automaticProgressCreation;
-    WizardProgress *m_wizardProgress;
+    bool m_automaticProgressCreation = true;
+    WizardProgress *m_wizardProgress = nullptr;
     QSet<QString> m_fieldNames;
 };
 
 Wizard::Wizard(QWidget *parent, Qt::WindowFlags flags) :
     QWizard(parent, flags), d_ptr(new WizardPrivate)
 {
-    d_ptr->q_ptr = this;
     d_ptr->m_wizardProgress = new WizardProgress(this);
     connect(this, &QWizard::currentIdChanged, this, &Wizard::_q_currentPageChanged);
     connect(this, &QWizard::pageAdded, this, &Wizard::_q_pageAdded);
@@ -384,12 +367,6 @@ WizardProgress *Wizard::wizardProgress() const
     return d->m_wizardProgress;
 }
 
-bool Wizard::validateCurrentPage()
-{
-    emit nextClicked();
-    return QWizard::validateCurrentPage();
-}
-
 bool Wizard::hasField(const QString &name) const
 {
     return d_ptr->m_fieldNames.contains(name);
@@ -453,7 +430,9 @@ void Wizard::showVariables()
 
     auto label = new QLabel(result);
     label->setWordWrap(true);
+    label->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard);
     scrollArea->setWidget(label);
+    scrollArea->setWidgetResizable(true);
 
     layout->addWidget(scrollArea);
     layout->addWidget(buttons);
@@ -498,7 +477,7 @@ void Wizard::_q_pageAdded(int pageId)
     Q_D(Wizard);
 
     QWizardPage *p = page(pageId);
-    WizardPage *wp = qobject_cast<WizardPage *>(p);
+    auto wp = qobject_cast<WizardPage *>(p);
     if (wp)
         wp->pageWasAdded();
 
@@ -522,8 +501,8 @@ void Wizard::_q_pageAdded(int pageId)
     if (index < pages.count() - 1)
         nextId = pages.at(index + 1);
 
-    WizardProgressItem *prevItem = 0;
-    WizardProgressItem *nextItem = 0;
+    WizardProgressItem *prevItem = nullptr;
+    WizardProgressItem *nextItem = nullptr;
 
     if (prevId >= 0)
         prevItem = d->m_wizardProgress->item(prevId);
@@ -531,9 +510,9 @@ void Wizard::_q_pageAdded(int pageId)
         nextItem = d->m_wizardProgress->item(nextId);
 
     if (prevItem)
-        prevItem->setNextItems(QList<WizardProgressItem *>() << item);
+        prevItem->setNextItems({item});
     if (nextItem)
-        item->setNextItems(QList<WizardProgressItem *>() << nextItem);
+        item->setNextItems({nextItem});
 }
 
 void Wizard::_q_pageRemoved(int pageId)
@@ -559,8 +538,8 @@ void Wizard::_q_pageRemoved(int pageId)
     if (index < pages.count() - 1)
         nextId = pages.at(index + 1);
 
-    WizardProgressItem *prevItem = 0;
-    WizardProgressItem *nextItem = 0;
+    WizardProgressItem *prevItem = nullptr;
+    WizardProgressItem *nextItem = nullptr;
 
     if (prevId >= 0)
         prevItem = d->m_wizardProgress->item(prevId);
@@ -586,12 +565,7 @@ class WizardProgressPrivate
     Q_DECLARE_PUBLIC(WizardProgress)
 
 public:
-    WizardProgressPrivate()
-        :
-        m_currentItem(0),
-        m_startItem(0)
-    {
-    }
+    WizardProgressPrivate() = default;
 
     bool isNextItem(WizardProgressItem *item, WizardProgressItem *nextItem) const;
     // if multiple paths are possible the empty list is returned
@@ -606,8 +580,8 @@ public:
     QList<WizardProgressItem *> m_visitedItems;
     QList<WizardProgressItem *> m_reachableItems;
 
-    WizardProgressItem *m_currentItem;
-    WizardProgressItem *m_startItem;
+    WizardProgressItem *m_currentItem = nullptr;
+    WizardProgressItem *m_startItem = nullptr;
 };
 
 class WizardProgressItemPrivate
@@ -652,7 +626,7 @@ QList<WizardProgressItem *> WizardProgressPrivate::singlePathBetween(WizardProgr
     // When we had X->A in addition and "from" was X and "to" was C, this would not work
     // (it should return the shortest path which would be X->A->C).
     if (item->nextItems().contains(toItem))
-        return QList<WizardProgressItem *>() << toItem;
+        return {toItem};
 
     QHash<WizardProgressItem *, QHash<WizardProgressItem *, bool> > visitedItemsToParents;
     QList<QPair<WizardProgressItem *, WizardProgressItem *> > workingItems; // next to prev item
@@ -688,13 +662,13 @@ QList<WizardProgressItem *> WizardProgressPrivate::singlePathBetween(WizardProgr
             return path;
         itItem = visitedItemsToParents.constFind(it);
     }
-    return QList<WizardProgressItem *>();
+    return {};
 }
 
 void WizardProgressPrivate::updateReachableItems()
 {
     m_reachableItems = m_visitedItems;
-    WizardProgressItem *item = 0;
+    WizardProgressItem *item = nullptr;
     if (m_visitedItems.count() > 0)
         item = m_visitedItems.last();
     if (!item) {
@@ -733,7 +707,7 @@ WizardProgressItem *WizardProgress::addItem(const QString &title)
 {
     Q_D(WizardProgress);
 
-    WizardProgressItem *item = new WizardProgressItem(this, title);
+    auto item = new WizardProgressItem(this, title);
     d->m_itemToItem.insert(item, item);
     emit itemAdded(item);
     return item;
@@ -856,7 +830,7 @@ void WizardProgress::setCurrentPage(int pageId)
     Q_D(WizardProgress);
 
     if (pageId < 0) { // reset history
-        d->m_currentItem = 0;
+        d->m_currentItem = nullptr;
         d->m_visitedItems.clear();
         d->m_reachableItems.clear();
         d->updateReachableItems();
@@ -924,7 +898,7 @@ WizardProgressItem::WizardProgressItem(WizardProgress *progress, const QString &
     d_ptr->m_title = title;
     d_ptr->m_titleWordWrap = false;
     d_ptr->m_wizardProgress = progress;
-    d_ptr->m_nextShownItem = 0;
+    d_ptr->m_nextShownItem = nullptr;
 }
 
 WizardProgressItem::~WizardProgressItem()
@@ -968,7 +942,7 @@ void WizardProgressItem::setNextItems(const QList<WizardProgressItem *> &items)
         return;
 
     if (!items.contains(d->m_nextShownItem))
-        setNextShownItem(0);
+        setNextShownItem(nullptr);
 
     // update prev items (remove this item from the old next items)
     for (int i = 0; i < d->m_nextItems.count(); i++) {
