@@ -31,7 +31,6 @@
 #include "androidconstants.h"
 #include "androidmanager.h"
 #include "androidsdkmanager.h"
-#include "androidqtsupport.h"
 #include "certificatesmodel.h"
 
 #include "javaparser.h"
@@ -40,20 +39,26 @@
 #include <coreplugin/icore.h>
 
 #include <projectexplorer/buildconfiguration.h>
+#include <projectexplorer/buildsteplist.h>
+#include <projectexplorer/processparameters.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/runconfiguration.h>
 #include <projectexplorer/target.h>
 
 #include <qtsupport/qtkitinformation.h>
 
 #include <utils/algorithm.h>
+#include <utils/qtcprocess.h>
 #include <utils/synchronousprocess.h>
 #include <utils/utilsicons.h>
 
-#include <qmakeprojectmanager/qmakeprojectmanagerconstants.h>
-
+#include <QDateTime>
 #include <QDialogButtonBox>
 #include <QHBoxLayout>
+#include <QJsonDocument>
+#include <QJsonObject>
 #include <QLabel>
 #include <QLineEdit>
 #include <QLoggingCategory>
@@ -65,6 +70,7 @@
 #include <memory>
 
 using namespace ProjectExplorer;
+using namespace Utils;
 using namespace Android::Internal;
 
 namespace {
@@ -73,8 +79,6 @@ Q_LOGGING_CATEGORY(buildapkstepLog, "qtc.android.build.androidbuildapkstep", QtW
 
 namespace Android {
 
-const Core::Id ANDROID_BUILD_APK_ID("QmakeProjectManager.AndroidBuildApkStep");
-
 const QVersionNumber gradleScriptRevokedSdkVersion(25, 3, 0);
 const char KeystoreLocationKey[] = "KeystoreLocation";
 const char BuildTargetSdkKey[] = "BuildTargetSdk";
@@ -87,12 +91,10 @@ static void setupProcessParameters(ProcessParameters *pp,
                                    const QString &command)
 {
     pp->setMacroExpander(bc->macroExpander());
-    pp->setWorkingDirectory(bc->buildDirectory().toString());
+    pp->setWorkingDirectory(bc->buildDirectory());
     Utils::Environment env = bc->environment();
     pp->setEnvironment(env);
-    pp->setCommand(command);
-    pp->setArguments(Utils::QtcProcess::joinArgs(arguments));
-    pp->resolveAll();
+    pp->setCommandLine({command, arguments});
 }
 
 class PasswordInputDialog : public QDialog
@@ -123,15 +125,25 @@ private:
 };
 
 AndroidBuildApkStep::AndroidBuildApkStep(BuildStepList *parent)
-    : AbstractProcessStep(parent, ANDROID_BUILD_APK_ID),
+    : AbstractProcessStep(parent, Constants::ANDROID_BUILD_APK_ID),
       m_buildTargetSdk(AndroidConfig::apiLevelNameFor(AndroidConfigurations::
                                          sdkManager()->latestAndroidSdkPlatform()))
 {
     //: AndroidBuildApkStep default display name
     setDefaultDisplayName(tr("Build Android APK"));
+    setImmutable(true);
 }
 
-bool AndroidBuildApkStep::init(QList<const BuildStep *> &earlierSteps)
+AndroidBuildApkStep *AndroidBuildApkStep::findInBuild(const BuildConfiguration *bc)
+{
+    if (!bc)
+        return nullptr;
+    BuildStepList *bsl = bc->stepList(ProjectExplorer::Constants::BUILDSTEPS_BUILD);
+    QTC_ASSERT(bsl, return nullptr);
+    return bsl->firstOfType<AndroidBuildApkStep>();
+}
+
+bool AndroidBuildApkStep::init()
 {
     ProjectExplorer::BuildConfiguration *bc = buildConfiguration();
 
@@ -148,13 +160,13 @@ bool AndroidBuildApkStep::init(QList<const BuildStep *> &earlierSteps)
                            OutputFormat::ErrorMessage);
     }
 
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
+    QtSupport::BaseQtVersion *version = QtSupport::QtKitAspect::qtVersion(target()->kit());
     if (!version)
         return false;
 
     const QVersionNumber sdkToolsVersion = AndroidConfigurations::currentConfig().sdkToolsVersion();
     if (sdkToolsVersion >= gradleScriptRevokedSdkVersion) {
-        if (!version->sourcePath().appendPath("src/3rdparty/gradle").exists()) {
+        if (!version->sourcePath().pathAppended("src/3rdparty/gradle").exists()) {
             emit addOutput(tr("The installed SDK tools version (%1) does not include Gradle "
                               "scripts. The minimum Qt version required for Gradle build to work "
                               "is %2").arg(sdkToolsVersion.toString()).arg("5.9.0/5.6.3"),
@@ -177,33 +189,42 @@ bool AndroidBuildApkStep::init(QList<const BuildStep *> &earlierSteps)
 
     auto parser = new JavaParser;
     parser->setProjectFileList(Utils::transform(target()->project()->files(ProjectExplorer::Project::AllFiles),
-                                                &Utils::FileName::toString));
+                                                &Utils::FilePath::toString));
 
-    AndroidQtSupport *qtSupport = AndroidManager::androidQtSupport(target());
-    QTC_ASSERT(qtSupport, return false);
-    QFileInfo sourceDirInfo(qtSupport->targetData(Constants::AndroidPackageSourceDir, target()).toString());
-    parser->setSourceDirectory(Utils::FileName::fromString(sourceDirInfo.canonicalFilePath()));
-    parser->setBuildDirectory(Utils::FileName::fromString(bc->buildDirectory().appendPath(Constants::ANDROID_BUILDDIRECTORY).toString()));
+    RunConfiguration *rc = target()->activeRunConfiguration();
+    const QString buildKey = rc ? rc->buildKey() : QString();
+    const ProjectNode *node = rc ? target()->project()->findNodeForBuildKey(buildKey) : nullptr;
+
+    QString sourceDirName;
+    if (node)
+        sourceDirName = node->data(Constants::AndroidPackageSourceDir).toString();
+
+    QFileInfo sourceDirInfo(sourceDirName);
+    parser->setSourceDirectory(Utils::FilePath::fromString(sourceDirInfo.canonicalFilePath()));
+    parser->setBuildDirectory(bc->buildDirectory().pathAppended(Constants::ANDROID_BUILDDIRECTORY));
     setOutputParser(parser);
 
     m_openPackageLocationForRun = m_openPackageLocation;
-    m_apkPath = AndroidManager::apkPath(target()).toString();
-    qCDebug(buildapkstepLog) << "APK path:" << m_apkPath;
+    m_packagePath = m_buildAAB ? AndroidManager::aabPath(target()).toString()
+                           : AndroidManager::apkPath(target()).toString();
+    qCDebug(buildapkstepLog) << "Package path:" << m_packagePath;
 
-    if (!AbstractProcessStep::init(earlierSteps))
+    if (!AbstractProcessStep::init())
         return false;
 
-    QString command = version->qmakeProperty("QT_HOST_BINS");
+    QString command = version->hostBinPath().toString();
     if (!command.endsWith('/'))
         command += '/';
     command += "androiddeployqt";
     if (Utils::HostOsInfo::isWindowsHost())
         command += ".exe";
 
-    QString outputDir = bc->buildDirectory().appendPath(Constants::ANDROID_BUILDDIRECTORY).toString();
+    QString outputDir = bc->buildDirectory().pathAppended(Constants::ANDROID_BUILDDIRECTORY).toString();
+
+    QString inputFile;
+    if (node)
+        inputFile = node->data(Constants::AndroidDeploySettingsFile).toString();
 
-    QString inputFile = AndroidManager::androidQtSupport(target())
-            ->targetData(Constants::AndroidDeploySettingsFile, target()).toString();
     if (inputFile.isEmpty()) {
         m_skipBuilding = true;
         return true;
@@ -226,6 +247,9 @@ bool AndroidBuildApkStep::init(QList<const BuildStep *> &earlierSteps)
 
     arguments << "--gradle";
 
+    if (m_buildAAB)
+        arguments << "--aab" <<  "--jarsigner";
+
     if (m_useMinistro)
         arguments << "--deployment" << "ministro";
 
@@ -258,7 +282,7 @@ bool AndroidBuildApkStep::init(QList<const BuildStep *> &earlierSteps)
     // Generate arguments with keystore password concealed
     ProjectExplorer::ProcessParameters pp2;
     setupProcessParameters(&pp2, bc, argumentsPasswordConcealed, command);
-    m_command = pp2.effectiveCommand();
+    m_command = pp2.effectiveCommand().toString();
     m_argumentsPasswordConcealed = pp2.prettyArguments();
 
     return true;
@@ -266,7 +290,7 @@ bool AndroidBuildApkStep::init(QList<const BuildStep *> &earlierSteps)
 
 void AndroidBuildApkStep::showInGraphicalShell()
 {
-    Core::FileUtils::showInGraphicalShell(Core::ICore::mainWindow(), m_apkPath);
+    Core::FileUtils::showInGraphicalShell(Core::ICore::mainWindow(), m_packagePath);
 }
 
 ProjectExplorer::BuildStepConfigWidget *AndroidBuildApkStep::createConfigWidget()
@@ -284,8 +308,8 @@ void AndroidBuildApkStep::processFinished(int exitCode, QProcess::ExitStatus sta
 bool AndroidBuildApkStep::verifyKeystorePassword()
 {
     if (!m_keystorePath.exists()) {
-        addOutput(tr("Cannot sign the package. Invalid keystore path (%1).")
-                  .arg(m_keystorePath.toString()), OutputFormat::ErrorMessage);
+        emit addOutput(tr("Cannot sign the package. Invalid keystore path (%1).")
+                           .arg(m_keystorePath.toString()), OutputFormat::ErrorMessage);
         return false;
     }
 
@@ -304,8 +328,8 @@ bool AndroidBuildApkStep::verifyCertificatePassword()
 {
     if (!AndroidManager::checkCertificateExists(m_keystorePath.toString(), m_keystorePasswd,
                                                  m_certificateAlias)) {
-        addOutput(tr("Cannot sign the package. Certificate alias %1 does not exist.")
-                  .arg(m_certificateAlias), OutputFormat::ErrorMessage);
+        emit addOutput(tr("Cannot sign the package. Certificate alias %1 does not exist.")
+                           .arg(m_certificateAlias), OutputFormat::ErrorMessage);
         return false;
     }
 
@@ -325,14 +349,124 @@ bool AndroidBuildApkStep::verifyCertificatePassword()
     return success;
 }
 
-void AndroidBuildApkStep::run(QFutureInterface<bool> &fi)
+
+static bool copyFileIfNewer(const QString &sourceFileName,
+                            const QString &destinationFileName)
+{
+    if (QFile::exists(destinationFileName)) {
+        QFileInfo destinationFileInfo(destinationFileName);
+        QFileInfo sourceFileInfo(sourceFileName);
+        if (sourceFileInfo.lastModified() <= destinationFileInfo.lastModified())
+            return true;
+        if (!QFile(destinationFileName).remove())
+            return false;
+    }
+
+    if (!QDir().mkpath(QFileInfo(destinationFileName).path()))
+        return false;
+    return QFile::copy(sourceFileName, destinationFileName);
+}
+
+void AndroidBuildApkStep::doRun()
 {
     if (m_skipBuilding) {
-        emit addOutput(tr("No application .pro file found, not building an APK."), BuildStep::OutputFormat::ErrorMessage);
-        reportRunResult(fi, true);
+        emit addOutput(tr("Android deploy settings file not found, not building an APK."), BuildStep::OutputFormat::ErrorMessage);
+        emit finished(true);
         return;
     }
-    AbstractProcessStep::run(fi);
+
+    auto setup = [this] {
+        auto bc = target()->activeBuildConfiguration();
+        const auto androidAbis = AndroidManager::applicationAbis(target());
+        for (const auto &abi : androidAbis) {
+            Utils::FilePath androidLibsDir = bc->buildDirectory()
+                    .pathAppended("android-build/libs")
+                    .pathAppended(abi);
+            if (!androidLibsDir.exists() && !QDir{bc->buildDirectory().toString()}.mkpath(androidLibsDir.toString()))
+                return false;
+        }
+
+        RunConfiguration *rc = target()->activeRunConfiguration();
+        const QString buildKey = rc ? rc->buildKey() : QString();
+        const ProjectNode *node = rc ? target()->project()->findNodeForBuildKey(buildKey) : nullptr;
+
+        if (!node)
+            return false;
+
+        FilePath deploymentSettingsFile = FilePath::fromString(node->data(Android::Constants::AndroidDeploySettingsFile).toString());
+        if (deploymentSettingsFile.exists())
+            return true; // cmake creates this file for us
+
+        auto targets = node->data(Android::Constants::AndroidTargets).toStringList();
+        if (targets.isEmpty())
+            return true; // qmake does this job for us
+
+        QJsonObject deploySettings = Android::AndroidManager::deploymentSettings(target());
+        QJsonObject architectures;
+
+        // Copy targets to android build folder
+        QString applicationBinary = target()->activeRunConfiguration()->buildTargetInfo().targetFilePath.toFileInfo().fileName();
+        for (const auto &abi : androidAbis) {
+            QString targetSuffix = QString{"_%1.so"}.arg(abi);
+            if (applicationBinary.endsWith(targetSuffix)) {
+                // Keep only TargetName from "lib[TargetName]_abi.so"
+                applicationBinary.remove(0, 3).chop(targetSuffix.size());
+            }
+
+            Utils::FilePath androidLibsDir = bc->buildDirectory()
+                                                 .pathAppended("android-build/libs")
+                                                 .pathAppended(abi);
+            for (const auto &target : targets) {
+                if (target.endsWith(targetSuffix)) {
+                    if (!copyFileIfNewer(target, androidLibsDir.pathAppended(QFileInfo{target}.fileName()).toString()))
+                        return false;
+                    if (abi == "x86") {
+                        architectures[abi] = "i686-linux-android";
+                    } else if (abi == "x86_64") {
+                        architectures[abi] = "x86_64-linux-android";
+                    } else if (abi == "arm64-v8a") {
+                        architectures[abi] = "aarch64-linux-android";
+                    } else {
+                        architectures[abi] = "arm-linux-androideabi";
+                    }
+                }
+            }
+        }
+
+        deploySettings["application-binary"] = applicationBinary;
+        deploySettings["architectures"] = architectures;
+
+        QString extraLibs = node->data(Android::Constants::AndroidExtraLibs).toString();
+        if (!extraLibs.isEmpty())
+            deploySettings["android-extra-libs"] = extraLibs;
+
+        QString androidSrcs = node->data(Android::Constants::AndroidPackageSourceDir).toString();
+        if (!androidSrcs.isEmpty())
+            deploySettings["android-package-source-directory"] = androidSrcs;
+
+        QString qmlImportPath = node->data("QML_IMPORT_PATH").toString();
+        if (!qmlImportPath.isEmpty())
+            deploySettings["qml-import-paths"] = qmlImportPath;
+
+        QString qmlRootPath = node->data("QML_ROOT_PATH").toString();
+        if (qmlRootPath.isEmpty())
+            qmlRootPath = target()->project()->rootProjectDirectory().toString();
+         deploySettings["qml-root-path"] = qmlRootPath;
+
+        QFile f{deploymentSettingsFile.toString()};
+        if (!f.open(QIODevice::WriteOnly))
+            return false;
+        f.write(QJsonDocument{deploySettings}.toJson());
+        return true;
+    };
+
+    if (!setup()) {
+        emit addOutput(tr("Cannot set up Android, not building an APK."), BuildStep::OutputFormat::ErrorMessage);
+        emit finished(false);
+        return;
+    }
+
+    AbstractProcessStep::doRun();
 }
 
 void AndroidBuildApkStep::processStarted()
@@ -345,7 +479,7 @@ void AndroidBuildApkStep::processStarted()
 
 bool AndroidBuildApkStep::fromMap(const QVariantMap &map)
 {
-    m_keystorePath = Utils::FileName::fromString(map.value(KeystoreLocationKey).toString());
+    m_keystorePath = Utils::FilePath::fromString(map.value(KeystoreLocationKey).toString());
     m_signPackage = false; // don't restore this
     m_buildTargetSdk = map.value(BuildTargetSdkKey).toString();
     if (m_buildTargetSdk.isEmpty()) {
@@ -367,7 +501,7 @@ QVariantMap AndroidBuildApkStep::toMap() const
     return map;
 }
 
-Utils::FileName AndroidBuildApkStep::keystorePath()
+Utils::FilePath AndroidBuildApkStep::keystorePath()
 {
     return m_keystorePath;
 }
@@ -380,10 +514,23 @@ QString AndroidBuildApkStep::buildTargetSdk() const
 void AndroidBuildApkStep::setBuildTargetSdk(const QString &sdk)
 {
     m_buildTargetSdk = sdk;
-    AndroidManager::updateGradleProperties(target());
 }
 
-void AndroidBuildApkStep::setKeystorePath(const Utils::FileName &path)
+QVariant AndroidBuildApkStep::data(Core::Id id) const
+{
+    if (id == Constants::AndroidNdkPlatform)
+        return AndroidConfigurations::currentConfig().bestNdkPlatformMatch(AndroidManager::minimumSDK(target())).mid(8);
+    if (id == Constants::NdkLocation)
+        return QVariant::fromValue(AndroidConfigurations::currentConfig().ndkLocation());
+    if (id == Constants::SdkLocation)
+        return QVariant::fromValue(AndroidConfigurations::currentConfig().sdkLocation());
+    if (id == Constants::AndroidABIs)
+        return AndroidManager::applicationAbis(target());
+
+    return AbstractProcessStep::data(id);
+}
+
+void AndroidBuildApkStep::setKeystorePath(const Utils::FilePath &path)
 {
     m_keystorePath = path;
     m_certificatePasswd.clear();
@@ -415,6 +562,16 @@ void AndroidBuildApkStep::setSignPackage(bool b)
     m_signPackage = b;
 }
 
+bool AndroidBuildApkStep::buildAAB() const
+{
+    return m_buildAAB;
+}
+
+void AndroidBuildApkStep::setBuildAAB(bool aab)
+{
+    m_buildAAB = aab;
+}
+
 bool AndroidBuildApkStep::openPackageLocation() const
 {
     return m_openPackageLocation;
@@ -467,8 +624,8 @@ QAbstractItemModel *AndroidBuildApkStep::keystoreCertificates()
 
     Utils::SynchronousProcess keytoolProc;
     keytoolProc.setTimeoutS(30);
-    const Utils::SynchronousProcessResponse response
-            = keytoolProc.run(AndroidConfigurations::currentConfig().keytoolPath().toString(), params);
+    const SynchronousProcessResponse response
+            = keytoolProc.run({AndroidConfigurations::currentConfig().keytoolPath(), params});
     if (response.result > Utils::SynchronousProcessResponse::FinishedError)
         QMessageBox::critical(nullptr, tr("Error"), tr("Failed to run keytool."));
     else
@@ -552,8 +709,7 @@ namespace Internal {
 
 AndroidBuildApkStepFactory::AndroidBuildApkStepFactory()
 {
-    registerStep<AndroidBuildApkStep>(ANDROID_BUILD_APK_ID);
-    setSupportedProjectType(QmakeProjectManager::Constants::QMAKEPROJECT_ID);
+    registerStep<AndroidBuildApkStep>(Constants::ANDROID_BUILD_APK_ID);
     setSupportedDeviceType(Constants::ANDROID_DEVICE_TYPE);
     setSupportedStepList(ProjectExplorer::Constants::BUILDSTEPS_BUILD);
     setDisplayName(AndroidBuildApkStep::tr("Build Android APK"));
