@@ -26,62 +26,54 @@
 #include "androiddevice.h"
 #include "androidconstants.h"
 #include "androidsignaloperation.h"
+#include "androidconfigurations.h"
+#include "androidmanager.h"
 
 #include <projectexplorer/runconfiguration.h>
-#include <projectexplorer/runnables.h>
+
+#include <utils/url.h>
 
 #include <QCoreApplication>
+#include <QLoggingCategory>
 
 using namespace ProjectExplorer;
 
+namespace {
+Q_LOGGING_CATEGORY(androidDeviceLog, "qtc.android.build.androiddevice", QtWarningMsg)
+}
+
 namespace Android {
 namespace Internal {
 
 AndroidDevice::AndroidDevice()
-    : IDevice(Core::Id(Constants::ANDROID_DEVICE_TYPE),
-                             IDevice::AutoDetected,
-                             IDevice::Hardware,
-                             Core::Id(Constants::ANDROID_DEVICE_ID))
 {
-    setDisplayName(QCoreApplication::translate("Android::Internal::AndroidDevice", "Run on Android"));
+    setupId(IDevice::AutoDetected, Constants::ANDROID_DEVICE_ID);
+    setType(Constants::ANDROID_DEVICE_TYPE);
+    setDefaultDisplayName(QCoreApplication::translate("Android::Internal::AndroidDevice",
+                                                      "Run on Android"));
+    setDisplayType(QCoreApplication::translate("Android::Internal::AndroidDevice", "Android"));
+    setMachineType(IDevice::Hardware);
+    setOsType(Utils::OsTypeOtherUnix);
+
     setDeviceState(DeviceReadyToUse);
+    const AndroidConfig &config = AndroidConfigurations::currentConfig();
+    if (config.qtLiveApkPath().exists()) {
+        QString activityPath;
+        AndroidManager::apkInfo(config.qtLiveApkPath(), nullptr, nullptr, &activityPath);
+        qCDebug(androidDeviceLog) << "Using Qt live apk from: " << config.qtLiveApkPath()
+                                  << "Activity Path:" << activityPath;
+        setQmlsceneCommand(activityPath);
+    }
 }
 
-AndroidDevice::AndroidDevice(const AndroidDevice &other)
-    : IDevice(other)
-{ }
-
-
 IDevice::DeviceInfo AndroidDevice::deviceInformation() const
 {
     return IDevice::DeviceInfo();
 }
 
-QString AndroidDevice::displayType() const
-{
-    return QCoreApplication::translate("Android::Internal::AndroidDevice", "Android");
-}
-
 IDeviceWidget *AndroidDevice::createWidget()
 {
-    return 0;
-}
-
-QList<Core::Id> AndroidDevice::actionIds() const
-{
-    return QList<Core::Id>();
-}
-
-QString AndroidDevice::displayNameForActionId(Core::Id actionId) const
-{
-    Q_UNUSED(actionId)
-    return QString();
-}
-
-void AndroidDevice::executeAction(Core::Id actionId, QWidget *parent)
-{
-    Q_UNUSED(actionId)
-    Q_UNUSED(parent)
+    return nullptr;
 }
 
 bool AndroidDevice::canAutoDetectPorts() const
@@ -94,23 +86,26 @@ DeviceProcessSignalOperation::Ptr AndroidDevice::signalOperation() const
     return DeviceProcessSignalOperation::Ptr(new AndroidSignalOperation());
 }
 
-Utils::OsType AndroidDevice::osType() const
-{
-    return Utils::OsTypeOtherUnix;
-}
-
-IDevice::Ptr AndroidDevice::clone() const
-{
-    return IDevice::Ptr(new AndroidDevice(*this));
-}
-
 QUrl AndroidDevice::toolControlChannel(const ControlChannelHint &) const
 {
     QUrl url;
-    url.setScheme(urlTcpScheme());
+    url.setScheme(Utils::urlTcpScheme());
     url.setHost("localhost");
     return url;
 }
 
+
+// Factory
+
+AndroidDeviceFactory::AndroidDeviceFactory()
+    : ProjectExplorer::IDeviceFactory(Constants::ANDROID_DEVICE_TYPE)
+{
+    setObjectName(QLatin1String("AndroidDeviceFactory"));
+    setDisplayName(tr("Android Device"));
+    setCombinedIcon(":/android/images/androiddevicesmall.png",
+                    ":/android/images/androiddevice.png");
+    setConstructionFunction(&AndroidDevice::create);
+}
+
 } // namespace Internal
 } // namespace Android
