@@ -25,16 +25,18 @@
 
 #include "androidmanifesteditorwidget.h"
 #include "androidmanifesteditor.h"
+#include "androidconfigurations.h"
 #include "androidconstants.h"
 #include "androidmanifestdocument.h"
 #include "androidmanager.h"
-#include "androidqtsupport.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/infobar.h>
 #include <coreplugin/editormanager/ieditor.h>
 
+#include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/projectwindow.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
@@ -43,6 +45,7 @@
 
 #include <texteditor/texteditoractionhandler.h>
 #include <texteditor/texteditor.h>
+
 #include <utils/algorithm.h>
 #include <utils/utilsicons.h>
 
@@ -83,17 +86,17 @@ bool checkPackageName(const QString &packageName)
     return QRegExp(packageNameRegExp).exactMatch(packageName);
 }
 
-Project *androidProject(const Utils::FileName &fileName)
+Project *androidProject(const Utils::FilePath &fileName)
 {
     for (Project *project : SessionManager::projects()) {
         if (!project->activeTarget())
             continue;
         Kit *kit = project->activeTarget()->kit();
-        if (DeviceTypeKitInformation::deviceTypeId(kit) == Android::Constants::ANDROID_DEVICE_TYPE
+        if (DeviceTypeKitAspect::deviceTypeId(kit) == Android::Constants::ANDROID_DEVICE_TYPE
                 && fileName.isChildOf(project->projectDirectory()))
             return project;
     }
-    return 0;
+    return nullptr;
 }
 
 } // anonymous namespace
@@ -127,15 +130,15 @@ void AndroidManifestEditorWidget::initializePage()
 {
     QWidget *mainWidget = new QWidget; // different name
 
-    QVBoxLayout *topLayout = new QVBoxLayout(mainWidget);
+    auto topLayout = new QVBoxLayout(mainWidget);
 
-    QGroupBox *packageGroupBox = new QGroupBox(mainWidget);
+    auto packageGroupBox = new QGroupBox(mainWidget);
     topLayout->addWidget(packageGroupBox);
 
     auto setDirtyFunc = [this] { setDirty(); };
     packageGroupBox->setTitle(tr("Package"));
     {
-        QFormLayout *formLayout = new QFormLayout();
+        auto formLayout = new QFormLayout();
 
         m_packageNameLineEdit = new QLineEdit(packageGroupBox);
         m_packageNameLineEdit->setToolTip(tr(
@@ -161,19 +164,15 @@ void AndroidManifestEditorWidget::initializePage()
         m_packageNameWarningIcon->setVisible(false);
         m_packageNameWarningIcon->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
-        QHBoxLayout *warningRow = new QHBoxLayout;
-        warningRow->setMargin(0);
+        auto warningRow = new QHBoxLayout;
+        warningRow->setContentsMargins(0, 0, 0, 0);
         warningRow->addWidget(m_packageNameWarningIcon);
         warningRow->addWidget(m_packageNameWarning);
 
         formLayout->addRow(QString(), warningRow);
 
-
-        m_versionCode = new QSpinBox(packageGroupBox);
-        m_versionCode->setMaximum(std::numeric_limits<int>::max());
-        m_versionCode->setValue(1);
-        m_versionCode->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-        formLayout->addRow(tr("Version code:"), m_versionCode);
+        m_versionCodeLineEdit = new QLineEdit(packageGroupBox);
+        formLayout->addRow(tr("Version code:"), m_versionCodeLineEdit);
 
         m_versionNameLinedit = new QLineEdit(packageGroupBox);
         formLayout->addRow(tr("Version name:"), m_versionNameLinedit);
@@ -199,26 +198,26 @@ void AndroidManifestEditorWidget::initializePage()
 
         connect(m_packageNameLineEdit, &QLineEdit::textEdited,
                 this, &AndroidManifestEditorWidget::setPackageName);
-        connect(m_versionCode, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
-                this, &AndroidManifestEditorWidget::setDirty);
+        connect(m_versionCodeLineEdit, &QLineEdit::textEdited,
+                 this, setDirtyFunc);
         connect(m_versionNameLinedit, &QLineEdit::textEdited,
                 this, setDirtyFunc);
         connect(m_androidMinSdkVersion,
-                static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+                QOverload<int>::of(&QComboBox::currentIndexChanged),
                 this, setDirtyFunc);
         connect(m_androidTargetSdkVersion,
-                static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+                QOverload<int>::of(&QComboBox::currentIndexChanged),
                 this, setDirtyFunc);
 
     }
 
     // Application
-    QGroupBox *applicationGroupBox = new QGroupBox(mainWidget);
+    auto applicationGroupBox = new QGroupBox(mainWidget);
     topLayout->addWidget(applicationGroupBox);
 
     applicationGroupBox->setTitle(tr("Application"));
     {
-        QFormLayout *formLayout = new QFormLayout();
+        auto formLayout = new QFormLayout();
 
         m_appNameLineEdit = new QLineEdit(applicationGroupBox);
         formLayout->addRow(tr("Application name:"), m_appNameLineEdit);
@@ -232,7 +231,7 @@ void AndroidManifestEditorWidget::initializePage()
         m_targetLineEdit->installEventFilter(this);
         formLayout->addRow(tr("Run:"), m_targetLineEdit);
 
-        QHBoxLayout *iconLayout = new QHBoxLayout();
+        auto iconLayout = new QHBoxLayout();
         m_lIconButton = new QToolButton(applicationGroupBox);
         m_lIconButton->setMinimumSize(QSize(48, 48));
         m_lIconButton->setMaximumSize(QSize(48, 48));
@@ -276,12 +275,12 @@ void AndroidManifestEditorWidget::initializePage()
 
 
     // Permissions
-    QGroupBox *permissionsGroupBox = new QGroupBox(mainWidget);
+    auto permissionsGroupBox = new QGroupBox(mainWidget);
     topLayout->addWidget(permissionsGroupBox);
 
     permissionsGroupBox->setTitle(tr("Permissions"));
     {
-        QGridLayout *layout = new QGridLayout(permissionsGroupBox);
+        auto layout = new QGridLayout(permissionsGroupBox);
 
         m_defaultPermissonsCheckBox = new QCheckBox(this);
         m_defaultPermissonsCheckBox->setText(tr("Include default permissions for Qt modules."));
@@ -459,9 +458,10 @@ void AndroidManifestEditorWidget::initializePage()
 
     topLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Fixed, QSizePolicy::MinimumExpanding));
 
-    QScrollArea *mainWidgetScrollArea = new QScrollArea;
+    auto mainWidgetScrollArea = new QScrollArea;
     mainWidgetScrollArea->setWidgetResizable(true);
     mainWidgetScrollArea->setWidget(mainWidget);
+    mainWidgetScrollArea->setFocusProxy(m_packageNameLineEdit);
 
     insertWidget(General, mainWidgetScrollArea);
     insertWidget(Source, m_textEditorWidget);
@@ -477,14 +477,29 @@ bool AndroidManifestEditorWidget::eventFilter(QObject *obj, QEvent *event)
     return QWidget::eventFilter(obj, event);
 }
 
+void AndroidManifestEditorWidget::focusInEvent(QFocusEvent *event)
+{
+    if (currentWidget()) {
+        if (currentWidget()->focusWidget())
+            currentWidget()->focusWidget()->setFocus(event->reason());
+        else
+            currentWidget()->setFocus(event->reason());
+    }
+}
+
 void AndroidManifestEditorWidget::updateTargetComboBox()
 {
     Project *project = androidProject(m_textEditorWidget->textDocument()->filePath());
     QStringList items;
     if (project) {
         Kit *kit = project->activeTarget()->kit();
-        if (DeviceTypeKitInformation::deviceTypeId(kit) == Constants::ANDROID_DEVICE_TYPE)
-            items = AndroidManager::androidQtSupport(project->activeTarget())->projectTargetApplications(project->activeTarget());
+        if (DeviceTypeKitAspect::deviceTypeId(kit) == Constants::ANDROID_DEVICE_TYPE) {
+            ProjectNode *root = project->rootProjectNode();
+            root->forEachProjectNode([&items](const ProjectNode *projectNode) {
+                items << projectNode->targetApplications();
+            });
+            items.sort();
+        }
     }
 
     // QComboBox randomly resets what the user has entered
@@ -590,13 +605,19 @@ void AndroidManifestEditorWidget::preSave()
 
 void AndroidManifestEditorWidget::postSave()
 {
-    const Utils::FileName docPath = m_textEditorWidget->textDocument()->filePath();
+    const Utils::FilePath docPath = m_textEditorWidget->textDocument()->filePath();
     ProjectExplorer::Project *project = androidProject(docPath);
     if (project) {
         if (Target *target = project->activeTarget()) {
-            AndroidQtSupport *androidQtSupport = AndroidManager::androidQtSupport(target);
-            if (androidQtSupport)
-                androidQtSupport->manifestSaved(target);
+            if (BuildConfiguration *bc = target->activeBuildConfiguration()) {
+                QString androidNdkPlatform = AndroidConfigurations::currentConfig()
+                        .bestNdkPlatformMatch(AndroidManager::minimumSDK(target));
+                if (m_androidNdkPlatform != androidNdkPlatform) {
+                    m_androidNdkPlatform = androidNdkPlatform;
+                    bc->updateCacheAndEmitEnvironmentChanged();
+                    bc->regenerateBuildFiles(nullptr);
+                }
+            }
         }
     }
 }
@@ -741,7 +762,7 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     m_stayClean = true;
     QDomElement manifest = doc.documentElement();
     m_packageNameLineEdit->setText(manifest.attribute(QLatin1String("package")));
-    m_versionCode->setValue(manifest.attribute(QLatin1String("android:versionCode")).toInt());
+    m_versionCodeLineEdit->setText(manifest.attribute(QLatin1String("android:versionCode")));
     m_versionNameLinedit->setText(manifest.attribute(QLatin1String("android:versionName")));
 
     QDomElement usesSdkElement = manifest.firstChildElement(QLatin1String("uses-sdk"));
@@ -898,13 +919,13 @@ void AndroidManifestEditorWidget::parseManifest(QXmlStreamReader &reader, QXmlSt
             << QLatin1String("android:versionName");
     QStringList values = QStringList()
             << m_packageNameLineEdit->text()
-            << QString::number(m_versionCode->value())
+            << m_versionCodeLineEdit->text()
             << m_versionNameLinedit->text();
 
     QXmlStreamAttributes result = modifyXmlStreamAttributes(attributes, keys, values);
     writer.writeAttributes(result);
 
-    QSet<QString> permissions = m_permissionsModel->permissions().toSet();
+    QSet<QString> permissions = Utils::toSet(m_permissionsModel->permissions());
 
     bool foundUsesSdk = false;
     bool foundPermissionComment = false;
@@ -1191,23 +1212,17 @@ void AndroidManifestEditorWidget::parseUnknownElement(QXmlStreamReader &reader,
     }
 }
 
-QString AndroidManifestEditorWidget::iconPath(const QString &baseDir, IconDPI dpi)
+QString AndroidManifestEditorWidget::iconPath(IconDPI dpi)
 {
-    Utils::FileName fileName = Utils::FileName::fromString(baseDir);
     switch (dpi) {
     case HighDPI:
-        fileName.appendPath(QLatin1String("res/drawable-hdpi/icon.png"));
-        break;
+        return QString("/res/drawable-hdpi/icon.png");
     case MediumDPI:
-        fileName.appendPath(QLatin1String("res/drawable-mdpi/icon.png"));
-        break;
+        return QString("/res/drawable-mdpi/icon.png");
     case LowDPI:
-        fileName.appendPath(QLatin1String("res/drawable-ldpi/icon.png"));
-        break;
-    default:
-        return QString();
+        return QString("/res/drawable-ldpi/icon.png");
     }
-    return fileName.toString();
+    return {};
 }
 
 QIcon AndroidManifestEditorWidget::icon(const QString &baseDir, IconDPI dpi)
@@ -1222,7 +1237,7 @@ QIcon AndroidManifestEditorWidget::icon(const QString &baseDir, IconDPI dpi)
     if (dpi == LowDPI && !m_lIconPath.isEmpty())
         return QIcon(m_lIconPath);
 
-    QString fileName = iconPath(baseDir, dpi);
+    QString fileName = baseDir + iconPath(dpi);
     if (fileName.isEmpty())
         return QIcon();
     return QIcon(fileName);
@@ -1233,7 +1248,7 @@ void AndroidManifestEditorWidget::copyIcon(IconDPI dpi, const QString &baseDir,
     if (!QFileInfo::exists(filePath))
         return;
 
-    const QString targetPath = iconPath(baseDir, dpi);
+    const QString targetPath = baseDir + iconPath(dpi);
     QFile::remove(targetPath);
     QDir dir;
     dir.mkpath(QFileInfo(targetPath).absolutePath());
