@@ -24,9 +24,10 @@
 ****************************************************************************/
 #pragma once
 
-#include "utils/fileutils.h"
 #include "androidsdkpackage.h"
 
+#include <utils/fileutils.h>
+
 #include <QObject>
 #include <QFuture>
 
@@ -62,7 +63,7 @@ public:
     };
 
     AndroidSdkManager(const AndroidConfig &config, QObject *parent = nullptr);
-    ~AndroidSdkManager();
+    ~AndroidSdkManager() override;
 
     SdkPlatformList installedSdkPlatforms();
     const AndroidSdkPackageList &allSdkPackages();
@@ -77,6 +78,8 @@ public:
     void reloadPackages(bool forceReload = false);
     bool isBusy() const;
 
+    bool packageListingSuccessful() const;
+
     QFuture<QString> availableArguments() const;
     QFuture<OperationOutput> updateAll();
     QFuture<OperationOutput> update(const QStringList &install, const QStringList &uninstall);
