@@ -63,8 +63,7 @@ void Android::Internal::AndroidSignalOperation::adbFindRunAsFinished(int exitCod
         m_state = Idle;
         emit finished(m_errorMessage);
     } else {
-        connect(m_adbProcess,
-                static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
+        connect(m_adbProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
                 this, &AndroidSignalOperation::adbKillFinished);
         m_state = Kill;
         m_timeout->start();
@@ -113,8 +112,7 @@ void Android::Internal::AndroidSignalOperation::signalOperationViaADB(qint64 pid
     m_adbProcess->disconnect(this);
     m_pid = pid;
     m_signal = signal;
-    connect(m_adbProcess,
-            static_cast<void (QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
+    connect(m_adbProcess, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished),
             this, &AndroidSignalOperation::adbFindRunAsFinished);
     m_state = RunAs;
     m_timeout->start();
@@ -129,7 +127,7 @@ void Android::Internal::AndroidSignalOperation::killProcess(qint64 pid)
 
 void Android::Internal::AndroidSignalOperation::killProcess(const QString &filePath)
 {
-    Q_UNUSED(filePath);
+    Q_UNUSED(filePath)
     m_errorMessage = QLatin1String("The android signal operation does "
                                    "not support killing by filepath.");
     emit finished(m_errorMessage);
@@ -142,7 +140,7 @@ void Android::Internal::AndroidSignalOperation::interruptProcess(qint64 pid)
 
 void Android::Internal::AndroidSignalOperation::interruptProcess(const QString &filePath)
 {
-    Q_UNUSED(filePath);
+    Q_UNUSED(filePath)
     m_errorMessage = QLatin1String("The android signal operation does "
                                    "not support interrupting by filepath.");
     emit finished(m_errorMessage);
