@@ -38,13 +38,7 @@
 namespace Autotest {
 namespace Internal {
 
-const static int outputLimit = 100000;
-
-static bool isSummaryItem(Result::Type type)
-{
-    return type == Result::MessageTestCaseSuccess || type == Result::MessageTestCaseSuccessWarn
-            || type == Result::MessageTestCaseFail || type == Result::MessageTestCaseFailWarn;
-}
+constexpr int outputLimit = 100000;
 
 TestResultDelegate::TestResultDelegate(QObject *parent)
     : QStyledItemDelegate(parent)
@@ -55,13 +49,11 @@ void TestResultDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
 {
     QStyleOptionViewItem opt = option;
     initStyleOption(&opt, index);
-    painter->save();
 
     QFontMetrics fm(opt.font);
     QBrush background;
     QColor foreground;
 
-    const QAbstractItemView *view = qobject_cast<const QAbstractItemView *>(opt.widget);
     const bool selected = opt.state & QStyle::State_Selected;
 
     if (selected) {
@@ -71,10 +63,14 @@ void TestResultDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
         background = opt.palette.window().color();
         foreground = opt.palette.text().color();
     }
+
+    auto resultFilterModel = qobject_cast<const TestResultFilterModel *>(index.model());
+    if (!resultFilterModel)
+        return;
+    painter->save();
     painter->fillRect(opt.rect, background);
     painter->setPen(foreground);
 
-    TestResultFilterModel *resultFilterModel = static_cast<TestResultFilterModel *>(view->model());
     LayoutPositions positions(opt, resultFilterModel);
     const TestResult *testResult = resultFilterModel->testResult(index);
     QTC_ASSERT(testResult, painter->restore();return);
@@ -87,12 +83,14 @@ void TestResultDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
         painter->drawPixmap(positions.left(), positions.top(),
                             icon.pixmap(window, QSize(positions.iconSize(), positions.iconSize())));
 
-    QString typeStr = TestResult::resultToString(testResult->result());
+    TestResultItem *item = resultFilterModel->itemForIndex(index);
+    QTC_ASSERT(item, painter->restore(); return);
+    const QString typeStr = item->resultString();
     if (selected) {
         painter->drawText(positions.typeAreaLeft(), positions.top() + fm.ascent(), typeStr);
     } else {
         QPen tmp = painter->pen();
-        if (isSummaryItem(testResult->result()))
+        if (testResult->result() == ResultType::TestStart)
             painter->setPen(opt.palette.mid().color());
         else
             painter->setPen(TestResult::colorForType(testResult->result()));
@@ -105,8 +103,7 @@ void TestResultDelegate::paint(QPainter *painter, const QStyleOptionViewItem &op
     if (selected) {
         output.replace('\n', QChar::LineSeparator);
 
-        if (AutotestPlugin::instance()->settings()->limitResultOutput
-                && output.length() > outputLimit)
+        if (AutotestPlugin::settings()->limitResultOutput && output.length() > outputLimit)
             output = output.left(outputLimit).append("...");
 
         recalculateTextLayout(index, output, painter->font(), positions.textAreaWidth());
@@ -163,8 +160,7 @@ QSize TestResultDelegate::sizeHint(const QStyleOptionViewItem &option, const QMo
         QString output = testResult->outputString(selected);
         output.replace('\n', QChar::LineSeparator);
 
-        if (AutotestPlugin::instance()->settings()->limitResultOutput
-                && output.length() > outputLimit)
+        if (AutotestPlugin::settings()->limitResultOutput && output.length() > outputLimit)
             output = output.left(outputLimit).append("...");
 
         recalculateTextLayout(index, output, opt.font, positions.textAreaWidth());
