@@ -48,11 +48,13 @@ class IContext;
 }
 
 namespace Autotest {
+
+class TestResult;
+
 namespace Internal {
 
 class TestResultModel;
 class TestResultFilterModel;
-class TestResult;
 class TestEditorMark;
 
 class ResultsTreeView : public Utils::TreeView
@@ -101,7 +103,7 @@ private:
     void onItemActivated(const QModelIndex &index);
     void onRunAllTriggered();
     void onRunSelectedTriggered();
-    void enableAllFilter();
+    void checkAllFilter(bool checked);
     void filterMenuTriggered(QAction *action);
 
     void initializeFilterMenu();
