@@ -32,32 +32,44 @@
 #include <cpptools/cppmodelmanager.h>
 #include <cpptools/cpptoolsreuse.h>
 #include <texteditor/texteditor.h>
+#include <utils/utilsicons.h>
 
 #include <QIcon>
 
 namespace Autotest {
-namespace Internal {
 
 TestTreeItem::TestTreeItem(const QString &name, const QString &filePath, Type type)
     : m_name(name),
       m_filePath(filePath),
       m_type(type)
 {
-    m_checked = (m_type == TestCase || m_type == TestFunctionOrSet || m_type == Root)
-            ? Qt::Checked : Qt::Unchecked;
+    switch (m_type) {
+    case Root:
+    case GroupNode:
+    case TestSuite:
+    case TestCase:
+    case TestFunction:
+        m_checked = Qt::Checked;
+        break;
+    default:
+        m_checked = Qt::Unchecked;
+        break;
+    }
 }
 
 static QIcon testTreeIcon(TestTreeItem::Type type)
 {
     static QIcon icons[] = {
         QIcon(),
-        CPlusPlus::Icons::iconForType(CPlusPlus::Icons::ClassIconType),
-        CPlusPlus::Icons::iconForType(CPlusPlus::Icons::SlotPrivateIconType),
-        QIcon(":/images/data.png")
+        Utils::Icons::OPENFILE.icon(),
+        QIcon(":/autotest/images/suite.png"),
+        Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Class),
+        Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::SlotPrivate),
+        QIcon(":/autotest/images/data.png")
     };
 
     if (int(type) >= int(sizeof icons / sizeof *icons))
-        return icons[2];
+        return icons[3];
     return icons[type];
 }
 
@@ -76,8 +88,10 @@ QVariant TestTreeItem::data(int /*column*/, int role) const
     case Qt::CheckStateRole:
         return QVariant();
     case LinkRole: {
+        if (m_type == GroupNode)
+            return QVariant();
         QVariant itemLink;
-        itemLink.setValue(TextEditor::TextEditorWidget::Link(m_filePath, m_line, m_column));
+        itemLink.setValue(Utils::Link(m_filePath, int(m_line), int(m_column)));
         return itemLink;
     }
     case ItalicRole:
@@ -93,9 +107,9 @@ QVariant TestTreeItem::data(int /*column*/, int role) const
 bool TestTreeItem::setData(int /*column*/, const QVariant &data, int role)
 {
     if (role == Qt::CheckStateRole) {
-        Qt::CheckState old = checked();
-        setChecked((Qt::CheckState)data.toInt());
-        return checked() != old;
+        Qt::CheckState old = m_checked;
+        m_checked = Qt::CheckState(data.toInt());
+        return m_checked != old;
     }
     return false;
 }
@@ -105,20 +119,19 @@ Qt::ItemFlags TestTreeItem::flags(int /*column*/) const
     static const Qt::ItemFlags defaultFlags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;
     switch (m_type) {
     case Root:
+    case GroupNode:
         return Qt::ItemIsEnabled | Qt::ItemIsAutoTristate | Qt::ItemIsUserCheckable;
+    case TestSuite:
     case TestCase:
         return defaultFlags | Qt::ItemIsAutoTristate | Qt::ItemIsUserCheckable;
-    case TestFunctionOrSet:
+    case TestFunction:
         return defaultFlags | Qt::ItemIsUserCheckable;
-    case TestDataFunction:
-    case TestSpecialFunction:
-    case TestDataTag:
     default:
         return defaultFlags;
     }
 }
 
-bool TestTreeItem::modifyTestCaseContent(const TestParseResult *result)
+bool TestTreeItem::modifyTestCaseOrSuiteContent(const TestParseResult *result)
 {
     bool hasBeenModified = modifyName(result->name);
     hasBeenModified |= modifyLineAndColumn(result);
@@ -154,39 +167,14 @@ bool TestTreeItem::modifyLineAndColumn(const TestParseResult *result)
     return hasBeenModified;
 }
 
-void TestTreeItem::setChecked(const Qt::CheckState checkState)
-{
-    switch (m_type) {
-    case TestDataTag: {
-        m_checked = (checkState == Qt::Unchecked ? Qt::Unchecked : Qt::Checked);
-        if (auto parent = parentItem())
-            parent->revalidateCheckState();
-        break;
-    }
-    case Root:
-    case TestFunctionOrSet:
-    case TestCase: {
-        Qt::CheckState usedState = (checkState == Qt::Unchecked ? Qt::Unchecked : Qt::Checked);
-        for (int row = 0, count = childCount(); row < count; ++row)
-            childItem(row)->setChecked(usedState);
-        m_checked = usedState;
-        if (m_type != Root) {
-            if (auto parent = parentItem())
-                parent->revalidateCheckState();
-        }
-        break;
-    }
-    default:
-        return;
-    }
-}
-
 Qt::CheckState TestTreeItem::checked() const
 {
     switch (m_type) {
     case Root:
+    case GroupNode:
+    case TestSuite:
     case TestCase:
-    case TestFunctionOrSet:
+    case TestFunction:
     case TestDataTag:
         return m_checked;
     default:
@@ -203,17 +191,17 @@ void TestTreeItem::markForRemovalRecursively(bool mark)
 {
     markForRemoval(mark);
     for (int row = 0, count = childCount(); row < count; ++row)
-        childItem(row)->markForRemovalRecursively(mark);
+        childAt(row)->markForRemovalRecursively(mark);
 }
 
 void TestTreeItem::markForRemovalRecursively(const QString &filePath)
 {
-    if (m_filePath == filePath)
-        markForRemoval(true);
-    for (int row = 0, count = childCount(); row < count; ++row) {
-        TestTreeItem *child = childItem(row);
+    bool mark = m_filePath == filePath;
+    forFirstLevelChildren([&mark, &filePath](TestTreeItem *child) {
         child->markForRemovalRecursively(filePath);
-    }
+        mark &= child->markedForRemoval();
+    });
+    markForRemoval(mark);
 }
 
 TestTreeItem *TestTreeItem::parentItem() const
@@ -221,32 +209,47 @@ TestTreeItem *TestTreeItem::parentItem() const
     return static_cast<TestTreeItem *>(parent());
 }
 
-TestTreeItem *TestTreeItem::childItem(int row) const
-{
-    return static_cast<TestTreeItem *>(childAt(row));
-}
-
 TestTreeItem *TestTreeItem::findChildByName(const QString &name)
 {
-    return findChildBy([name](const TestTreeItem *other) -> bool {
-        return other->name() == name;
-    });
+    return findFirstLevelChild([name](const TestTreeItem *other) { return other->name() == name; });
 }
 
 TestTreeItem *TestTreeItem::findChildByFile(const QString &filePath)
 {
-    return findChildBy([filePath](const TestTreeItem *other) -> bool {
+    return findFirstLevelChild([filePath](const TestTreeItem *other) {
         return other->filePath() == filePath;
     });
 }
 
+TestTreeItem *TestTreeItem::findChildByFileAndType(const QString &filePath, Type tType)
+{
+    return findFirstLevelChild([filePath, tType](const TestTreeItem *other) {
+        return other->type() == tType && other->filePath() == filePath;
+    });
+}
+
 TestTreeItem *TestTreeItem::findChildByNameAndFile(const QString &name, const QString &filePath)
 {
-    return findChildBy([name, filePath](const TestTreeItem *other) -> bool {
+    return findFirstLevelChild([name, filePath](const TestTreeItem *other) {
         return other->filePath() == filePath && other->name() == name;
     });
 }
 
+TestConfiguration *TestTreeItem::asConfiguration(TestRunMode mode) const
+{
+    switch (mode) {
+    case TestRunMode::Run:
+    case TestRunMode::RunWithoutDeploy:
+        return testConfiguration();
+    case TestRunMode::Debug:
+    case TestRunMode::DebugWithoutDeploy:
+        return debugConfiguration();
+    default:
+        break;
+    }
+    return nullptr;
+}
+
 QList<TestConfiguration *> TestTreeItem::getAllTestConfigurations() const
 {
     return QList<TestConfiguration *>();
@@ -257,6 +260,11 @@ QList<TestConfiguration *> TestTreeItem::getSelectedTestConfigurations() const
     return QList<TestConfiguration *>();
 }
 
+QList<TestConfiguration *> TestTreeItem::getTestConfigurationsForFile(const Utils::FilePath &) const
+{
+    return QList<TestConfiguration *>();
+}
+
 bool TestTreeItem::lessThan(const TestTreeItem *other, SortMode mode) const
 {
     const QString &lhs = data(0, Qt::DisplayRole).toString();
@@ -268,10 +276,11 @@ bool TestTreeItem::lessThan(const TestTreeItem *other, SortMode mode) const
             return index().row() > other->index().row();
         return lhs > rhs;
     case Naturally: {
-        const TextEditor::TextEditorWidget::Link &leftLink =
-                data(0, LinkRole).value<TextEditor::TextEditorWidget::Link>();
-        const TextEditor::TextEditorWidget::Link &rightLink =
-                other->data(0, LinkRole).value<TextEditor::TextEditorWidget::Link>();
+        if (m_type == GroupNode && other->type() == GroupNode)
+            return m_filePath > other->filePath();
+
+        const Utils::Link &leftLink = data(0, LinkRole).value<Utils::Link>();
+        const Utils::Link &rightLink = other->data(0, LinkRole).value<Utils::Link>();
         if (leftLink.targetFileName == rightLink.targetFileName) {
             return leftLink.targetLine == rightLink.targetLine
                     ? leftLink.targetColumn > rightLink.targetColumn
@@ -279,9 +288,23 @@ bool TestTreeItem::lessThan(const TestTreeItem *other, SortMode mode) const
         }
         return leftLink.targetFileName > rightLink.targetFileName;
     }
-    default:
-        return true;
     }
+    return true;
+}
+
+bool TestTreeItem::isGroupNodeFor(const TestTreeItem *other) const
+{
+    QTC_ASSERT(other, return false);
+    if (type() != TestTreeItem::GroupNode)
+        return false;
+
+    // for now there's only the possibility to have 'Folder' nodes
+    return QFileInfo(other->filePath()).absolutePath() == filePath();
+}
+
+bool TestTreeItem::isGroupable() const
+{
+    return true;
 }
 
 QSet<QString> TestTreeItem::internalTargets() const
@@ -292,47 +315,26 @@ QSet<QString> TestTreeItem::internalTargets() const
     if (projectParts.isEmpty())
         return TestTreeItem::dependingInternalTargets(cppMM, m_filePath);
     QSet<QString> targets;
-    for (const CppTools::ProjectPart::Ptr part : projectParts) {
-        targets.insert(part->buildSystemTarget + '|' + part->projectFile);
-        if (part->buildTargetType != CppTools::ProjectPart::Executable)
+    for (const CppTools::ProjectPart::Ptr &part : projectParts) {
+        targets.insert(part->buildSystemTarget);
+        if (part->buildTargetType != ProjectExplorer::BuildTargetType::Executable)
             targets.unite(TestTreeItem::dependingInternalTargets(cppMM, m_filePath));
     }
     return targets;
 }
 
-void TestTreeItem::revalidateCheckState()
+void TestTreeItem::copyBasicDataFrom(const TestTreeItem *other)
 {
-    const Type ttiType = type();
-    if (ttiType != TestCase && ttiType != TestFunctionOrSet && ttiType != Root)
+    if (!other)
         return;
-    if (childCount() == 0) // can this happen? (we're calling revalidateCS() on parentItem()
-        return;
-    bool foundChecked = false;
-    bool foundUnchecked = false;
-    bool foundPartiallyChecked = false;
-    for (int row = 0, count = childCount(); row < count; ++row) {
-        TestTreeItem *child = childItem(row);
-        switch (child->type()) {
-        case TestDataFunction:
-        case TestSpecialFunction:
-            continue;
-        default:
-            break;
-        }
-
-        foundChecked |= (child->checked() == Qt::Checked);
-        foundUnchecked |= (child->checked() == Qt::Unchecked);
-        foundPartiallyChecked |= (child->checked() == Qt::PartiallyChecked);
-        if (foundPartiallyChecked || (foundChecked && foundUnchecked)) {
-            m_checked = Qt::PartiallyChecked;
-            if (ttiType == TestFunctionOrSet || ttiType == TestCase)
-                parentItem()->revalidateCheckState();
-            return;
-        }
-    }
-    m_checked = (foundUnchecked ? Qt::Unchecked : Qt::Checked);
-    if (ttiType == TestFunctionOrSet || ttiType == TestCase)
-        parentItem()->revalidateCheckState();
+    m_name = other->m_name;
+    m_filePath = other->m_filePath;
+    m_type = other->m_type;
+    m_checked = other->m_checked;
+    m_line = other->m_line;
+    m_column = other->m_column;
+    m_proFile = other->m_proFile;
+    m_status = other->m_status;
 }
 
 inline bool TestTreeItem::modifyFilePath(const QString &filePath)
@@ -353,16 +355,6 @@ inline bool TestTreeItem::modifyName(const QString &name)
     return false;
 }
 
-TestTreeItem *TestTreeItem::findChildBy(CompareFunction compare) const
-{
-    for (int row = 0, count = childCount(); row < count; ++row) {
-        TestTreeItem *child = childItem(row);
-        if (compare(child))
-            return child;
-    }
-    return nullptr;
-}
-
 /*
  * try to find build system target that depends on the given file - if the file is no header
  * try to find the corresponding header and use this instead to find the respective target
@@ -377,14 +369,13 @@ QSet<QString> TestTreeItem::dependingInternalTargets(CppTools::CppModelManager *
     bool wasHeader;
     const QString correspondingFile
             = CppTools::correspondingHeaderOrSource(file, &wasHeader, CppTools::CacheUsage::ReadOnly);
-    const Utils::FileNameList dependingFiles = snapshot.filesDependingOn(
+    const Utils::FilePathList dependingFiles = snapshot.filesDependingOn(
                 wasHeader ? file : correspondingFile);
-    for (const Utils::FileName &fn : dependingFiles) {
-        for (const CppTools::ProjectPart::Ptr part : cppMM->projectPart(fn))
-            result.insert(part->buildSystemTarget + '|' + part->projectFile);
+    for (const Utils::FilePath &fn : dependingFiles) {
+        for (const CppTools::ProjectPart::Ptr &part : cppMM->projectPart(fn))
+            result.insert(part->buildSystemTarget);
     }
     return result;
 }
 
-} // namespace Internal
 } // namespace Autotest
