@@ -27,6 +27,7 @@
 #include "autotestplugin.h"
 #include "testcodeparser.h"
 #include "testframeworkmanager.h"
+#include "testprojectsettings.h"
 #include "testsettings.h"
 #include "testtreeitem.h"
 #include "testtreemodel.h"
@@ -39,7 +40,8 @@
 #include <utils/qtcassert.h>
 
 namespace Autotest {
-namespace Internal {
+
+using namespace Internal;
 
 TestTreeModel::TestTreeModel(QObject *parent) :
     TreeModel<>(parent),
@@ -53,6 +55,7 @@ TestTreeModel::TestTreeModel(QObject *parent) :
             this, &TestTreeModel::sweep, Qt::QueuedConnection);
     connect(m_parser, &TestCodeParser::parsingFailed,
             this, &TestTreeModel::sweep, Qt::QueuedConnection);
+
     setupParsingConnections();
 }
 
@@ -73,12 +76,11 @@ TestTreeModel::~TestTreeModel()
 
 void TestTreeModel::setupParsingConnections()
 {
-    if (!m_connectionsInitialized)
-        m_parser->setDirty();
-
-    m_parser->setState(TestCodeParser::Idle);
-    if (m_connectionsInitialized)
+    static bool connectionsInitialized = false;
+    if (connectionsInitialized)
         return;
+    m_parser->setDirty();
+    m_parser->setState(TestCodeParser::Idle);
 
     ProjectExplorer::SessionManager *sm = ProjectExplorer::SessionManager::instance();
     connect(sm, &ProjectExplorer::SessionManager::startupProjectChanged,
@@ -97,7 +99,7 @@ void TestTreeModel::setupParsingConnections()
             m_parser, &TestCodeParser::onQmlDocumentUpdated, Qt::QueuedConnection);
     connect(qmlJsMM, &QmlJS::ModelManagerInterface::aboutToRemoveFiles,
             this, &TestTreeModel::removeFiles, Qt::QueuedConnection);
-    m_connectionsInitialized = true;
+    connectionsInitialized = true;
 }
 
 bool TestTreeModel::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -109,20 +111,18 @@ bool TestTreeModel::setData(const QModelIndex &index, const QVariant &value, int
     if (item && item->setData(index.column(), value, role)) {
         emit dataChanged(index, index);
         if (role == Qt::CheckStateRole) {
-            switch (item->type()) {
-            case TestTreeItem::Root:
-            case TestTreeItem::TestCase:
-                if (item->childCount() > 0)
-                    emit dataChanged(index.child(0, 0), index.child(item->childCount() - 1, 0));
-                break;
-            case TestTreeItem::TestFunctionOrSet:
-                emit dataChanged(index.parent(), index.parent());
-                break;
-            default: // avoid warning regarding unhandled enum member
-                break;
+            Qt::CheckState checked = item->checked();
+            if (item->hasChildren() && checked != Qt::PartiallyChecked) {
+                // handle the new checkstate for children as well...
+                for (Utils::TreeItem *child : *item) {
+                    const QModelIndex &idx = indexForItem(child);
+                    setData(idx, checked ? Qt::Checked : Qt::Unchecked, Qt::CheckStateRole);
+                }
             }
+            if (item->parent() != rootItem() && item->parentItem()->checked() != checked)
+                revalidateCheckState(item->parentItem()); // handle parent too
+            return true;
         }
-        return true;
     }
     return false;
 }
@@ -161,20 +161,127 @@ QList<TestConfiguration *> TestTreeModel::getSelectedTests() const
     return result;
 }
 
-void TestTreeModel::syncTestFrameworks()
+QList<TestConfiguration *> TestTreeModel::getTestsForFile(const Utils::FilePath &fileName) const
 {
-    // remove all currently registered
-    removeTestRootNodes();
+    QList<TestConfiguration *> result;
+    for (Utils::TreeItem *frameworkRoot : *rootItem())
+        result.append(static_cast<TestTreeItem *>(frameworkRoot)->getTestConfigurationsForFile(fileName));
+    return result;
+}
 
-    TestFrameworkManager *frameworkManager = TestFrameworkManager::instance();
-    QVector<Core::Id> sortedIds = frameworkManager->sortedActiveFrameworkIds();
-    for (const Core::Id &id : sortedIds)
-        rootItem()->appendChild(frameworkManager->rootNodeForTestFramework(id));
+QList<TestTreeItem *> TestTreeModel::testItemsByName(TestTreeItem *root, const QString &testName)
+{
+    QList<TestTreeItem *> result;
+
+    root->forFirstLevelChildren([&testName, &result, this](TestTreeItem *node) {
+        if (node->type() == TestTreeItem::TestSuite || node->type() == TestTreeItem::TestCase) {
+            if (node->name() == testName) {
+                result << node;
+                return; // prioritize test suites and cases over test functions
+            }
+            TestTreeItem *testCase = node->findFirstLevelChild([&testName](TestTreeItem *it) {
+                QTC_ASSERT(it, return false);
+                return (it->type() == TestTreeItem::TestCase
+                        || it->type() == TestTreeItem::TestFunction) && it->name() == testName;
+            }); // collect only actual tests, not special functions like init, cleanup etc.
+            if (testCase)
+                result << testCase;
+        } else {
+            result << testItemsByName(node, testName);
+        }
+    });
+    return result;
+}
+
+QList<TestTreeItem *> TestTreeModel::testItemsByName(const QString &testName)
+{
+    QList<TestTreeItem *> result;
+    for (Utils::TreeItem *frameworkRoot : *rootItem())
+        result << testItemsByName(static_cast<TestTreeItem *>(frameworkRoot), testName);
+
+    return result;
+}
+
+void TestTreeModel::synchronizeTestFrameworks()
+{
+    ProjectExplorer::Project *project = ProjectExplorer::SessionManager::startupProject();
+    QList<Core::Id> sortedIds;
+    TestFrameworkManager *manager = TestFrameworkManager::instance();
+    const QVariant useGlobal = project ? project->namedSettings(Constants::SK_USE_GLOBAL)
+                                       : QVariant();
+    if (!useGlobal.isValid() || AutotestPlugin::projectSettings(project)->useGlobalSettings()) {
+        sortedIds = manager->sortedActiveFrameworkIds();
+    } else { // we've got custom project settings
+        const TestProjectSettings *settings = AutotestPlugin::projectSettings(project);
+        const QMap<Core::Id, bool> active = settings->activeFrameworks();
+        sortedIds = Utils::filtered(active.keys(), [active](const Core::Id &id) {
+            return active.value(id);
+        });
+    }
+
+    // pre-check to avoid further processing when frameworks are unchanged
+    Utils::TreeItem *invisibleRoot = rootItem();
+    QSet<Core::Id> newlyAdded;
+    QList<Utils::TreeItem *> oldFrameworkRoots;
+    for (Utils::TreeItem *oldFrameworkRoot : *invisibleRoot)
+        oldFrameworkRoots.append(oldFrameworkRoot);
+
+    for (Utils::TreeItem *oldFrameworkRoot : oldFrameworkRoots)
+        takeItem(oldFrameworkRoot);  // do NOT delete the ptr is still held by TestFrameworkManager
+
+    for (const Core::Id &id : sortedIds) {
+        TestTreeItem *frameworkRootNode = manager->rootNodeForTestFramework(id);
+        invisibleRoot->appendChild(frameworkRootNode);
+        if (!oldFrameworkRoots.removeOne(frameworkRootNode))
+            newlyAdded.insert(id);
+    }
+    for (Utils::TreeItem *oldFrameworkRoot : oldFrameworkRoots)
+        oldFrameworkRoot->removeChildren();
 
     m_parser->syncTestFrameworks(sortedIds);
+    if (!newlyAdded.isEmpty())
+        m_parser->updateTestTree(newlyAdded);
     emit updatedActiveFrameworks(sortedIds.size());
 }
 
+void TestTreeModel::filterAndInsert(TestTreeItem *item, TestTreeItem *root, bool groupingEnabled)
+{
+    TestTreeItem *filtered = item->applyFilters();
+    if (item->shouldBeAddedAfterFiltering())
+        insertItemInParent(item, root, groupingEnabled);
+    else // might be that all children have been filtered out
+        delete item;
+    if (filtered)
+        insertItemInParent(filtered, root, groupingEnabled);
+}
+
+void TestTreeModel::rebuild(const QList<Core::Id> &frameworkIds)
+{
+    TestFrameworkManager *frameworkManager = TestFrameworkManager::instance();
+    for (const Core::Id &id : frameworkIds) {
+        TestTreeItem *frameworkRoot = frameworkManager->rootNodeForTestFramework(id);
+        const bool groupingEnabled = TestFrameworkManager::instance()->groupingEnabled(id);
+        for (int row = frameworkRoot->childCount() - 1; row >= 0; --row) {
+            auto testItem = frameworkRoot->childAt(row);
+            if (testItem->type() == TestTreeItem::GroupNode) {
+                // process children of group node and delete it afterwards if necessary
+                for (int childRow = testItem->childCount() - 1; childRow >= 0; --childRow) {
+                    // FIXME should this be done recursively until we have a non-GroupNode?
+                    TestTreeItem *childTestItem = testItem->childAt(childRow);
+                    takeItem(childTestItem);
+                    filterAndInsert(childTestItem, frameworkRoot, groupingEnabled);
+                }
+                if (!groupingEnabled || testItem->childCount() == 0)
+                    delete takeItem(testItem);
+            } else {
+                takeItem(testItem);
+                filterAndInsert(testItem, frameworkRoot, groupingEnabled);
+            }
+        }
+        revalidateCheckState(frameworkRoot);
+    }
+}
+
 void TestTreeModel::removeFiles(const QStringList &files)
 {
     for (const QString &file : files)
@@ -198,7 +305,7 @@ void TestTreeModel::markForRemoval(const QString &filePath)
     for (Utils::TreeItem *frameworkRoot : *rootItem()) {
         TestTreeItem *root = static_cast<TestTreeItem *>(frameworkRoot);
         for (int childRow = root->childCount() - 1; childRow >= 0; --childRow) {
-            TestTreeItem *child = root->childItem(childRow);
+            TestTreeItem *child = root->childAt(childRow);
             child->markForRemovalRecursively(filePath);
         }
     }
@@ -209,6 +316,7 @@ void TestTreeModel::sweep()
     for (Utils::TreeItem *frameworkRoot : *rootItem()) {
         TestTreeItem *root = static_cast<TestTreeItem *>(frameworkRoot);
         sweepChildren(root);
+        revalidateCheckState(root);
     }
     // even if nothing has changed by the sweeping we might had parse which added or modified items
     emit testTreeModelChanged();
@@ -225,14 +333,18 @@ bool TestTreeModel::sweepChildren(TestTreeItem *item)
 {
     bool hasChanged = false;
     for (int row = item->childCount() - 1; row >= 0; --row) {
-        TestTreeItem *child = item->childItem(row);
+        TestTreeItem *child = item->childAt(row);
 
         if (child->type() != TestTreeItem::Root && child->markedForRemoval()) {
             destroyItem(child);
-            item->revalidateCheckState();
+            revalidateCheckState(item);
             hasChanged = true;
         } else if (child->hasChildren()) {
             hasChanged |= sweepChildren(child);
+            if (!child->hasChildren() && child->removeOnSweepIfEmpty()) {
+                destroyItem(child);
+                revalidateCheckState(item);
+            }
         } else {
             hasChanged |= child->newlyAdded();
         }
@@ -240,6 +352,102 @@ bool TestTreeModel::sweepChildren(TestTreeItem *item)
     return hasChanged;
 }
 
+static TestTreeItem *fullCopyOf(TestTreeItem *other)
+{
+    QTC_ASSERT(other, return nullptr);
+    TestTreeItem *result = other->copyWithoutChildren();
+    for (int row = 0, count = other->childCount(); row < count; ++row)
+        result->appendChild(fullCopyOf(other->childAt(row)));
+    return result;
+}
+
+static void applyParentCheckState(TestTreeItem *parent, TestTreeItem *newItem)
+{
+    QTC_ASSERT(parent && newItem, return);
+
+    if (parent->checked() != newItem->checked()) {
+        const Qt::CheckState checkState = parent->checked() == Qt::Unchecked ? Qt::Unchecked
+                                                                             : Qt::Checked;
+        newItem->setData(0, checkState, Qt::CheckStateRole);
+        newItem->forAllChildren([checkState](Utils::TreeItem *it) {
+            it->setData(0, checkState, Qt::CheckStateRole);
+        });
+    }
+}
+
+void TestTreeModel::insertItemInParent(TestTreeItem *item, TestTreeItem *root, bool groupingEnabled)
+{
+    TestTreeItem *parentNode = root;
+    if (groupingEnabled && item->isGroupable()) {
+        parentNode = root->findFirstLevelChild([item] (const TestTreeItem *it) {
+            return it->isGroupNodeFor(item);
+        });
+        if (!parentNode) {
+            parentNode = item->createParentGroupNode();
+            if (!QTC_GUARD(parentNode)) // we might not get a group node at all
+                parentNode = root;
+            else
+                root->appendChild(parentNode);
+        }
+    }
+    // check if a similar item is already present (can happen for rebuild())
+    if (auto otherItem = parentNode->findChild(item)) {
+        // only handle item's children and add them to the already present one
+        for (int row = 0, count = item->childCount(); row < count; ++row) {
+            TestTreeItem *child = fullCopyOf(item->childAt(row));
+            applyParentCheckState(otherItem, child);
+            otherItem->appendChild(child);
+        }
+        delete item;
+    } else {
+        // we could try to add a non-checked item to a checked group or vice versa
+        applyParentCheckState(parentNode, item);
+        parentNode->appendChild(item);
+    }
+}
+
+void TestTreeModel::revalidateCheckState(TestTreeItem *item)
+{
+    QTC_ASSERT(item, return);
+
+    const TestTreeItem::Type type = item->type();
+    if (type == TestTreeItem::TestSpecialFunction || type == TestTreeItem::TestDataFunction
+            || type == TestTreeItem::TestDataTag) {
+        return;
+    }
+    const Qt::CheckState oldState = Qt::CheckState(item->data(0, Qt::CheckStateRole).toInt());
+    Qt::CheckState newState = Qt::Checked;
+    bool foundChecked = false;
+    bool foundUnchecked = false;
+    bool foundPartiallyChecked = false;
+    for (int row = 0, count = item->childCount(); row < count; ++row) {
+        TestTreeItem *child = item->childAt(row);
+        switch (child->type()) {
+        case TestTreeItem::TestDataFunction:
+        case TestTreeItem::TestSpecialFunction:
+            continue;
+        default:
+            break;
+        }
+
+        foundChecked |= (child->checked() == Qt::Checked);
+        foundUnchecked |= (child->checked() == Qt::Unchecked);
+        foundPartiallyChecked |= (child->checked() == Qt::PartiallyChecked);
+        if (foundPartiallyChecked || (foundChecked && foundUnchecked)) {
+            newState = Qt::PartiallyChecked;
+            break;
+        }
+    }
+    if (newState != Qt::PartiallyChecked)
+        newState = foundUnchecked ? Qt::Unchecked : Qt::Checked;
+    if (oldState != newState) {
+        item->setData(0, newState, Qt::CheckStateRole);
+        emit dataChanged(item->index(), item->index());
+        if (item->parent() != rootItem() && item->parentItem()->checked() != newState)
+            revalidateCheckState(item->parentItem());
+    }
+}
+
 void TestTreeModel::onParseResultReady(const TestParseResultPtr result)
 {
     TestTreeItem *rootNode
@@ -250,10 +458,19 @@ void TestTreeModel::onParseResultReady(const TestParseResultPtr result)
 
 void TestTreeModel::handleParseResult(const TestParseResult *result, TestTreeItem *parentNode)
 {
+    const bool groupingEnabled =
+            TestFrameworkManager::instance()->groupingEnabled(result->frameworkId);
     // lookup existing items
     if (TestTreeItem *toBeModified = parentNode->find(result)) {
         // found existing item... Do not remove
         toBeModified->markForRemoval(false);
+        // if it's a reparse we need to mark the group node as well to avoid purging it in sweep()
+        if (groupingEnabled) {
+            if (auto directParent = toBeModified->parentItem()) {
+                if (directParent->type() == TestTreeItem::GroupNode)
+                    directParent->markForRemoval(false);
+            }
+        }
         // modify and when content has changed inform ui
         if (toBeModified->modify(result)) {
             const QModelIndex &idx = indexForItem(toBeModified);
@@ -267,13 +484,9 @@ void TestTreeModel::handleParseResult(const TestParseResult *result, TestTreeIte
     // if there's no matching item, add the new one
     TestTreeItem *newItem = result->createTestTreeItem();
     QTC_ASSERT(newItem, return);
-    parentNode->appendChild(newItem);
-    // new items are checked by default - revalidation of parents might be necessary
-    if (parentNode->checked() != Qt::Checked) {
-        parentNode->revalidateCheckState();
-        const QModelIndex &idx = indexForItem(parentNode);
-        emit dataChanged(idx, idx);
-    }
+
+    // it might be necessary to "split" created item
+    filterAndInsert(newItem, parentNode, groupingEnabled);
 }
 
 void TestTreeModel::removeAllTestItems()
@@ -282,7 +495,7 @@ void TestTreeModel::removeAllTestItems()
         item->removeChildren();
         TestTreeItem *testTreeItem = static_cast<TestTreeItem *>(item);
         if (testTreeItem->checked() == Qt::PartiallyChecked)
-            testTreeItem->setChecked(Qt::Checked);
+            testTreeItem->setData(0, Qt::Checked, Qt::CheckStateRole);
     }
     emit testTreeModelChanged();
 }
@@ -300,24 +513,30 @@ void TestTreeModel::removeTestRootNodes()
 
 #ifdef WITH_TESTS
 // we're inside tests - so use some internal knowledge to make testing easier
-TestTreeItem *qtRootNode()
+static TestTreeItem *qtRootNode()
 {
     return TestFrameworkManager::instance()->rootNodeForTestFramework(
                 Core::Id(Constants::FRAMEWORK_PREFIX).withSuffix("QtTest"));
 }
 
-TestTreeItem *quickRootNode()
+static TestTreeItem *quickRootNode()
 {
     return TestFrameworkManager::instance()->rootNodeForTestFramework(
                 Core::Id(Constants::FRAMEWORK_PREFIX).withSuffix("QtQuickTest"));
 }
 
-TestTreeItem *gtestRootNode()
+static TestTreeItem *gtestRootNode()
 {
     return TestFrameworkManager::instance()->rootNodeForTestFramework(
                 Core::Id(Constants::FRAMEWORK_PREFIX).withSuffix("GTest"));
 }
 
+static TestTreeItem *boostTestRootNode()
+{
+    return TestFrameworkManager::instance()->rootNodeForTestFramework(
+                Core::Id(Constants::FRAMEWORK_PREFIX).withSuffix("Boost"));
+}
+
 int TestTreeModel::autoTestsCount() const
 {
     TestTreeItem *rootNode = qtRootNode();
@@ -327,7 +546,7 @@ int TestTreeModel::autoTestsCount() const
 bool TestTreeModel::hasUnnamedQuickTests(const TestTreeItem *rootNode) const
 {
     for (int row = 0, count = rootNode->childCount(); row < count; ++row) {
-        if (rootNode->childItem(row)->name().isEmpty())
+        if (rootNode->childAt(row)->name().isEmpty())
             return true;
     }
     return false;
@@ -339,12 +558,7 @@ TestTreeItem *TestTreeModel::unnamedQuickTests() const
     if (!rootNode)
         return nullptr;
 
-    for (int row = 0, count = rootNode->childCount(); row < count; ++row) {
-        TestTreeItem *child = rootNode->childItem(row);
-        if (child->name().isEmpty())
-            return child;
-    }
-    return nullptr;
+    return rootNode->findFirstLevelChild([](TestTreeItem *it) { return it->name().isEmpty(); });
 }
 
 int TestTreeModel::namedQuickTestsCount() const
@@ -369,11 +583,11 @@ int TestTreeModel::dataTagsCount() const
         return 0;
 
     int dataTagCount = 0;
-    for (Utils::TreeItem *item : *rootNode) {
-        TestTreeItem *classItem = static_cast<TestTreeItem *>(item);
-        for (Utils::TreeItem *functionItem : *classItem)
+    rootNode->forFirstLevelChildren([&dataTagCount](TestTreeItem *classItem) {
+        classItem->forFirstLevelChildren([&dataTagCount](TestTreeItem *functionItem) {
             dataTagCount += functionItem->childCount();
-   }
+        });
+    });
     return dataTagCount;
 }
 
@@ -388,20 +602,37 @@ QMultiMap<QString, int> TestTreeModel::gtestNamesAndSets() const
     QMultiMap<QString, int> result;
 
     if (TestTreeItem *rootNode = gtestRootNode()) {
-        for (int row = 0, count = rootNode->childCount(); row < count; ++row) {
-            const TestTreeItem *current = rootNode->childItem(row);
-            result.insert(current->name(), current->childCount());
-        }
+        rootNode->forFirstLevelChildren([&result](TestTreeItem *child) {
+            result.insert(child->name(), child->childCount());
+        });
     }
     return result;
 }
+
+int TestTreeModel::boostTestNamesCount() const
+{
+    TestTreeItem *rootNode = boostTestRootNode();
+    return rootNode ? rootNode->childCount() : 0;
+}
+
+QMultiMap<QString, int> TestTreeModel::boostTestSuitesAndTests() const
+{
+    QMultiMap<QString, int> result;
+
+    if (TestTreeItem *rootNode = boostTestRootNode()) {
+        rootNode->forFirstLevelChildren([&result](TestTreeItem *child) {
+            result.insert(child->name(), child->childCount());
+        });
+    }
+    return result;
+}
+
 #endif
 
 /***************************** Sort/Filter Model **********************************/
 
 TestTreeSortFilterModel::TestTreeSortFilterModel(TestTreeModel *sourceModel, QObject *parent)
-    : QSortFilterProxyModel(parent),
-      m_sourceModel(sourceModel)
+    : QSortFilterProxyModel(parent)
 {
     setSourceModel(sourceModel);
 }
@@ -451,7 +682,7 @@ bool TestTreeSortFilterModel::lessThan(const QModelIndex &left, const QModelInde
 
 bool TestTreeSortFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
-    QModelIndex index = m_sourceModel->index(sourceRow, 0,sourceParent);
+    const QModelIndex index = sourceModel()->index(sourceRow, 0,sourceParent);
     if (!index.isValid())
         return false;
 
@@ -467,5 +698,4 @@ bool TestTreeSortFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex
     }
 }
 
-} // namespace Internal
 } // namespace Autotest
