@@ -1,9 +1,9 @@
-/**************************************************************************
+/****************************************************************************
 **
-** Copyright (C) 2015 Openismus GmbH.
-** Authors: Peter Penz (ppenz@openismus.com)
-**          Patricia Santana Cruz (patriciasantanacruz@gmail.com)
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 Openismus GmbH.
+** Author: Peter Penz (ppenz@openismus.com)
+** Author: Patricia Santana Cruz (patriciasantanacruz@gmail.com)
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -11,32 +11,29 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef MAKEFILEPARSER_H
-#define MAKEFILEPARSER_H
+#pragma once
+
+#include <projectexplorer/projectmacro.h>
 
 #include <QMutex>
 #include <QStringList>
 #include <QTextStream>
 #include <QObject>
+#include <QVector>
 
 QT_FORWARD_DECLARE_CLASS(QDir)
 
@@ -55,6 +52,8 @@ class MakefileParser : public QObject
 {
     Q_OBJECT
 
+    using Macros = ProjectExplorer::Macros;
+
 public:
     /**
      * @param makefile  Filename including path of the autotools
@@ -104,7 +103,7 @@ public:
      * #define X12_HAS_DEPRECATED
      * @endcode
      */
-    QByteArray defines() const;
+     Macros macros() const;
 
     /**
      * @return List of compiler flags for C.
@@ -201,7 +200,7 @@ private:
      *                     contained a variable like $(test). Note that all variables are not
      *                     part of the return value, as they cannot get interpreted currently.
      */
-    QStringList targetValues(bool *hasVariables = 0);
+    QStringList targetValues(bool *hasVariables = nullptr);
 
     /**
      * Adds recursively all sources of the current folder to m_sources and removes
@@ -263,10 +262,9 @@ private:
      */
     bool maybeParseCPPFlag(const QString &term);
 
-private:
-    bool m_success;             ///< Return value for MakefileParser::parse().
+    bool m_success = false;     ///< Return value for MakefileParser::parse().
 
-    bool m_cancel;              ///< True, if the parsing should be cancelled.
+    bool m_cancel = false;      ///< True, if the parsing should be cancelled.
     mutable QMutex m_mutex;     ///< Mutex to protect m_cancel.
 
     QString m_makefile;         ///< Filename of the makefile
@@ -274,7 +272,7 @@ private:
     QStringList m_sources;      ///< Return value for MakefileParser::sources()
     QStringList m_makefiles;    ///< Return value for MakefileParser::makefiles()
     QStringList m_includePaths; ///< Return value for MakefileParser::includePaths()
-    QByteArray m_defines;       ///< Return value for MakefileParser::defines()
+    Macros m_macros;            ///< Return value for MakefileParser::macros()
     QStringList m_cflags;       ///< Return value for MakefileParser::cflags()
     QStringList m_cxxflags;     ///< Return value for MakefileParser::cxxflags()
     QStringList m_cppflags;     ///< The cpp flags, which will be part of both cflags and cxxflags
@@ -285,6 +283,3 @@ private:
 
 } // namespace Internal
 } // namespace AutotoolsProjectManager
-
-#endif // MAKEFILEPARSER_H
-
