@@ -24,9 +24,13 @@
 **
 ****************************************************************************/
 
+#include "baremetalconstants.h"
 #include "baremetaldevice.h"
 #include "baremetaldeviceconfigurationwidget.h"
+#include "baremetaldeviceconfigurationwizard.h"
+
 #include "defaultgdbserverprovider.h"
+
 #include "gdbserverprovidermanager.h"
 #include "gdbserverproviderprocess.h"
 
@@ -44,19 +48,13 @@ namespace Internal {
 
 const char gdbServerProviderIdKeyC[] = "GdbServerProviderId";
 
-BareMetalDevice::Ptr BareMetalDevice::create()
-{
-    return Ptr(new BareMetalDevice);
-}
+// BareMetalDevice
 
-BareMetalDevice::Ptr BareMetalDevice::create(const QString &name, Core::Id type, MachineType machineType, Origin origin, Core::Id id)
+BareMetalDevice::BareMetalDevice()
 {
-    return Ptr(new BareMetalDevice(name, type, machineType, origin, id));
-}
-
-BareMetalDevice::Ptr BareMetalDevice::create(const BareMetalDevice &other)
-{
-    return Ptr(new BareMetalDevice(other));
+    setDisplayType(QCoreApplication::translate("BareMetal::Internal::BareMetalDevice", "Bare Metal"));
+    setDefaultDisplayName(defaultDisplayName());
+    setOsType(Utils::OsTypeOther);
 }
 
 BareMetalDevice::~BareMetalDevice()
@@ -65,6 +63,11 @@ BareMetalDevice::~BareMetalDevice()
         provider->unregisterDevice(this);
 }
 
+QString BareMetalDevice::defaultDisplayName()
+{
+    return QCoreApplication::translate("BareMetal::Internal::BareMetalDevice", "Bare Metal Device");
+}
+
 QString BareMetalDevice::gdbServerProviderId() const
 {
     return m_gdbServerProviderId;
@@ -99,13 +102,13 @@ void BareMetalDevice::providerUpdated(GdbServerProvider *provider)
 void BareMetalDevice::setChannelByServerProvider(GdbServerProvider *provider)
 {
     QTC_ASSERT(provider, return);
-    const QString channel = provider->channel();
+    const QString channel = provider->channelString();
     const int colon = channel.indexOf(QLatin1Char(':'));
     if (colon < 0)
         return;
     QSsh::SshConnectionParameters sshParams = sshParameters();
     sshParams.setHost(channel.left(colon));
-    sshParams.setPort(channel.mid(colon + 1).toUShort());
+    sshParams.setPort(channel.midRef(colon + 1).toUShort());
     setSshParameters(sshParams);
 }
 
@@ -119,10 +122,9 @@ void BareMetalDevice::fromMap(const QVariantMap &map)
             gdbServerProvider = provider->id();
         } else {
             const QSsh::SshConnectionParameters sshParams = sshParameters();
-            DefaultGdbServerProvider *newProvider = new DefaultGdbServerProvider;
+            const auto newProvider = new DefaultGdbServerProvider;
+            newProvider->setChannel(sshParams.url);
             newProvider->setDisplayName(name);
-            newProvider->m_host = sshParams.host();
-            newProvider->m_port = sshParams.port();
             if (GdbServerProviderManager::registerProvider(newProvider))
                 gdbServerProvider = newProvider->id();
             else
@@ -139,63 +141,39 @@ QVariantMap BareMetalDevice::toMap() const
     return map;
 }
 
-BareMetalDevice::IDevice::Ptr BareMetalDevice::clone() const
-{
-    return Ptr(new BareMetalDevice(*this));
-}
-
 DeviceProcessSignalOperation::Ptr BareMetalDevice::signalOperation() const
 {
     return DeviceProcessSignalOperation::Ptr();
 }
 
-QString BareMetalDevice::displayType() const
-{
-    return QCoreApplication::translate("BareMetal::Internal::BareMetalDevice", "Bare Metal");
-}
-
 IDeviceWidget *BareMetalDevice::createWidget()
 {
     return new BareMetalDeviceConfigurationWidget(sharedFromThis());
 }
 
-QList<Core::Id> BareMetalDevice::actionIds() const
-{
-    return QList<Core::Id>(); // no actions
-}
-
-QString BareMetalDevice::displayNameForActionId(Core::Id actionId) const
-{
-    QTC_ASSERT(actionIds().contains(actionId), return QString());
-    return QString();
-}
-
-void BareMetalDevice::executeAction(Core::Id actionId, QWidget *parent)
-{
-    QTC_ASSERT(actionIds().contains(actionId), return);
-    Q_UNUSED(parent);
-}
-
-Utils::OsType BareMetalDevice::osType() const
-{
-    return Utils::OsTypeOther;
-}
-
 DeviceProcess *BareMetalDevice::createProcess(QObject *parent) const
 {
     return new GdbServerProviderProcess(sharedFromThis(), parent);
 }
 
-BareMetalDevice::BareMetalDevice(const QString &name, Core::Id type, MachineType machineType, Origin origin, Core::Id id)
-    : IDevice(type, origin, machineType, id)
+// Factory
+
+BareMetalDeviceFactory::BareMetalDeviceFactory()
+    : IDeviceFactory(Constants::BareMetalOsType)
 {
-    setDisplayName(name);
+    setDisplayName(tr("Bare Metal Device"));
+    setCombinedIcon(":/baremetal/images/baremetaldevicesmall.png",
+                    ":/baremetal/images/baremetaldevice.png");
+    setCanCreate(true);
+    setConstructionFunction(&BareMetalDevice::create);
 }
 
-BareMetalDevice::BareMetalDevice(const BareMetalDevice &other)
-    : IDevice(other)
+IDevice::Ptr BareMetalDeviceFactory::create() const
 {
-    setGdbServerProviderId(other.gdbServerProviderId());
+    BareMetalDeviceConfigurationWizard wizard;
+    if (wizard.exec() != QDialog::Accepted)
+        return {};
+    return wizard.device();
 }
 
 } //namespace Internal
