@@ -23,81 +23,29 @@
 **
 ****************************************************************************/
 
-#include "defaultgdbserverprovider.h"
 #include "baremetalconstants.h"
+#include "defaultgdbserverprovider.h"
 #include "gdbserverprovidermanager.h"
 
 #include <utils/qtcassert.h>
 
 #include <coreplugin/variablechooser.h>
 
+#include <QCheckBox>
 #include <QFormLayout>
 #include <QPlainTextEdit>
 
 namespace BareMetal {
 namespace Internal {
 
-static const char hostKeyC[] = "BareMetal.DefaultGdbServerProvider.Host";
-static const char portKeyC[] = "BareMetal.DefaultGdbServerProvider.Port";
+// DefaultGdbServerProvider
 
 DefaultGdbServerProvider::DefaultGdbServerProvider()
     : GdbServerProvider(QLatin1String(Constants::DEFAULT_PROVIDER_ID))
-    , m_host(QLatin1String("localhost"))
-    , m_port(3333)
 {
-}
-
-DefaultGdbServerProvider::DefaultGdbServerProvider(const DefaultGdbServerProvider &other)
-    : GdbServerProvider(other)
-    , m_host(other.m_host)
-    , m_port(other.m_port)
-{
-}
-
-quint16 DefaultGdbServerProvider::port() const
-{
-    return m_port;
-}
-
-void DefaultGdbServerProvider::setPort(const quint16 &port)
-{
-    m_port = port;
-}
-
-QString DefaultGdbServerProvider::host() const
-{
-    return m_host;
-}
-
-void DefaultGdbServerProvider::setHost(const QString &host)
-{
-    if (m_host == host)
-        return;
-    m_host = host;
-    providerUpdated();
-}
-
-QString DefaultGdbServerProvider::typeDisplayName() const
-{
-    return DefaultGdbServerProviderFactory::tr("Default");
-}
-
-QString DefaultGdbServerProvider::channel() const
-{
-    // Just return as "host:port" form.
-    if (m_port == 0)
-        return m_host;
-    return m_host + QLatin1Char(':') + QString::number(m_port);
-}
-
-bool DefaultGdbServerProvider::isValid() const
-{
-    if (!GdbServerProvider::isValid())
-        return false;
-
-    if (m_host.isEmpty())
-        return false;
-    return true;
+    setDefaultChannel("localhost", 3333);
+    setSettingsKeyBase("BareMetal.DefaultGdbServerProvider");
+    setTypeDisplayName(DefaultGdbServerProviderFactory::tr("Default"));
 }
 
 GdbServerProvider *DefaultGdbServerProvider::clone() const
@@ -105,38 +53,13 @@ GdbServerProvider *DefaultGdbServerProvider::clone() const
     return new DefaultGdbServerProvider(*this);
 }
 
-QVariantMap DefaultGdbServerProvider::toMap() const
-{
-    auto data = GdbServerProvider::toMap();
-    data.insert(QLatin1String(hostKeyC), m_host);
-    data.insert(QLatin1String(portKeyC), m_port);
-    return data;
-}
-
-bool DefaultGdbServerProvider::fromMap(const QVariantMap &data)
-{
-    if (!GdbServerProvider::fromMap(data))
-        return false;
-
-    m_host = data.value(QLatin1String(hostKeyC)).toString();
-    m_port = data.value(QLatin1String(portKeyC)).toInt();
-    return true;
-}
-
-bool DefaultGdbServerProvider::operator==(const GdbServerProvider &other) const
-{
-    if (!GdbServerProvider::operator==(other))
-        return false;
-
-    const auto p = static_cast<const DefaultGdbServerProvider *>(&other);
-    return m_host == p->m_host && m_port == p->m_port;
-}
-
 GdbServerProviderConfigWidget *DefaultGdbServerProvider::configurationWidget()
 {
     return new DefaultGdbServerProviderConfigWidget(this);
 }
 
+// DefaultGdbServerProviderFactory
+
 DefaultGdbServerProviderFactory::DefaultGdbServerProviderFactory()
 {
     setId(QLatin1String(Constants::DEFAULT_PROVIDER_ID));
@@ -157,14 +80,16 @@ bool DefaultGdbServerProviderFactory::canRestore(const QVariantMap &data) const
 
 GdbServerProvider *DefaultGdbServerProviderFactory::restore(const QVariantMap &data)
 {
-    auto p = new DefaultGdbServerProvider;
-    auto updated = data;
+    const auto p = new DefaultGdbServerProvider;
+    const auto updated = data;
     if (p->fromMap(updated))
         return p;
     delete p;
-    return 0;
+    return nullptr;
 }
 
+// GdbServerProviderConfigWidget
+
 DefaultGdbServerProviderConfigWidget::DefaultGdbServerProviderConfigWidget(
         DefaultGdbServerProvider *provider)
     : GdbServerProviderConfigWidget(provider)
@@ -174,6 +99,9 @@ DefaultGdbServerProviderConfigWidget::DefaultGdbServerProviderConfigWidget(
     m_hostWidget = new HostWidget(this);
     m_mainLayout->addRow(tr("Host:"), m_hostWidget);
 
+    m_useExtendedRemoteCheckBox = new QCheckBox(this);
+    m_useExtendedRemoteCheckBox->setToolTip("Use GDB target extended-remote");
+    m_mainLayout->addRow(tr("Extended mode:"), m_useExtendedRemoteCheckBox);
     m_initCommandsTextEdit = new QPlainTextEdit(this);
     m_initCommandsTextEdit->setToolTip(defaultInitCommandsTooltip());
     m_mainLayout->addRow(tr("Init commands:"), m_initCommandsTextEdit);
@@ -184,12 +112,14 @@ DefaultGdbServerProviderConfigWidget::DefaultGdbServerProviderConfigWidget(
     addErrorLabel();
     setFromProvider();
 
-    auto chooser = new Core::VariableChooser(this);
+    const auto chooser = new Core::VariableChooser(this);
     chooser->addSupportedWidget(m_initCommandsTextEdit);
     chooser->addSupportedWidget(m_resetCommandsTextEdit);
 
     connect(m_hostWidget, &HostWidget::dataChanged,
             this, &GdbServerProviderConfigWidget::dirty);
+    connect(m_useExtendedRemoteCheckBox, &QCheckBox::stateChanged,
+            this, &GdbServerProviderConfigWidget::dirty);
     connect(m_initCommandsTextEdit, &QPlainTextEdit::textChanged,
             this, &GdbServerProviderConfigWidget::dirty);
     connect(m_resetCommandsTextEdit, &QPlainTextEdit::textChanged,
@@ -201,8 +131,8 @@ void DefaultGdbServerProviderConfigWidget::applyImpl()
     auto p = static_cast<DefaultGdbServerProvider *>(provider());
     Q_ASSERT(p);
 
-    p->setHost(m_hostWidget->host());
-    p->setPort(m_hostWidget->port());
+    p->setChannel(m_hostWidget->channel());
+    p->setUseExtendedRemote(m_useExtendedRemoteCheckBox->isChecked());
     p->setInitCommands(m_initCommandsTextEdit->toPlainText());
     p->setResetCommands(m_resetCommandsTextEdit->toPlainText());
 }
@@ -217,12 +147,11 @@ void DefaultGdbServerProviderConfigWidget::setFromProvider()
     const auto p = static_cast<DefaultGdbServerProvider *>(provider());
     Q_ASSERT(p);
 
-    const auto b = blockSignals(true);
-    m_hostWidget->setHost(p->m_host);
-    m_hostWidget->setPort(p->m_port);
+    const QSignalBlocker blocker(this);
+    m_hostWidget->setChannel(p->channel());
+    m_useExtendedRemoteCheckBox->setChecked(p->useExtendedRemote());
     m_initCommandsTextEdit->setPlainText(p->initCommands());
     m_resetCommandsTextEdit->setPlainText(p->resetCommands());
-    blockSignals(b);
 }
 
 } // namespace Internal
