@@ -23,23 +23,21 @@
 **
 ****************************************************************************/
 
+#include "baremetaldevice.h"
 #include "gdbserverprovider.h"
 #include "gdbserverprovidermanager.h"
-#include "baremetaldevice.h"
 
-#include <utils/asconst.h>
-#include <utils/qtcassert.h>
 #include <utils/environment.h>
+#include <utils/qtcassert.h>
 
-#include <QCoreApplication>
-#include <QUuid>
-
-#include <QFormLayout>
-#include <QLineEdit>
-#include <QLabel>
 #include <QComboBox>
-#include <QSpinBox>
+#include <QCoreApplication>
+#include <QFormLayout>
+#include <QLabel>
+#include <QLineEdit>
 #include <QPlainTextEdit>
+#include <QSpinBox>
+#include <QUuid>
 
 namespace BareMetal {
 namespace Internal {
@@ -49,6 +47,10 @@ const char displayNameKeyC[] = "BareMetal.GdbServerProvider.DisplayName";
 const char startupModeKeyC[] = "BareMetal.GdbServerProvider.Mode";
 const char initCommandsKeyC[] = "BareMetal.GdbServerProvider.InitCommands";
 const char resetCommandsKeyC[] = "BareMetal.GdbServerProvider.ResetCommands";
+const char useExtendedRemoteKeyC[] = "BareMetal.GdbServerProvider.UseExtendedRemote";
+
+const char hostKeySuffixC[] = ".Host";
+const char portKeySuffixC[] = ".Port";
 
 static QString createId(const QString &id)
 {
@@ -57,9 +59,10 @@ static QString createId(const QString &id)
     return newId;
 }
 
+// GdbServerProvider
+
 GdbServerProvider::GdbServerProvider(const QString &id)
     : m_id(createId(id))
-    , m_startupMode(NoStartup)
 {
 }
 
@@ -68,6 +71,7 @@ GdbServerProvider::GdbServerProvider(const GdbServerProvider &other)
     , m_startupMode(other.m_startupMode)
     , m_initCommands(other.m_initCommands)
     , m_resetCommands(other.m_resetCommands)
+    , m_useExtendedRemote(other.useExtendedRemote())
 {
     m_displayName = QCoreApplication::translate(
                 "BareMetal::GdbServerProvider", "Clone of %1")
@@ -122,6 +126,21 @@ void GdbServerProvider::setInitCommands(const QString &cmds)
     m_initCommands = cmds;
 }
 
+bool GdbServerProvider::useExtendedRemote() const
+{
+    return m_useExtendedRemote;
+}
+
+QString GdbServerProvider::typeDisplayName() const
+{
+    return m_typeDisplayName;
+}
+
+void GdbServerProvider::setUseExtendedRemote(bool useExtendedRemote)
+{
+    m_useExtendedRemote = useExtendedRemote;
+}
+
 QString GdbServerProvider::resetCommands() const
 {
     return m_resetCommands;
@@ -132,14 +151,25 @@ void GdbServerProvider::setResetCommands(const QString &cmds)
     m_resetCommands = cmds;
 }
 
-QString GdbServerProvider::executable() const
+void GdbServerProvider::setChannel(const QUrl &channel)
 {
-    return QString();
+    m_channel = channel;
 }
 
-QStringList GdbServerProvider::arguments() const
+void GdbServerProvider::setDefaultChannel(const QString &host, int port)
 {
-    return QStringList();
+    m_channel.setHost(host);
+    m_channel.setPort(port);
+}
+
+QUrl GdbServerProvider::channel() const
+{
+    return m_channel;
+}
+
+Utils::CommandLine GdbServerProvider::command() const
+{
+    return {};
 }
 
 bool GdbServerProvider::operator==(const GdbServerProvider &other) const
@@ -152,9 +182,19 @@ bool GdbServerProvider::operator==(const GdbServerProvider &other) const
 
     // We ignore displayname
     return thisId == otherId
+            && m_channel == other.m_channel
             && m_startupMode == other.m_startupMode
             && m_initCommands == other.m_initCommands
-            && m_resetCommands == other.m_resetCommands;
+            && m_resetCommands == other.m_resetCommands
+            && m_useExtendedRemote == other.m_useExtendedRemote;
+}
+
+QString GdbServerProvider::channelString() const
+{
+    // Just return as "host:port" form.
+    if (m_channel.port() <= 0)
+        return m_channel.host();
+    return m_channel.host() + ':' + QString::number(m_channel.port());
 }
 
 QVariantMap GdbServerProvider::toMap() const
@@ -164,13 +204,16 @@ QVariantMap GdbServerProvider::toMap() const
         {QLatin1String(displayNameKeyC), m_displayName},
         {QLatin1String(startupModeKeyC), m_startupMode},
         {QLatin1String(initCommandsKeyC), m_initCommands},
-        {QLatin1String(resetCommandsKeyC), m_resetCommands}
+        {QLatin1String(resetCommandsKeyC), m_resetCommands},
+        {QLatin1String(useExtendedRemoteKeyC), m_useExtendedRemote},
+        {m_settingsBase + hostKeySuffixC, m_channel.host()},
+        {m_settingsBase + portKeySuffixC, m_channel.port()},
     };
 }
 
 bool GdbServerProvider::isValid() const
 {
-    return !channel().isNull();
+    return !channelString().isEmpty();
 }
 
 bool GdbServerProvider::canStartupMode(StartupMode m) const
@@ -191,7 +234,7 @@ void GdbServerProvider::unregisterDevice(BareMetalDevice *device)
 void GdbServerProvider::providerUpdated()
 {
     GdbServerProviderManager::notifyAboutUpdate(this);
-    for (BareMetalDevice *device : Utils::asConst(m_devices))
+    for (BareMetalDevice *device : qAsConst(m_devices))
         device->providerUpdated(this);
 }
 
@@ -202,9 +245,25 @@ bool GdbServerProvider::fromMap(const QVariantMap &data)
     m_startupMode = static_cast<StartupMode>(data.value(QLatin1String(startupModeKeyC)).toInt());
     m_initCommands = data.value(QLatin1String(initCommandsKeyC)).toString();
     m_resetCommands = data.value(QLatin1String(resetCommandsKeyC)).toString();
+    m_useExtendedRemote = data.value(QLatin1String(useExtendedRemoteKeyC)).toBool();
+    m_channel.setHost(data.value(m_settingsBase + hostKeySuffixC).toString());
+    m_channel.setPort(data.value(m_settingsBase + portKeySuffixC).toInt());
+
     return true;
 }
 
+void GdbServerProvider::setTypeDisplayName(const QString &typeDisplayName)
+{
+    m_typeDisplayName = typeDisplayName;
+}
+
+void GdbServerProvider::setSettingsKeyBase(const QString &settingsBase)
+{
+    m_settingsBase = settingsBase;
+}
+
+// GdbServerProviderFactory
+
 QString GdbServerProviderFactory::id() const
 {
     return m_id;
@@ -230,11 +289,13 @@ QString GdbServerProviderFactory::idFromMap(const QVariantMap &data)
     return data.value(QLatin1String(idKeyC)).toString();
 }
 
-void GdbServerProviderFactory::idToMap(QVariantMap &data, const QString id)
+void GdbServerProviderFactory::idToMap(QVariantMap &data, const QString &id)
 {
     data.insert(QLatin1String(idKeyC), id);
 }
 
+// GdbServerProviderConfigWidget
+
 GdbServerProviderConfigWidget::GdbServerProviderConfigWidget(
         GdbServerProvider *provider)
     : m_provider(provider)
@@ -259,7 +320,7 @@ GdbServerProviderConfigWidget::GdbServerProviderConfigWidget(
     connect(m_nameLineEdit, &QLineEdit::textChanged,
             this, &GdbServerProviderConfigWidget::dirty);
     connect(m_startupModeComboBox,
-            static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            QOverload<int>::of(&QComboBox::currentIndexChanged),
             this, &GdbServerProviderConfigWidget::dirty);
 }
 
@@ -355,10 +416,9 @@ void GdbServerProviderConfigWidget::clearErrorMessage()
 
 void GdbServerProviderConfigWidget::setFromProvider()
 {
-    const bool b = blockSignals(true);
+    const QSignalBlocker blocker(this);
     m_nameLineEdit->setText(m_provider->displayName());
     setStartupMode(m_provider->startupMode());
-    blockSignals(b);
 }
 
 QString GdbServerProviderConfigWidget::defaultInitCommandsTooltip()
@@ -375,6 +435,8 @@ QString GdbServerProviderConfigWidget::defaultResetCommandsTooltip()
                                        "The MCU should be halted after these commands.");
 }
 
+// HostWidget
+
 HostWidget::HostWidget(QWidget *parent)
     : QWidget(parent)
 {
@@ -385,39 +447,30 @@ HostWidget::HostWidget(QWidget *parent)
     m_portSpinBox->setRange(0, 65535);
     m_portSpinBox->setToolTip(tr("Enter TCP/IP port which will be listened by "
                                  "the GDB server provider."));
-    auto layout = new QHBoxLayout(this);
+    const auto layout = new QHBoxLayout(this);
     layout->setContentsMargins(0, 0, 0, 0);
     layout->addWidget(m_hostLineEdit);
     layout->addWidget(m_portSpinBox);
 
     connect(m_hostLineEdit, &QLineEdit::textChanged,
             this, &HostWidget::dataChanged);
-    connect(m_portSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+    connect(m_portSpinBox, QOverload<int>::of(&QSpinBox::valueChanged),
             this, &HostWidget::dataChanged);
 }
 
-void HostWidget::setHost(const QString &host)
+void HostWidget::setChannel(const QUrl &channel)
 {
-    const bool b = blockSignals(true);
-    m_hostLineEdit->setText(host);
-    blockSignals(b);
+    const QSignalBlocker blocker(this);
+    m_hostLineEdit->setText(channel.host());
+    m_portSpinBox->setValue(channel.port());
 }
 
-QString HostWidget::host() const
+QUrl HostWidget::channel() const
 {
-    return m_hostLineEdit->text();
-}
-
-void HostWidget::setPort(const quint16 &port)
-{
-    const bool b = blockSignals(true);
-    m_portSpinBox->setValue(port);
-    blockSignals(b);
-}
-
-quint16 HostWidget::port() const
-{
-    return m_portSpinBox->value();
+    QUrl url;
+    url.setHost(m_hostLineEdit->text());
+    url.setPort(m_portSpinBox->value());
+    return url;
 }
 
 } // namespace Internal
