@@ -60,7 +60,7 @@ public:
                    settings.boolPointer(BazaarSettings::diffIgnoreBlankLinesKey));
     }
 
-    QStringList arguments() const
+    QStringList arguments() const override
     {
         QStringList args;
         // Bazaar wants "--diff-options=-w -B.."
@@ -178,9 +178,9 @@ VcsBaseEditorWidget *BazaarClient::annotate(
                                    QStringList(extraOptions) << QLatin1String("--long"));
 }
 
-bool BazaarClient::isVcsDirectory(const FileName &fileName) const
+bool BazaarClient::isVcsDirectory(const FilePath &fileName) const
 {
-    return fileName.toFileInfo().isDir()
+    return fileName.isDir()
             && !fileName.fileName().compare(Constants::BAZAARREPO, HostOsInfo::fileNameCaseSensitivity());
 }
 
